package org.processmining.plugins.automapgen;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.plugins.mapvis.importdata.visualization.MapVisData;
import org.xml.sax.SAXException;

/**
 * @author ngupta
 * This is the main plug-in class for getting input values from proM framework
 */
@Plugin(name = "Automatic Map Generator ", 
		returnLabels = { "Map File" }, 
		returnTypes = { MapVisData.class }, 
		parameterLabels = {"Event log","Petri Net", "Petri Net", "Petri Net", "Petri Net","Petri Net","Petri Net","Petri Net","Petri Net","Petri Net","Petri Net"}, 
		userAccessible = true)

 public class AutoMapGen {
	@PluginVariant(requiredParameterLabels = { 0 })
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "N.gupta", email = "n.gupta@tue.nl")
	public static MapVisData mapVisGenPlugin(UIPluginContext context, XLog log) throws IOException {
		History history = History.getInstance();
		wizard = new AutoMapWizard();
		MapVisData constructedMaps = null;
		try {
			constructedMaps = wizard.showWizard(context, history , log, null);
		} catch (SAXException | ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		context.log("Successfully created visualization maps");
		return constructedMaps;
	}

	
	@PluginVariant(requiredParameterLabels = {0, 1})
	@UITopiaVariant(affiliation = UITopiaVariant.EHV,
					author = "N.gupta", 	
					email = "n.gupta@tue.nl")
	public static MapVisData mapVisGenPlugin(UIPluginContext context, XLog log, PetrinetGraph... net ) throws IOException {
		History history = History.getInstance();
		wizard = new AutoMapWizard();	
		MapVisData constructedMaps = null;
		try {
			constructedMaps = wizard.showWizard(context, history, log, net);
		} catch (SAXException | ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		context.log("Successfully created visualization maps");
		return constructedMaps;
	}
	
	private static AutoMapWizard wizard;
}
