package org.processmining.plugins.automapgen;

import java.awt.image.RenderedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.imageio.ImageIO;
import javax.swing.JPanel;
import javax.xml.parsers.ParserConfigurationException;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XLog;
import org.freehep.graphics2d.PixelGraphics2D;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.plugins.automapgen.petrinet.PetriNetWizard;
import org.processmining.plugins.mapvis.importdata.visualization.MapVisData;
import org.processmining.plugins.mapvis.importdata.visualization.MapVisMap;
import org.xml.sax.SAXException;

/**
 * @author ngupta This class shows an interaction window and define and number
 *         of Cartesian and petri-net graph user has to create
 */
class AutoMapWizard {

	private static JPanel steps[];
	private static MapStep initStep;
	private static int noOfSteps;
	int currentStep = 0;
	InteractionResult result;
	int noOfPetriNet = 0;

	AutoMapWizard() {
		noOfSteps = 0;

	}

	// this method control the interaction window
	MapVisData showWizard(UIPluginContext context, History history, XLog log, PetrinetGraph[] net) throws IOException,
			SAXException, ParserConfigurationException {
		int i = 0;
		if (net != null) {
			for (i = 0; i < net.length; i++) {
				noOfPetriNet++;
			}
		}
		initStep = new MapStep(i);

		result = context.showWizard("Map generator selection", true, false, initStep);
		if (result == InteractionResult.CANCEL) {
			context.getFutureResult(0).cancel(true);
			return null;
		}

		String stepTitles[] = titleNumbers(history, log, context, net);
		int lastStep = noOfSteps - 1;
		boolean finished = false;

		while (!finished) {
			result = context.showWizard(stepTitles[currentStep], (currentStep == 0), (currentStep == lastStep),
					steps[currentStep]);
			switch (result) {
				case NEXT :
					currentStep++;
					break;
				case PREV :
					currentStep--;
					break;
				case FINISHED :
					finished = true;
					break;
				case CANCEL :
					context.getFutureResult(0).cancel(true);
					context.log("User cancelled the plug-in");
					return null;
				default :
					System.out.println("WARNING: Default branch reached on wizard loop");
					finished = true;
					break;

			}
		}

		return constructMaps();
	}

	// Identify number of interaction has to be created according to user input
	private String[] titleNumbers(History history, XLog log, UIPluginContext context, PetrinetGraph[] net) {
		noOfSteps = initStep.getNumberOfPlots() + initStep.getNumberOfPetriNetMap();
		steps = new JPanel[noOfSteps];
		String stepTitles[] = new String[noOfSteps];

		int i;
		for (i = 0; i < initStep.getNumberOfPlots(); i++) {
			String title = "Cartesian Graph -" + Integer.toString(i + 1);
			steps[i] = new CartesianWizard(title, history, log);
			stepTitles[i] = title;
		}

		for (int j = 0; j < initStep.getNumberOfPetriNetMap(); j++) {
			String title = "PetriNet Graph-" + Integer.toString(j + 1);
			steps[i + j] = new PetriNetWizard(history, log, net[j], context);
			stepTitles[i + j] = title;
		}
		return stepTitles;
	}

	// create map files according to selected user input 
	MapVisData constructMaps() throws IOException, SAXException, ParserConfigurationException {
		MapVisData mapVisData = new MapVisData();
		Date date = new Date();
		DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd_HHmmss");
		int i;
		for (i = 0; i < initStep.getNumberOfPlots(); i++) {
			CartesianWizard step = (CartesianWizard) steps[i];
			String title = step.getTitle();
			String filename = initStep.getFilePath() + title + "_" + i +"_" + dateFormat.format(date) + ".png";
			
			String filename2 = new java.io.File(filename).getAbsolutePath().replace("\\", "\\\\");

			// get plotted image according to user selection
			RenderedImage plotImage = step.mapSelection();
			//save the file at selected location
			File file = new File(filename);
			ImageIO.write(plotImage, "png", file);
			mapVisData.addMap(title, "file:\\" + filename2, step.getSelectedXAxis(), step.getSelectedYAxis());

			MapVisMap mapVisMap = mapVisData.getMap(title);

			//get x,y,z coordinate query

			Map<String, String> xQueries = step.createQuery();
			//String xQuery = step.createQuery();

			for (String taskName : xQueries.keySet()) {
				String xQuery = xQueries.get(taskName);
				mapVisMap.addQuery(taskName, xQuery);
			}
		}

		
		for (int j = 0; j < initStep.getNumberOfPetriNetMap(); j++) {
			PetriNetWizard step = (PetriNetWizard) steps[j + i];
			String filename = initStep.getFilePath() + "ProcessModel" +"_"+ j +  "_" + dateFormat.format(date) + ".gif";
			String formattedFileName = new java.io.File(filename).getAbsolutePath().replace("\\", "\\\\");
			String myTitle = "Petri-Net " + (j + 1);
			try {
				PixelGraphics2D g = new org.freehep.graphicsio.gif.GIFGraphics2D(new File(filename),
						step.getDimension());
				g.setProperties(new Properties());
				g.startExport();
				step.getGraph().print(g);
				g.endExport();
			} catch (FileNotFoundException ex) {
				ex.printStackTrace();
			}

			mapVisData.addMap(myTitle, "file:\\" + formattedFileName, "", "");
			MapVisMap mapVisMap = mapVisData.getMap(myTitle);

			HashMap<String, String> xQueries = step.getXQueries();

			for (String taskName : xQueries.keySet()) {
				String xQuery = xQueries.get(taskName);
				mapVisMap.addQuery(taskName, xQuery);
			}
		}

		return mapVisData;
	}

}
