package org.processmining.plugins.automapgen;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.regex.Pattern;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.deckfour.xes.model.XLog;
import org.processmining.framework.util.ui.widgets.ProMCheckComboBox;
import org.processmining.framework.util.ui.widgets.ProMComboBox;
import org.processmining.framework.util.ui.widgets.ProMPropertiesPanel;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.framework.util.ui.widgets.WidgetImages;
import org.processmining.plugins.automapgen.cartesian.CartesianFactory;
import org.processmining.plugins.automapgen.cartesian.MapType;

import com.fluxicon.slickerbox.components.IconVerticalTabbedPane;
import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

/**
 * @author ngupta This class creates a Cartesian graph according to the user
 *         inputs
 */
public class CartesianWizard extends IconVerticalTabbedPane implements ActionListener {
	private static final long serialVersionUID = 1L;
	private String defaultTitle;
	private ProMPropertiesPanel properties, previewPanel;
	private ProMTextField graphName, xMinValueTxtFeild, xMaxValueTxtFeild, yMinValueTxtFeild, yMaxValueTxtFeild,
			xDivision, yDivision;
	@SuppressWarnings("rawtypes")
	private ProMComboBox xAxisCombo, yAxisCombo, imageHeightCombo, imageWidthCombo;
	private ProMCheckComboBox eventCBox;
	private JButton refreshButton;
	private JPanel xBoundsPanel, yBoundsPanel, imagePanel;
//	private CartesianPreview previewImagePanel;
	
	private CartesianPlotImage previewImagePanel;
	private XLog log;
	private History history;

	private BufferedImage image;
	private Collection<?> eventNames;

	private int xSize, ySize;
	private SortedSet<String> xAttributeValues = new TreeSet<String>();
	private SortedSet<String> yAttributeValues = new TreeSet<String>(); 
	private PanelObjects panelObj;
                       
	boolean xResult, yResult;
	private Map<String, String> xQuery = new HashMap<String, String>();

	public CartesianWizard(String defaultTitle, History history, XLog log) {
		super(new Color(230, 230, 230, 210), new Color(20, 20, 20, 160));
		this.log = log;
		this.defaultTitle = defaultTitle;
		this.history = history;
		panelObj = new PanelObjects();
		initComponent(log);
	}

	// define JPanel components for Cartesian maps
	private void initComponent(final XLog log) {
		properties = new ProMPropertiesPanel("Select Coordinates");
		this.addTab("Parameters", WidgetImages.dashboardIcon, properties);
		graphName = properties.addTextField("Enter graph name");
		onlyAlphaNumeric(graphName);
		graphName.setText(defaultTitle);
		SortedSet<String> setAttributeNames = axisComboValues();

		// get attributes names for x and y axis
		xAxisCombo = panelObj.createCombo(setAttributeNames);
		yAxisCombo = panelObj.createCombo(setAttributeNames);

		imageHeightCombo = panelObj.createComboImage(getHeightSize());
		imageWidthCombo = panelObj.createComboImage(getWidhtSize());

		imagePanel = new JPanel();
		imagePanel.setLayout(new BoxLayout(imagePanel, BoxLayout.X_AXIS));
		imagePanel.setOpaque(false);
		imagePanel.add(imageHeightCombo);
		imagePanel.add(panelObj.createLabel(" - "));
		imagePanel.add(imageWidthCombo);

		//Get all event list
		SortedSet<String> setEventNames = history.getEventList(log);
		//List events on y combo box
		eventCBox = new ProMCheckComboBox(setEventNames);
		eventCBox.addSelectedItems(setEventNames);

		//Add all above objects on the Properties panel
		properties.addProperty("X-axis attribute:", xAxisCombo);

		xAxisCombo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				xResult = history.checkNumbericAttributeValue(getSelectedXAxis(), log);
				if (!xResult) {
					xMaxMinValueDisable();
					xMinValueTxtFeild.setText("N/A");
					xMaxValueTxtFeild.setText("N/A");
					xDivision.setText("N/A");

				} else {
					xMaxMinValueEnable();
					SortedSet<Integer> attrValue = history.getAttributesValuesInteger(getSelectedXAxis(), log);
					xMinValueTxtFeild.setText(attrValue.first().toString());
					xMaxValueTxtFeild.setText(attrValue.last().toString());
					xDivision.setText("5");
				}
			}
		});

		yAxisCombo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				yResult = history.checkNumbericAttributeValue(getSelectedYAxis(), log);
				if (!yResult) {
					yMaxMinValueDisable();
					yMinValueTxtFeild.setText("N/A");
					yMaxValueTxtFeild.setText("N/A");
					yDivision.setText("N/A");
				} else {
					yMaxMinValueEnable();
					SortedSet<Integer> attrValue = history.getAttributesValuesInteger(getSelectedYAxis(), log);
					yMinValueTxtFeild.setText(attrValue.first().toString());
					yMaxValueTxtFeild.setText(attrValue.last().toString());
					yDivision.setText("5");
				}
			}
		});

		xResult = history.checkNumbericAttributeValue(getSelectedXAxis(), log);
		xAddValuePanel(xResult);

		properties.addProperty("Y-axis attribute:", yAxisCombo);

		yResult = history.checkNumbericAttributeValue(getSelectedYAxis(), log);
		yAddValuePanel(yResult);

		properties.addProperty("Event list:", eventCBox);
		properties.addProperty("Height - Width", imagePanel);

		//Create preview panel on the vertical side
		previewPanel = new ProMPropertiesPanel("Preview Map");
		this.addTab("Preview", WidgetImages.inspectorIcon, previewPanel);
		previewImagePanel = new CartesianPlotImage(getSelectedHeight(), getSelectedWidht());
		JScrollPane scrollPane = new JScrollPane(previewImagePanel);
		scrollPane.getHorizontalScrollBar().setUI(new SlickerScrollBarUI(scrollPane.getHorizontalScrollBar()));
		scrollPane.getVerticalScrollBar().setUI(new SlickerScrollBarUI(scrollPane.getVerticalScrollBar()));
		scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setBorder(BorderFactory.createEmptyBorder());
		scrollPane.setBackground(Color.BLACK);
		previewPanel.add(scrollPane);

		refreshButton = panelObj.createButton("Refresh");
		refreshButton.addActionListener(this);
		previewPanel.add(refreshButton);

	}

	private void onlyNumeric(ProMTextField text) {
		text.addKeyListener(new KeyAdapter() {
			public void keyTyped(KeyEvent e) {
				char c = e.getKeyChar();
				if (!((c >= '0') && (c <= '9') || c != '\b' || (c == KeyEvent.VK_DECIMAL)
						|| (c == KeyEvent.VK_BACK_SPACE) || (c == KeyEvent.VK_DELETE) || (c == KeyEvent.VK_ENTER))) {
					getToolkit().beep();
					e.consume();
				}
			}
		});
	}

	private void onlyAlphaNumeric(ProMTextField text) {
		text.addKeyListener(new KeyAdapter() {
			public void keyTyped(KeyEvent e) {
				char c = e.getKeyChar();
				if (!((c >= '0') && (c <= '9') || (c >= 'a') && (c <= 'z') || (c >= 'A') && (c <= 'Z') || (c == '-')
						|| (c == KeyEvent.VK_BACK_SPACE) || (c == KeyEvent.VK_SPACE) || (c == KeyEvent.VK_DELETE) || (c == KeyEvent.VK_ENTER))) {
					getToolkit().beep();
					e.consume();
				}
			}
		});
	}

	public void actionPerformed(ActionEvent e) {
		if ("Refresh".equals(e.getActionCommand()))
			mapSelection();
	}

	public RenderedImage mapSelection() {
		getMapCreationValues();
		MapType type = null;
		if (!xResult && !yResult) {
			type = MapType.LITERAL;
		} else if (xResult && yResult) {
			type = MapType.CONTINOUS;
		} else if (!xResult && yResult) {
			type = MapType.XLITERAL;
		} else if (xResult && !yResult) {
			type = MapType.YLITERAL;
		}

		CartesianFactory cartFactory = new CartesianFactory(type, getSelectedHeight(), getSelectedWidht(), getTitle(),
				getSelectedXAxis(), getSelectedYAxis(), xSize, ySize, getXMinBoundaryValue(), getXMaxBoundaryValue(),
				getXDivision(), getYMinBoundaryValue(), getYMaxBoundaryValue(), getYDivision(), xAttributeValues,
				yAttributeValues, eventNames);
		image = cartFactory.buildMapImage(type);
		previewImagePanel.setImage(image);
		
		xQuery = cartFactory.createXQuery(type);
		return image;
	}

	public Map<String, String> createQuery() {
		return this.xQuery;
	}

	public RenderedImage getCartesianImage() {
		RenderedImage image = mapSelection();
		return image;
	}

	private ArrayList<Integer> getWidhtSize() {
		ArrayList<Integer> imageSize = new ArrayList<Integer>();
		imageSize.add(1800);
		imageSize.add(400);
		imageSize.add(800);
		imageSize.add(1600);
		imageSize.add(2000);
		return imageSize;

	}

	private ArrayList<Integer> getHeightSize() {
		ArrayList<Integer> imageSize = new ArrayList<Integer>();
		imageSize.add(1200);
		imageSize.add(400);
		imageSize.add(800);
		imageSize.add(1600);
		imageSize.add(2000);
		return imageSize;

	}

	private void xMaxMinValueDisable() {
		xMinValueTxtFeild.setEditable(false);
		xMaxValueTxtFeild.setEditable(false);
		xDivision.setEditable(false);
	}

	private void yMaxMinValueDisable() {
		yMinValueTxtFeild.setEditable(false);
		yMaxValueTxtFeild.setEditable(false);
		yDivision.setEditable(false);
	}

	private void xMaxMinValueEnable() {
		xMinValueTxtFeild.setEditable(true);
		xMaxValueTxtFeild.setEditable(true);
		xDivision.setEditable(true);
	}

	private void yMaxMinValueEnable() {
		yMinValueTxtFeild.setEditable(true);
		yMaxValueTxtFeild.setEditable(true);
		yDivision.setEditable(true);
	}

	private void xAddValuePanel(Boolean xValue) {
		if (xValue) {
			//x  axis division options
			xDivision = new ProMTextField("5");

			//x axis attribute
			SortedSet<Integer> attrValue = history.getAttributesValuesInteger(getSelectedXAxis(), log);
			xMinValueTxtFeild = new ProMTextField(attrValue.first().toString());
			xMaxValueTxtFeild = new ProMTextField(attrValue.last().toString());
			xBoundsPanel = panelObj.createAxisPanel(xMinValueTxtFeild, xMaxValueTxtFeild);
		} else {
			xDivision = new ProMTextField("N/A");
			xMinValueTxtFeild = new ProMTextField("N/A");
			xMaxValueTxtFeild = new ProMTextField("N/A");
			xBoundsPanel = panelObj.createAxisPanel(xMinValueTxtFeild, xMaxValueTxtFeild);
			xMaxMinValueDisable();
		}

		onlyNumeric(xDivision);
		onlyNumeric(xMinValueTxtFeild);
		onlyNumeric(xMaxValueTxtFeild);
		properties.addProperty("X axis - Boundaries", xBoundsPanel);
		properties.addProperty("No. of divisions:", xDivision);
	}

	private void yAddValuePanel(Boolean yValue) {
		if (yValue) {
			//y  axis division options
			yDivision = new ProMTextField("5");
			//y axis attribute
			SortedSet<Integer> attrValue = history.getAttributesValuesInteger(getSelectedYAxis(), log);
			yMinValueTxtFeild = new ProMTextField(attrValue.first().toString());
			yMaxValueTxtFeild = new ProMTextField(attrValue.last().toString());
			yBoundsPanel = panelObj.createAxisPanel(yMinValueTxtFeild, yMaxValueTxtFeild);
		} else {
			yDivision = new ProMTextField("N/A");
			yMinValueTxtFeild = new ProMTextField("N/A");
			yMaxValueTxtFeild = new ProMTextField("N/A");
			yBoundsPanel = panelObj.createAxisPanel(yMinValueTxtFeild, yMaxValueTxtFeild);
			yMaxMinValueDisable();
		}

		onlyNumeric(yDivision);
		onlyNumeric(yMinValueTxtFeild);
		onlyNumeric(yMaxValueTxtFeild);
		properties.addProperty("Y axis - Boundaries", yBoundsPanel);
		properties.addProperty("No. of divisions:", yDivision);
	}

	// Get x and y axis attributes names from the data set
	private SortedSet<String> axisComboValues() {
		SortedSet<String> selectedAttribs = new TreeSet<String>();
		//Get all the attributes list from the data set
		SortedSet<String> setAttributeNames = history.getAttributesSet(log);
		for (String attributeName : setAttributeNames) {
			if (!attributeName.contains(":"))
				selectedAttribs.add(attributeName);
		}
		return selectedAttribs;
	}
	
	private void getMapCreationValues() {
		//Get X-axis 
		xAttributeValues = history.getAttributesValues(getSelectedXAxis(), log);
		xSize = xAttributeValues.size();

		//Get Y-axis values
		yAttributeValues = history.getAttributesValues(getSelectedYAxis(), log);
		ySize = yAttributeValues.size();

		//Get event names
		setEventNames(eventNames());
	}

	//Get entered graph name
	public String getTitle() {
		return graphName.getText();
	}

	//get all event name which need to plotted on the Cartesian map
	@SuppressWarnings("unchecked")
	public Collection<String> eventNames() {
		return eventCBox.getSelectedItems();
	}

	//get all dimension for X-axis
	public String getSelectedXAxis() {
		return xAxisCombo.getSelectedItem().toString();
	}

	//get all dimension for Y-axis
	public String getSelectedYAxis() {
		return yAxisCombo.getSelectedItem().toString();
	}

	//get Height of the image
	public Integer getSelectedHeight() {
		int height;
		if (imageHeightCombo.getSelectedItem().toString().isEmpty()) {
			height = 1200;
		} else {
			height = (Integer) imageHeightCombo.getSelectedItem();
		}
		return height;
	}

	//get width of the image
	public Integer getSelectedWidht() {
		int width;
		if (imageWidthCombo.getSelectedItem().toString().isEmpty()) {
			width = 1800;
		} else {
			width = (Integer) imageWidthCombo.getSelectedItem();
		}
		return width;
	}

	private float getValue(String val) {
		float value = 0f;
		if (val != null) {
			String decimalPattern = "([0-9]*)\\.([0-9]*)";
			boolean match = Pattern.matches(decimalPattern, val);
			if (match) {
				value = Float.valueOf(val);
			} else {
				value = Integer.parseInt(val);
			}
		}
		return value;
	}

	//get x axis minimum value
	public float getXMinBoundaryValue() {
		float min = 0f;
		if (xMinValueTxtFeild.getText().isEmpty()) {
			SortedSet<Integer> attrValue = history.getAttributesValuesInteger(getSelectedXAxis(), log);
			min = getValue(attrValue.first().toString());
		} else {
			if (!xMinValueTxtFeild.getText().equals("N/A")) {
				min = getValue(xMinValueTxtFeild.getText());
			}
		}
		if (!xMinValueTxtFeild.getText().equals("N/A")) {
			if (getValue(xMinValueTxtFeild.getText()) > getValue(xMinValueTxtFeild.getText())) {
				SortedSet<Integer> attrValue = history.getAttributesValuesInteger(getSelectedXAxis(), log);
				min = getValue(attrValue.first().toString());
				xMinValueTxtFeild.setText(Float.toString(min));

			}
		}
		return min;
	}

	//get y axis minimum value
	public float getYMinBoundaryValue() {
		float min = 0f;
		if (yMinValueTxtFeild.getText().isEmpty()) {
			SortedSet<Integer> attrValue = history.getAttributesValuesInteger(getSelectedYAxis(), log);
			min = getValue(attrValue.first().toString());
		} else {
			if (!yMinValueTxtFeild.getText().equals("N/A")) {
				min = getValue(yMinValueTxtFeild.getText());
			}
		}
		if (!yMinValueTxtFeild.getText().equals("N/A")) {
			if (getValue(yMinValueTxtFeild.getText()) > getValue(yMinValueTxtFeild.getText())) {
				SortedSet<Integer> attrValue = history.getAttributesValuesInteger(getSelectedXAxis(), log);
				min = getValue(attrValue.first().toString());
				yMinValueTxtFeild.setText(Float.toString(min));
			}
		}
		return min;
	}

	//get x axis maximum value
	public float getXMaxBoundaryValue() {
		float max = 0f;
		if (xMaxValueTxtFeild.getText().isEmpty()) {
			SortedSet<Integer> attrValue = history.getAttributesValuesInteger(getSelectedXAxis(), log);
			max = getValue(attrValue.last().toString());
		} else {
			if (!xMaxValueTxtFeild.getText().equals("N/A")) {
				max = getValue(xMaxValueTxtFeild.getText());
			}
		}
		return max;
	}

	//get y axis maximum value
	public float getYMaxBoundaryValue() {
		float max = 0f;
		if (yMaxValueTxtFeild.getText().isEmpty()) {
			SortedSet<Integer> attrValue = history.getAttributesValuesInteger(getSelectedYAxis(), log);
			max = getValue(attrValue.last().toString());
		} else {
			if (!yMaxValueTxtFeild.getText().equals("N/A")) {
				max = getValue(yMaxValueTxtFeild.getText());
			}
		}
		return max;
	}

	//get x axis division 
	public int getXDivision() {
		int xdiv = 0;
		float min = getXMinBoundaryValue();
		float max = getXMaxBoundaryValue();
		if (xDivision.getText().isEmpty()) {
			xdiv = (int) (max / min);
			if (xdiv > 12) {
				xdiv = 12;
			}
		} else {
			if (!xDivision.getText().equals("N/A")) {
				xdiv = Integer.parseInt(xDivision.getText());
			}
		}

		return xdiv;
	}

	//get y axis division 
	public int getYDivision() {
		int ydiv = 0;
		float min = getYMinBoundaryValue();
		float max = getYMaxBoundaryValue();
		if (yDivision.getText().isEmpty()) {

			ydiv = (int) (max / min);
			if (ydiv > 12) {
				ydiv = 12;
			}
		} else {
			if (!yDivision.getText().equals("N/A")) {
				ydiv = Integer.parseInt(yDivision.getText());
			}
		}
		return ydiv;
	}

	public void divMinMax(float min, float max, int div) {
		int i = (int) (max / min);
		if (i < div) {
			int input = JOptionPane.showOptionDialog(null, "Do you want to have float values", "Warning",
					JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE, null, null, null);
			if (input == JOptionPane.OK_OPTION) {

			}
		}
	}

	public Collection<?> getEventNames() {
		return eventNames;
	}

	public void setEventNames(Collection<?> eventNames) {
		this.eventNames = eventNames;
	}
}
