package org.processmining.plugins.automapgen;

import java.util.Iterator;
import java.util.SortedSet;
import java.util.TreeSet;

import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;

public class History {
	private static History history;
	private Boolean result = false;
	private transient TreeSet<String> activityNames = new TreeSet<String>();

	public void cleanup() {
	}

	public SortedSet<String> getActivityNames() {
		return activityNames;
	}

	public void addActvityNames(XLog log) {
		for (XTrace xTrace : log) {
			// iterating though the events in a trace
			for (XEvent xEvent : xTrace) {
				XAttributeMap attributes = xEvent.getAttributes();
				String activity = attributes.get("concept:name").toString().replace(":", "_");
				activityNames.add(activity);
			}
		}
	}
	
	//Create a sorted list to get all attributes names from the event log file
	protected SortedSet<String> getAttributesSet(XLog log) {
		TreeSet<String> attributeNames = new TreeSet<String>();
		for (XTrace trace : log) {
			attributeNames.addAll(trace.getAttributes().keySet());
			for (XEvent event : trace)
				attributeNames.addAll(event.getAttributes().keySet());
		}
		return attributeNames;
	}

	//Create a sorted list to get all Event names from the event log file
	protected SortedSet<String> getEventList(XLog log) {
		TreeSet<String> eventNames = new TreeSet<String>();
		for (XTrace trace : log) {
			for (XEvent event : trace) {
				String Event = event.getAttributes().get("concept:name").toString();
				eventNames.add(Event);
			}
		}
		return eventNames;
	}

	
		
	// get all attributes values for the selected axis coordinates
	protected SortedSet<String> getAttributesValues(String attributeName, XLog log) {
		SortedSet<String> attrValues = new TreeSet<String>();
		for (XTrace trace : log) {
			XAttributeMap attributes = (XAttributeMap) trace.getAttributes().clone();
			// Iteration over the event log traces
			Iterator<XEvent> iter = trace.iterator();
			while (iter.hasNext()) {
				XEvent xEvent = iter.next();
				attributes.putAll(xEvent.getAttributes());
				//get all values of attributes
				XAttribute attrValueObject = attributes.get(attributeName);
				if (attrValueObject != null) {
					//get attribute value type 
					String attrValue = getValue(attrValueObject).toString();
					//put all the values and keys into a hash map table 
					if (!attrValues.contains(attrValue)) {
						attrValues.add(attrValue);
					}
				}
			}
		}
		return attrValues;
	}

	// get all attributes values for the selected axis coordinates
	protected SortedSet<Integer> getAttributesValuesInteger(String attributeName, XLog log) {
		SortedSet<Integer> attrValues = new TreeSet<Integer>();
		for (XTrace trace : log) {
			XAttributeMap attributes = (XAttributeMap) trace.getAttributes().clone();
			// Iteration over the event log traces
			Iterator<XEvent> iter = trace.iterator();
			while (iter.hasNext()) {
				XEvent xEvent = iter.next();
				attributes.putAll(xEvent.getAttributes());
				//get all values of attributes
				XAttribute attrValueObject = attributes.get(attributeName);

				if (attrValueObject != null) {
					int attrValue;
					Object obj = (getValue(attrValueObject));
					String objType = obj.getClass().getName();
					if (objType == "java.lang.Double" || objType == "java.lang.Float") {
						Double objFloat = new Double((double) obj);
						attrValue = objFloat.intValue();
					} else {
						attrValue = Integer.parseInt(getValue(attrValueObject).toString());
					}
					//put all the values and keys into a hash map table 
					if (!attrValues.contains(attrValue)) {
						attrValues.add(attrValue);
					}
				}
			}
		}
		return attrValues;
	}

	//Check whether attribute has continuous value or not
	public boolean checkNumbericAttributeValue(String attributeName, XLog log) {
		//	SortedSet<String> numericValue= new TreeSet<String>();
		for (XTrace trace : log) {
			XAttributeMap attributes = (XAttributeMap) trace.getAttributes().clone();
			Iterator<XEvent> iter = trace.iterator();
			while (iter.hasNext()) {
				XEvent xEvent = iter.next();
				attributes.putAll(xEvent.getAttributes());
				//get all values of attributes
				XAttribute attrValueObject = attributes.get(attributeName);
				if (attrValueObject != null) {
					if (attrValueObject instanceof XAttributeContinuous
							|| attrValueObject instanceof XAttributeDiscrete) {
						result = true;

					} else {
						result = false;

					}
				}
			}
		}
		return result;
	}

	// find the value type of the attribute 
	public static Object getValue(XAttribute attribute) {
		if (attribute instanceof XAttributeBoolean)
			return ((XAttributeBoolean) attribute).getValue();
		else if (attribute instanceof XAttributeContinuous)
			return ((XAttributeContinuous) attribute).getValue();
		else if (attribute instanceof XAttributeDiscrete)
			return ((XAttributeDiscrete) attribute).getValue();
		else if (attribute instanceof XAttributeLiteral)
			return ((XAttributeLiteral) attribute).getValue();
		else if (attribute instanceof XAttributeTimestamp)
			return ((XAttributeTimestamp) attribute).getValue().getTime();
		else
			return null;
	}

	public void free() {
		history = null;
	}

	public static History getInstance() {
		if (history == null)
			history = new History();
		return history;
	}

	public static void setInstance(History history) {
		History.history = history;
	}
}
