package org.processmining.plugins.automapgen;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;

import org.processmining.framework.util.ui.widgets.ProMPropertiesPanel;

import com.fluxicon.slickerbox.components.NiceIntegerSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;

public class MapStep extends ProMPropertiesPanel implements ActionListener {
	private static final long serialVersionUID = 1L;
	private static final String title = "Select maps";
	private NiceIntegerSlider nrCartesian, nrPetriNet;
	private JButton chooserButton;
	private JFileChooser chooser;
	private String path = System.getProperty("user.home") + "\\" + "Documents";

	int petriNetInputNo;
	private JLabel dirPath;

	public MapStep(int petriNetInputNo) {
		super(title);
		this.petriNetInputNo = petriNetInputNo;
		initComponents(petriNetInputNo);
	}

	private void initComponents(int petriNetInputNo) {
		SlickerFactory slickerFactory = SlickerFactory.instance();
		nrCartesian = this.addProperty("Cartesian Map ",
				slickerFactory.createNiceIntegerSlider("Number of plots", 0, 15, 1, Orientation.HORIZONTAL));
		if (petriNetInputNo != 0) {
			nrPetriNet = this.addProperty("Petri-Net map ", slickerFactory.createNiceIntegerSlider("Number of plots",
					0, petriNetInputNo, 1, Orientation.HORIZONTAL));
		}

		chooserButton = slickerFactory.createButton("Choose directory");
		dirPath = slickerFactory.createLabel(path);
		dirPath.setFont(new Font("Calibri", Font.BOLD, 20));
		chooserButton.addActionListener(this);

		this.addProperty("Image output directory", chooserButton);
		this.addProperty("Default path: ", dirPath);

	}

	public int getNumberOfPlots() {
		return nrCartesian.getValue();
	}

	public int getNumberOfPetriNetMap() {
		if (petriNetInputNo != 0) {
			return nrPetriNet.getValue();
		} else
			return 0;
	}

	public String getFilePath() {
		return path + "/";
	}

	public void actionPerformed(ActionEvent e) {
		chooser = new JFileChooser();
		chooser.setDialogTitle("Select output directory for images");
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		File workingDirectory = new File(path);
		chooser.setCurrentDirectory(workingDirectory);
		if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
			path = chooser.getSelectedFile().getPath();
			dirPath.setText(path);
		}
	}

}
