package org.processmining.plugins.automapgen.cartesian;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.io.File;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Collection;
import java.util.Map;
import java.util.SortedSet;

import javax.imageio.ImageIO;

/**
 * @author ngupta
 *
 */
public class Cartesian {
	protected String title, xTitle, yTitle, xAttributesName, yAttributesName;
	protected BufferedImage bufferedImage;
	protected Graphics2D g2d;
	final float ratio = 1.0f;
	protected SortedSet<String> xKeys, yKeys;
	protected int xSize, ySize, xDivision, yDivision, rows, cols, imgHeight, imgWidth, yGridSize, xGridSize;
	protected float xMin, xMax, yMin, yMax;
	protected Collection<?> eventName;
	private static float BASIC_STROKE = 1f;
	private RenderedImage image;
	private Map<String, String> query;
	protected NumberFormat  df = new DecimalFormat("#.##");
	
	public Cartesian(MapType type, int imgHeight, int imgWidth, String title, String xTitle, String yTitle, int xSize,
			int ySize, float xMin, float xMax, int xDivision, float yMin, float yMax, int yDivision,
			SortedSet<String> xKeys, SortedSet<String> yKeys, Collection<?> eventNames) {
		this.imgHeight = imgHeight;
		this.imgWidth = imgWidth;
		this.title = title;
		this.xTitle = xTitle;
		this.yTitle = yTitle;
		this.xSize = xSize;
		this.ySize = ySize;
		this.xMin = xMin;
		this.xMax = xMax;
		this.yMin = yMin;
		this.yMax = yMax;
		this.yDivision = yDivision;
		this.xDivision = xDivision;
		this.xKeys = xKeys;
		this.yKeys = yKeys;
		this.eventName = eventNames;
	}

	protected RenderedImage drawBasicCartesianImage() {
		bufferedImage = new BufferedImage(imgWidth, imgHeight, BufferedImage.TYPE_INT_RGB);
		g2d = bufferedImage.createGraphics();
		FontMetrics fm = g2d.getFontMetrics();
		// Draw graphics
		g2d.setColor(Color.WHITE);
		g2d.fillRect(0, 0, imgWidth, imgHeight);

		//graph title name
		g2d.setFont(new Font("Calibri", Font.PLAIN, getTitleFontSize(yGridSize)));
		g2d.setColor(new Color(32, 63, 134));

		int width = fm.stringWidth(title);
		g2d.drawString(title, (imgWidth / 2) - width, imgHeight / 20);

		getAxisValueSize();

		width = fm.stringWidth(xTitle);
		//X axis title name
		g2d.drawString(xTitle, (imgWidth / 2) - (width / 2), imgHeight - imgHeight / 35);

		// Rotate Y title in vertical direction
		AffineTransform fontAT = new AffineTransform();
		fontAT.rotate(-1.57);
		Font rotatedFont = g2d.getFont().deriveFont(fontAT);
		g2d.setFont(rotatedFont);

		width = fm.stringWidth(yTitle);
		//y axis title name
		g2d.drawString(yTitle, imgWidth / 45, (imgHeight / 2) - (width / 2));

		// draw lines
		g2d.setStroke(new BasicStroke(getBasicStroke()));

		// boundaries of the graph
		g2d.drawLine(xGridSize, yGridSize, xGridSize, imgHeight - yGridSize);
		g2d.drawLine(xGridSize, imgHeight - yGridSize, imgWidth - xGridSize, imgHeight - yGridSize);

		return bufferedImage;
	}

	
	protected String getLabelText(float label){
		String text =Float.toString(label);
		if(text.contains(".0")){
			int value  = (int) label;
			text = Integer.toString(value);
		}	
		return text;
	}
	
	protected int getTitleFontSize(int yGridSize) {
		return yGridSize / 4;
	}

	protected int getAxisFontSize(int gridSize) {
		return gridSize / 7;
	}

	protected void getAxisValueSize() {
		int y = getAxisFontSize(yGridSize);
		int x = getAxisFontSize(xGridSize);
		if (x <= y)
			g2d.setFont(new Font("Calibri", Font.PLAIN, getLabelFontSize(xGridSize)));
		else
			g2d.setFont(new Font("Calibri", Font.PLAIN, getLabelFontSize(yGridSize)));
	}

	protected int getLabelFontSize(int gridSize) {
		return gridSize / 5;
	}

	protected void getValueSize() {
		int y = getLabelFontSize(yGridSize);
		int x = getLabelFontSize(xGridSize);

		if (x <= y)
			g2d.setFont(new Font("Calibri", Font.PLAIN, getLabelFontSize(xGridSize)));
		else
			g2d.setFont(new Font("Calibri", Font.PLAIN, getLabelFontSize(yGridSize)));
	}

	protected float getBasicStroke() {
		if (imgWidth <= 400)
			return BASIC_STROKE = 1f;
		if (imgWidth >= 400 && imgWidth < 800)
			return BASIC_STROKE = 2f;
		if (imgWidth >= 800 && imgWidth < 1200)
			return BASIC_STROKE = 3f;
		if (imgWidth >= 1200 && imgWidth < 1600)
			return BASIC_STROKE = 5f;
		if (imgWidth >= 1600 && imgWidth < 2000)
			return BASIC_STROKE = 7f;
		if (imgWidth > 2000)
			return BASIC_STROKE = 9f;
		return BASIC_STROKE;
	}

	protected void literalCal(String xAttributesName, String yAttributesName) {
		xTitle = xAttributesName.replaceAll("[^a-zA-Z]", " ");
		yTitle = yAttributesName.replaceAll("[^a-zA-Z]", " ");
		this.xAttributesName = xAttributesName;
		this.yAttributesName = yAttributesName;
	}

	public void updateMap(RenderedImage newImage) {
		this.image = newImage;
		// save image into a file
		File outputfile = new File("D:/UWV final project/saved.png");
		try {
			ImageIO.write(newImage, "png", outputfile);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/*
	 * creates Cartesian image
	 */
	public RenderedImage createMapImage() {
		return this.image;
	}

	/*
	 * creates XQuey
	 */
	public Map<String, String> createMapQuery() {
		return this.query;
	}
}
