package org.processmining.plugins.automapgen.cartesian;

import java.awt.image.BufferedImage;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.SortedSet;

/**
 * @author ngupta
 *
 */
public class CartesianContinuous extends Cartesian {
	private StringBuffer query = new StringBuffer();
	private float XLabel, YLabel;
	

	public CartesianContinuous(MapType mapType, int imgHeight, int imgWidth, String title, String xTitle, String yTitle,
			int xSize, int ySize, float xMin, float xMax, int xDivision, float yMin, float yMax, int yDivision,
			SortedSet<String> xKeys, SortedSet<String> yKeys, Collection<?> eventNames) {
		super(mapType, imgHeight, imgWidth, title, xTitle, yTitle, xSize, ySize, xMin, xMax, xDivision, yMin, yMax,
				yDivision, xKeys, yKeys, eventNames);
	}

	@Override
	public BufferedImage createMapImage() {
		literalCal(xTitle, yTitle);

		rows = xDivision + 2;
		cols = yDivision + 2;
		xGridSize = imgWidth / rows;
		yGridSize = imgHeight / cols;
		drawBasicCartesianImage();
		float boundX = xMax - xMin;
		float boundY = yMax - yMin;
		getValueSize();

		float xlabel = contXLabel(xMin, 0, boundX, xMax, xDivision);
		df.format(xlabel);	
		g2d.drawString(getLabelText(xlabel), xGridSize - 5, imgHeight - yGridSize * 3 / 5);
		float ylabel = contYLabel(yMin, 0, boundY, yMax, yDivision);
		
		df.format(ylabel);
		g2d.drawString(getLabelText(ylabel), xGridSize * 3 / 5 - 10, imgHeight - 1 * yGridSize);

		for (int i = 1; i < rows - 1; i++) {
			for (int j = 1; j < cols - 1; j++) {
				//draw x axis line
				g2d.drawLine((i + 1) * xGridSize, yGridSize, (i + 1) * xGridSize, imgHeight - yGridSize);

				g2d.drawLine(xGridSize, imgHeight - (j + 1) * yGridSize, imgWidth - xGridSize, imgHeight - (j + 1)
						* yGridSize);

				//X label
				xlabel = contXLabel(xMin, i, boundX, xMax, xDivision);
				df.format(xlabel);
				g2d.drawString(getLabelText(xlabel), (i + 1) * xGridSize - 5, imgHeight - yGridSize * 3 / 5);

				// Y label
				ylabel = contYLabel(yMin, j, boundY, yMax, yDivision);
				df.format(ylabel);
				g2d.drawString(getLabelText(ylabel), xGridSize * 3 / 5 - 10, imgHeight - (j + 1) * yGridSize);

				// get first value of x and y axis
				if (i == 1 && j == 1) {
					XLabel = xlabel;
					YLabel = ylabel;
				}
			}
		}
		g2d.dispose();
		return bufferedImage;
	}

	private float contXLabel(float xMin, int i, float boundX, float xMax, int xDivision) {
		return  xMin + i * (boundX / xDivision);
	}

	private float contYLabel(float yMin, int j, float boundY, float yMax, int yDivision) {
		return  yMin + j * (boundY / yDivision);	
	}

	// X coordinate query
	public Map<String, String> createMapQuery() {
		Map<String, String> xQueries = new HashMap<String, String>();
		for (Object taskName : eventName) {
			// X coordinate query
			String xList = createXContinous(xGridSize, XLabel);
			String yList = createYContinous(yGridSize, YLabel);
			StringBuffer xquery = new StringBuffer("<attributes>");
			xquery.append("\n\t").append("<attribute><x>").append(taskName);
			xquery.append("</x>").append("<y>").append(taskName).append("</y>").append("\n");
			xquery.append("\t").append("<query>").append("\n\t<![CDATA[");
			xquery.append("<coordinate><x>{").append(xList).append("}</x><y>{").append(yList);
			xquery.append("}</y><z>0</z></coordinate>]]>").append("\n\t").append("</query>").append("\n");
			xquery.append("</attribute>").append("\n");
			xquery.append("</attributes>");
			xQueries.put(taskName.toString(), xquery.toString());
		}
		return xQueries;
	}

	private String createYContinous(int YGridSize, float YLabel) {
		return imgHeight + "- (" + YGridSize + " + " + "((number(//" + yAttributesName + ")) div " + YLabel + ")*"
				+ YGridSize + ")";
	}

	private String createXContinous(int XGridSize, float XLabel) {
		return XGridSize + " + " + "(number(//" + xTitle + ") div  " + " " + XLabel + ") *" + XGridSize;
	}

	public String getQuery() {
		return query.toString();
	}

}
