package org.processmining.plugins.automapgen.cartesian;

import java.awt.image.BufferedImage;
import java.util.Collection;
import java.util.Map;
import java.util.SortedSet;

/**
 * @author ngupta
 *
 */
public class CartesianFactory {
	private CartesianLiteral literal;
	private CartesianContinuous continuous;
	private CartesianYLiteral yLiteral;
	private CartesianXLiteral xLiteral;
	private Map<String, String> query;
	private BufferedImage image;

	public CartesianFactory(MapType mapType, int imgHeight, int imgWidth, String title, String xTitle, String yTitle,
			int xSize, int ySize, float xMin, float xMax, int xDivision, float yMin, float yMax, int yDivision,
			SortedSet<String> xKeys, SortedSet<String> yKeys, Collection<?> eventNames) {
		switch (mapType) {
			case LITERAL :
				literal = new CartesianLiteral(mapType, imgHeight, imgWidth, title, xTitle, yTitle, xSize, ySize, xMin,
						xMax, xDivision, yMin, yMax, yDivision, xKeys, yKeys, eventNames);
				break;
			case CONTINOUS :
				continuous = new CartesianContinuous(mapType, imgHeight, imgWidth, title, xTitle, yTitle, xSize, ySize,
						xMin, xMax, xDivision, yMin, yMax, yDivision, xKeys, yKeys, eventNames);
				break;
			case XLITERAL :
				xLiteral = new CartesianXLiteral(mapType, imgHeight, imgWidth, title, xTitle, yTitle, xSize, ySize,
						xMin, xMax, xDivision, yMin, yMax, yDivision, xKeys, yKeys, eventNames);
				break;
			case YLITERAL :
				yLiteral = new CartesianYLiteral(mapType, imgHeight, imgWidth, title, xTitle, yTitle, xSize, ySize,
						xMin, xMax, xDivision, yMin, yMax, yDivision, xKeys, yKeys, eventNames);
				break;
			default :
				System.out.println("WARNING: issue in cartesian graph creation");
				break;
		}
	}

	public BufferedImage buildMapImage(MapType mapType) {
		switch (mapType) {
			case LITERAL :
				image = literal.createMapImage();
				break;
			case CONTINOUS :
				image = continuous.createMapImage();
				break;
			case XLITERAL :
				image = xLiteral.createMapImage();
				break;
			case YLITERAL :
				image = yLiteral.createMapImage();
				break;
			default :
				System.out.println("WARNING: issue in cartesian graph creation");
				break;
		}
		return image;
	}

	public Map<String, String> createXQuery(MapType mapType) {
		switch (mapType) {
			case LITERAL :
				query = literal.createMapQuery();
				break;
			case CONTINOUS :
				query = continuous.createMapQuery();
				break;
			case XLITERAL :
				query = xLiteral.createMapQuery();
				break;
			case YLITERAL :
				query = yLiteral.createMapQuery();
				break;
			default :
				System.out.println("WARNING: issue in cartesian graph update");
				break;
		}
		return query;
	}
}
