package org.processmining.plugins.automapgen.cartesian;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;

/**
 * @author ngupta
 *
 */
public class CartesianLiteral extends Cartesian {
	private ArrayList<Integer> xEventDotPoints;
	private ArrayList<Integer> yEventDotPoints;
	private Iterator<String> iter;

	public CartesianLiteral(MapType mapType, int imgHeight, int imgWidth, String title, String xTitle, String yTitle,
			int xSize, int ySize, float xMin, float xMax, int xDivision, float yMin, float yMax, int yDivision,
			SortedSet<String> xKeys, SortedSet<String> yKeys, Collection<?> eventNames) {
		super(mapType, imgHeight, imgWidth, title, xTitle, yTitle, xSize, ySize, xMin, xMax, xDivision, yMin, yMax,
				yDivision, xKeys, yKeys, eventNames);
		xEventDotPoints = new ArrayList<Integer>();
		yEventDotPoints = new ArrayList<Integer>();
	}

	@Override
	public BufferedImage createMapImage() {
		literalCal(xTitle, yTitle);
		rows = xSize + 1;
		cols = ySize + 1;
		xGridSize = imgWidth / rows;
		yGridSize = imgHeight / cols;

		drawBasicCartesianImage();
		for (int i = 1; i < (rows - 1); i++) {
			for (int j = 1; j < (cols - 1); j++) {
				//draw x axis line
				g2d.drawLine((i + 1) * xGridSize, yGridSize, (i + 1) * xGridSize, imgHeight - yGridSize);

				//draw y axis lines
				g2d.drawLine(xGridSize, imgHeight - (j + 1) * yGridSize, imgWidth - xGridSize, imgHeight - (j + 1)
						* yGridSize);

			}
		}
		for (int i = 1; i < rows; i++) {
			for (int j = 1; j < cols; j++) {
				// find mid-point of x and y axis intersection to plot a dot 
				xEventDotPoints.add((i) * xGridSize);
				yEventDotPoints.add(imgHeight - (j) * yGridSize);
			}
		}
		getValueSize();
		xDrawVlaues(xKeys);
		yDrawValues(yKeys);
		g2d.dispose();
		return bufferedImage;
	}

	private void rotateAxisLabels(double d) {
		AffineTransform at = new AffineTransform();
		at.rotate(d);
		Font rotatedFont = g2d.getFont().deriveFont(at);
		g2d.setFont(rotatedFont);
	}

	/*
	 * x axis lines value
	 */
	private RenderedImage xDrawVlaues(SortedSet<String> xKeys) {
		iter = xKeys.iterator();
		int xStartvalue = 20;
		rotateAxisLabels(Math.PI / 4.0);
		for (int i = 0; iter.hasNext(); i++) {
			String xValue = iter.next();
			if (xValue.length() > 15) {
				xValue = xValue.substring(0, 14);
			}	
			
			g2d.drawString(xValue, (i + 1) * xGridSize, imgHeight - yGridSize + xStartvalue);
		}
		return bufferedImage;
	}

	/*
	 * y axis lines value
	 */
	private RenderedImage yDrawValues(SortedSet<String> yKeys) {
		iter = yKeys.iterator();
		AffineTransform fontAT = new AffineTransform();
		fontAT.rotate(-Math.PI / .5);
		Font rotatedFont = g2d.getFont().deriveFont(fontAT);
		g2d.setFont(rotatedFont);
		for (int j = 0; iter.hasNext(); j++) {
			String yValue = iter.next();

			if (yValue.length() > 15) {
				yValue = yValue.substring(0, 15);
			}
			
			FontMetrics fm = g2d.getFontMetrics();
			int width = fm.stringWidth(yValue);

			g2d.drawString(yValue, imgWidth - xGridSize * (rows - 1) - (width + 5), imgHeight - (j + 1) * yGridSize);
		}
		return bufferedImage;
	}

	public Map<String, String> createMapQuery() {
		Map<String, String> xQueries = new HashMap<String, String>();
		String newXQuery = createXQuery();
		for (Object taskName : eventName) {
			xQueries.put(taskName.toString(), newXQuery);
		}
		return xQueries;
	}

	public String createXQuery() {
		StringBuffer xquery = new StringBuffer("<attributes>");
		iter = xKeys.iterator();

		while (iter.hasNext()) {
			String xValue = iter.next();
			Iterator<String> yiter = yKeys.iterator();
			while (yiter.hasNext()) {
				String yValue = yiter.next();
				xquery.append("\n\t").append("<attribute><x>").append(xValue).append("</x>").append("<y>")
						.append(yValue).append("</y>").append("\n");
				xquery.append("\t").append("<query>").append("\n\t<![CDATA[");
				xquery.append("<coordinate><x>").append("X").append("</x><y>").append("Y")
						.append("</y><z>0</z></coordinate>]]>").append("\n\t").append("</query>").append("\n");
				xquery.append("</attribute>").append("\n");
			}
		}

		xquery.append("</attributes>");

		String xSplitQuery = "";
		List<String> elements;
		elements = Arrays.asList(xquery.toString().split("X"));

		xEventDotPoints.add(0);
		Iterator<String> querySplit = elements.iterator();
		Iterator<Integer> iterEventPoints = xEventDotPoints.iterator();

		while (iterEventPoints.hasNext()) {
			while (querySplit.hasNext()) {
				if (!querySplit.hasNext() && !iterEventPoints.hasNext())
					continue;
				String xPixel = iterEventPoints.next().toString();
				String splitValue = querySplit.next();
				xSplitQuery = xSplitQuery + splitValue + xPixel;
			}
		}

		String ySplitQuery = "";
		yEventDotPoints.add(0);
		Iterator<Integer> yiterEvent = yEventDotPoints.iterator();

		elements = Arrays.asList(xSplitQuery.toString().split("Y"));

		Iterator<String> yquerySplit = elements.iterator();
		while (yiterEvent.hasNext()) {
			while (yquerySplit.hasNext()) {

				if (!yquerySplit.hasNext() && yiterEvent.hasNext())
					continue;

				String yPixel = yiterEvent.next().toString();
				ySplitQuery = ySplitQuery + yquerySplit.next().toString() + yPixel;
			}
		}

		ySplitQuery = ySplitQuery.substring(0, (ySplitQuery.length() - 2));
		return ySplitQuery;
	}
}
