package org.processmining.plugins.automapgen.cartesian;

import java.awt.Font;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;

/**
 * @author ngupta
 *
 */
public class CartesianXLiteral extends Cartesian {
	private ArrayList<Integer> xEventDotPoints = new ArrayList<Integer>();
	private Iterator<String> iter;
	private float YLabel;

	public CartesianXLiteral(MapType mapType, int imgHeight, int imgWidth, String title, String xTitle, String yTitle,
			int xSize, int ySize, float xMin, float xMax, int xDiv, float yMin, float yMax, int yDiv,
			SortedSet<String> xKeys, SortedSet<String> yKeys, Collection<?> eventNames) {
		super(mapType, imgHeight, imgWidth, title, xTitle, yTitle, xSize, ySize, xMin, xMax, xDiv, yMin, yMax, yDiv,
				xKeys, yKeys, eventNames);
	}

	@Override
	public BufferedImage createMapImage() {
		rows = xSize + 1;
		cols = yDivision + 2;
		float boundY = yMax - yMin;
		xGridSize = imgWidth / rows;
		yGridSize = imgHeight / cols;

		literalCal(xTitle, yTitle);
		drawBasicCartesianImage();
		getValueSize();

		float ylabel = contYLabel(yMin, 0, boundY, yMax, yDivision);
		df.format(ylabel);
		g2d.drawString(getLabelText(ylabel), xGridSize * 3 / 5 - 10, imgHeight - 1 * yGridSize);

		for (int i = 1; i < (rows - 1); i++) {
			for (int j = 1; j < (cols - 1); j++) {
				//draw x axis line
				g2d.drawLine((i + 1) * xGridSize, yGridSize, (i + 1) * xGridSize, imgHeight - yGridSize);

				// Y label
				ylabel = contYLabel(yMin, j, boundY, yMax, yDivision);
				df.format(ylabel);
				g2d.drawString(getLabelText(ylabel), xGridSize * 3 / 5 - 10, imgHeight - (j + 1) * yGridSize);

				//draw y axis lines
				g2d.drawLine(xGridSize, imgHeight - (j + 1) * yGridSize, imgWidth - xGridSize, imgHeight - (j + 1)
						* yGridSize);

				if (i == 1 && j == 1) {
					YLabel = ylabel;
				}
			}
		}

		// find mid-point of x intersection to plot a dot 
		for (int i = 1; i < rows; i++) {
			xEventDotPoints.add(i * xGridSize);
		}
		xDrawVlaues(xKeys);
		g2d.dispose();
		return bufferedImage;
	}

	private void rotateAxisLabels(double d) {
		AffineTransform at = new AffineTransform();
		at.rotate(d);
		Font rotatedFont = g2d.getFont().deriveFont(at);
		g2d.setFont(rotatedFont);
	}

	private RenderedImage xDrawVlaues(SortedSet<String> xKeys) {
		iter = xKeys.iterator();
		int xStartvalue = 20;
		rotateAxisLabels(Math.PI / 4.0);
		for (int i = 0; iter.hasNext(); i++) {
			String xValue = iter.next();
			if (xValue.length() > 15) {
				xValue = xValue.substring(0, 14);
			}

			g2d.drawString(xValue, (i + 1) * xGridSize, imgHeight - yGridSize + xStartvalue);
		}
		return bufferedImage;

	}

	private float contYLabel(float yMin, int j, float boundY, float yMax, int yDivision) {
		return (yMin + j * (boundY / yDivision));
	}

	private String createYContinous(int YGridSize, float YLabel) {
		return imgHeight + "- (" + YGridSize + " + " + "((number(//" + yAttributesName + ")) div " + YLabel + ")*"
				+ YGridSize + ")";
	}

	public Map<String, String> createMapQuery() {
		// X coordinate query
		ArrayList<String> xList = createXLiteral(xKeys);
		String Xliteral = XLiteralQuery(xList);
		String yQuery = createYContinous(yGridSize, YLabel);

		Map<String, String> xQueries = new HashMap<String, String>();
		for (Object taskName : eventName) {
			StringBuffer xquery = new StringBuffer();
			xquery.append("<attributes>");
			xquery.append("\n\t").append("<attribute><x>").append(taskName);
			xquery.append("</x>").append("<y>").append(taskName).append("</y>").append("\n");
			xquery.append("\t").append("<query>").append("\n\t<![CDATA[");
			xquery.append("<coordinate><x>{").append(Xliteral).append("}</x><y>{").append(yQuery);
			xquery.append("}</y><z>0</z></coordinate>]]>").append("\n\t").append("</query>").append("\n");
			xquery.append("</attribute>").append("\n");
			xquery.append("</attributes>");

			xQueries.put(taskName.toString(), xquery.toString());
		}
		return xQueries;
	}

	private ArrayList<String> createXLiteral(SortedSet<String> xValues) {
		// X coordinate query
		iter = xValues.iterator();
		ArrayList<String> xList = new ArrayList<String>();
		while (iter.hasNext()) {
			String xValue = iter.next();
			String x = "string(//" + xAttributesName + ")= " + '"' + xValue + '"';
			xList.add(x);
		}
		return xList;
	}

	private String XLiteralQuery(ArrayList<String> xList) {
		iter = xList.iterator();
		String xquery = "";
		String query = "";
		for (int i = 1; iter.hasNext(); i++) {
			String xValue = iter.next();
			if (i == 1) {
				xquery = xquery + "if(" + xValue + ") then split";
			} else {
				query = "if(" + xValue + ") then split";
				xquery = xquery + " else " + query;
			}
		}

		List<String> elements;
		elements = Arrays.asList(xquery.split("split"));

		String xSplitQuery = "";
		Iterator<Integer> iterEvent = xEventDotPoints.iterator();

		Iterator<String> querySplit = elements.iterator();
		while (iterEvent.hasNext()) {
			while (querySplit.hasNext()) {
				if (!querySplit.hasNext())
					continue;
				String xPixel = iterEvent.next().toString();
				xSplitQuery = xSplitQuery + querySplit.next().toString() + xPixel;
			}
		}

		xSplitQuery = xSplitQuery + " else -1 ";
		return xSplitQuery;
	}
}
