package org.processmining.plugins.automapgen.cartesian;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;

/**
 * @author ngupta
 *
 */
public class CartesianYLiteral extends Cartesian {
	private ArrayList<Integer> yEventDotPoints;
	private Iterator<String> iter;
	private StringBuffer query = new StringBuffer();
	private float XLabel;

	public CartesianYLiteral(MapType mapType, int imgHeight, int imgWidth, String title, String xTitle, String yTitle,
			int xSize, int ySize, float xMin, float xMax, int xDivision, float yMin, float yMax, int yDivision,
			SortedSet<String> xKeys, SortedSet<String> yKeys, Collection<?> eventNames) {
		super(mapType, imgHeight, imgWidth, title, xTitle, yTitle, xSize, ySize, xMin, xMax, xDivision, yMin, yMax,
				yDivision, xKeys, yKeys, eventNames);
		yEventDotPoints = new ArrayList<Integer>();
	}

	@Override
	public BufferedImage createMapImage() {
		rows = xDivision + 2;
		cols = ySize + 1;
		xGridSize = imgWidth / rows;
		yGridSize = imgHeight / cols;

		literalCal(xTitle, yTitle);
		drawBasicCartesianImage();

		float boundX = xMax - xMin;
		getValueSize();
		float xlabel = contXLabel(xMin, 0, boundX, xMax, xDivision);
		df.format(xlabel);
		g2d.drawString(getLabelText(xlabel), xGridSize - 5, imgHeight - yGridSize * 3 / 5);

		for (int i = 1; i < rows - 1; i++) {
			for (int j = 1; j < cols - 1; j++) {
				//draw x axis line
				g2d.drawLine((i + 1) * xGridSize, yGridSize, (i + 1) * xGridSize, imgHeight - yGridSize);

				//draw y axis lines
				g2d.drawLine(xGridSize, imgHeight - (j + 1) * yGridSize, imgWidth - xGridSize, imgHeight - (j + 1)
						* yGridSize);

				//X label
				xlabel = contXLabel(xMin, i, boundX, xMax, xDivision);
				df.format(xlabel);
				g2d.drawString(getLabelText(xlabel), (i + 1) * xGridSize - 5, imgHeight - yGridSize * 3 / 5);

				// find mid-point of y axis intersection to plot a dot 
				if (!yEventDotPoints.contains(imgHeight - (j) * yGridSize)) {
					yEventDotPoints.add(imgHeight - (j) * yGridSize);
				}
				if (i == 1 && j == 1) {
					XLabel = xlabel;
				}
			}
		}

		yEventDotPoints.add(imgHeight - (rows - 1) * yGridSize);
		yDrawValues(yKeys);
		g2d.dispose();
		return bufferedImage;
	}

	private RenderedImage yDrawValues(SortedSet<String> yKeys) {
		//y axis values
		iter = yKeys.iterator();
		AffineTransform fontAT = new AffineTransform();
		fontAT.rotate(-Math.PI / .5);
		Font rotatedFont = g2d.getFont().deriveFont(fontAT);
		g2d.setFont(rotatedFont);
		for (int j = 0; iter.hasNext(); j++) {
			String yValue = iter.next();

			if (yValue.length() > 15) {
				yValue = yValue.substring(0, 15);
			}

			FontMetrics fm = g2d.getFontMetrics();
			int width = fm.stringWidth(yValue);

			g2d.drawString(yValue, imgWidth - xGridSize * (rows - 1) - (width + 5), imgHeight - (j + 1) * yGridSize);
		}
		return bufferedImage;
	}

	private float contXLabel(float xMin, int i, float boundX, float xMax, int xDivision) {
		return (xMin + i * (boundX / xDivision));
	}

	public Map<String, String> createMapQuery() {
		// X coordinate query
		String xList = createXContinous(xGridSize, XLabel);
		ArrayList<String> yList = createYLiteral(yKeys);
		String YLiteral = YLiteralQuery(yList);

		Map<String, String> xQueries = new HashMap<String, String>();
		for (Object taskName : eventName) {
			StringBuffer xquery = new StringBuffer("<attributes>");
			xquery.append("\n\t").append("<attribute><x>").append(taskName);
			xquery.append("</x>").append("<y>").append(taskName).append("</y>").append("\n");
			xquery.append("\t").append("<query>").append("\n\t<![CDATA[");
			xquery.append("<coordinate><x>{").append(xList).append("}</x><y>{").append(YLiteral);
			xquery.append("}</y><z>0</z></coordinate>]]>").append("\n\t").append("</query>").append("\n");
			xquery.append("</attribute>").append("\n");
			xquery.append("</attributes>");

			xQueries.put(taskName.toString(), xquery.toString());
		}
		return xQueries;
	}

	private String createXContinous(int XGridSize, float XLabel) {
		return XGridSize + " + " + "(number(//" + xAttributesName + ") div  " + " " + XLabel + ") *" + XGridSize;
	}

	private String YLiteralQuery(ArrayList<String> yList) {
		iter = yList.iterator();
		String yquery = "";
		String query = "";
		for (int i = 1; iter.hasNext(); i++) {
			String xValue = iter.next();
			if (i == 1) {
				yquery = yquery + "if(" + xValue + ") then split ";
			} else {
				query = "if(" + xValue + ") then split";
				yquery = yquery + " else " + query;
			}
		}

		List<String> elements;
		elements = Arrays.asList(yquery.split("split"));
		//Y Coordinate query
		String ySplitQuery = "";
		Iterator<Integer> yiterEvent = yEventDotPoints.iterator();

		Iterator<String> yquerySplit = elements.iterator();
		while (yiterEvent.hasNext()) {
			while (yquerySplit.hasNext()) {
				String yPixel = yiterEvent.next().toString();
				ySplitQuery = ySplitQuery + yquerySplit.next().toString() + yPixel;

			}
		}
		ySplitQuery = ySplitQuery + " else -1 ";
		return ySplitQuery;
	}

	private ArrayList<String> createYLiteral(SortedSet<String> yValues) {
		iter = yValues.iterator();
		ArrayList<String> yList = new ArrayList<String>();
		while (iter.hasNext()) {
			String yValue = iter.next().toString();
			String y = "string(//" + yAttributesName + ")= " + '"' + yValue + '"';
			yList.add(y);
		}
		return yList;
	}

	public String getQuery() {
		return query.toString();
	}
}
