/**
 * 
 */
package org.processmining.plugins.automapgen.petrinet;

import java.awt.Dimension;
import java.awt.geom.Point2D;
import java.util.Map;

import javax.swing.JFrame;
import javax.swing.SwingConstants;

import org.processmining.framework.util.ui.scalableview.ScalableViewPanel;
import org.processmining.framework.util.ui.scalableview.interaction.ZoomInteractionPanel;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.ViewSpecificAttributeMap;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.jgraph.ProMGraphModel;
import org.processmining.models.jgraph.ProMJGraph;
import org.processmining.models.jgraph.visualization.ProMJGraphPanel;

import com.jgraph.layout.JGraphFacade;
import com.jgraph.layout.hierarchical.JGraphHierarchicalLayout;

/**
 * @author ngupta
 *
 */
public class PetriNetVisualizer extends JFrame {

	private static final long serialVersionUID = 1L;
	private PetrinetGraph net;
	private GraphLayoutConnection layoutConnection;
	ProMJGraphPanel panel;
	private ZoomInteractionPanel zoomPanel;
	int size_x = 0, size_y = 0;
	ProMJGraph graph;
	private  JFrame petriFrame;

	public PetriNetVisualizer(PetrinetGraph net, String title) {
		this.net = net;
		petriFrame = new JFrame();
		petriFrame.setTitle(net.getLabel());
		initComponent();
	}

	void initComponent() {
		// Graph layout
		setLayoutConnection(new GraphLayoutConnection(net));
		ProMGraphModel model = new ProMGraphModel(net);
		ViewSpecificAttributeMap map = new ViewSpecificAttributeMap();
		setGraph(new ProMJGraph(model, map, getLayoutConnection()));
		JGraphHierarchicalLayout layout = new JGraphHierarchicalLayout();
		layout.setDeterministic(false);
		layout.setCompactLayout(false);
		layout.setFineTuning(true);
		layout.setParallelEdgeSpacing(15);
		layout.setFixRoots(false);
		layout.setOrientation(map.get(net, AttributeMap.PREF_ORIENTATION, SwingConstants.SOUTH));
		if (!getLayoutConnection().isLayedOut()) {
			JGraphFacade facade = new JGraphFacade(getGraph());
			facade.setOrdered(true);
			facade.setEdgePromotion(true);
			facade.setIgnoresCellsInGroups(false);
			facade.setIgnoresHiddenCells(false);
			facade.setIgnoresUnconnectedCells(false);
			facade.setDirected(true);
			facade.resetControlPoints();
			facade.run(layout, true);
			
			Map<?, ?> nested = facade.createNestedMap(true, true);
			getGraph().getGraphLayoutCache().edit(nested);
			getLayoutConnection().setLayedOut(true);
		}
		getGraph().setUpdateLayout(layout);
		panel = new ProMJGraphPanel(getGraph());
	
		
		zoomPanel = new ZoomInteractionPanel(panel, ScalableViewPanel.MAX_ZOOM);
		panel.addViewInteractionPanel(zoomPanel, SwingConstants.WEST);
		getLayoutConnection().updated();

		int new_size_x = 0, new_size_y = 0;
		for (PetrinetNode t : net.getTransitions()) {
			Point2D p = getLayoutConnection().getPosition(t);
			Dimension dim = getLayoutConnection().getSize(t);
			new_size_x = (int) p.getX() + dim.width;
			new_size_y = (int) p.getY() + dim.height;
			if (new_size_x > size_x)
				size_x = new_size_x;
			if (new_size_y > size_y)
				size_y = new_size_y;
		}
		for (PetrinetNode t : net.getNodes()) {
			Point2D p = getLayoutConnection().getPosition(t);
			Dimension dim = getLayoutConnection().getSize(t);
			new_size_x = (int) p.getX() + dim.width;
			new_size_y = (int) p.getY() + dim.height;
			if (new_size_x > size_x)
				size_x = new_size_x;
			if (new_size_y > size_y)
				size_y = new_size_y;
		}
		size_x += 10;
		size_y += 10;
		panel.setPreferredSize(new Dimension(size_x, size_y));
		petriFrame.setSize(400, 400);
		petriFrame.setVisible(true);		
		petriFrame.add(panel);

		
		
	}

	public ProMJGraph getGraph() {
		return graph;
	}

	public void setGraph(ProMJGraph graph) {
		this.graph = graph;
		
	}

	public GraphLayoutConnection getLayoutConnection() {
		return layoutConnection;
	}

	public void setLayoutConnection(GraphLayoutConnection layoutConnection) {
		this.layoutConnection = layoutConnection;
	}

	

}
