package org.processmining.plugins.automapgen.petrinet;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.geom.Point2D;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.util.ui.widgets.ProMComboBox;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.jgraph.ProMJGraph;
import org.processmining.plugins.automapgen.History;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class PetriNetWizard extends JPanel implements java.awt.event.ActionListener {

	private static final long serialVersionUID = 1L;
	private PetriGraphMapping<String, Transition> mappingPanel;
	private UIPluginContext context;
	private PetrinetGraph net;
	int size_x = 0, size_y = 0;
	private History history;
	GraphLayoutConnection layout;
	private PetriNetVisualizer visualizer;
	private JPanel petriNetPanel;
	@SuppressWarnings("rawtypes")
	private Map<String, ProMComboBox> comboBoxes = new HashMap<String, ProMComboBox>();
	private JCheckBox defaultNoMapping;

	public PetriNetWizard(History history, XLog log, PetrinetGraph net, UIPluginContext context) {
		super();
		this.context = context;
		this.net = net;
		this.history = history;
		visualizer = new PetriNetVisualizer(net, "PetriNet visualizer");
		initComponent(log);
	}

	private void initComponent(XLog log) {
		history.addActvityNames(log);
		petriNetPanel = new JPanel();
		petriNetPanel.setSize(1000, 400);
		petriNetPanel.setLayout(new BoxLayout(petriNetPanel, BoxLayout.Y_AXIS));
		petriNetPanel.add(Box.createVerticalStrut(10));
		
		createPanel(false);
		
		// scroll able panel
		JScrollPane panelPane = new JScrollPane(petriNetPanel);
		petriNetPanel.setAutoscrolls(true);
		this.add(panelPane);
	}

	public void createPanel(boolean isNoMapping) {

		petriNetPanel.setName("Event Mapping");

		petriNetPanel.setBackground(Color.BLACK);
		JPanel componentPanel = new JPanel();
		componentPanel.setLayout(new GridLayout());

		JLabel dataSetEvent = new JLabel();
		dataSetEvent.setText("Data-Set event names");
		dataSetEvent.setFont(new Font("Calibri", Font.BOLD, 20));
		dataSetEvent.setForeground(Color.white);

		JLabel transitionNames = new JLabel();
		transitionNames.setText("Petri-Net transition names");
		transitionNames.setFont(new Font("Calibri", Font.BOLD, 20));
		transitionNames.setForeground(Color.white);

		componentPanel.add(dataSetEvent);
		componentPanel.add(transitionNames);

		componentPanel.setBackground(Color.GRAY);
		petriNetPanel.add(componentPanel);

		JLabel info = new JLabel();
		info.setText("If user does not want to project an event on visualization tool");
		info.setFont(new Font("Calibri", Font.PLAIN, 12));
		info.setForeground(Color.white);

		JLabel blankinfo = new JLabel();
		blankinfo.setText(" ");
		blankinfo.setFont(new Font("Calibri", Font.PLAIN, 12));

		JPanel componentPanel1 = new JPanel();
		componentPanel1.setLayout(new GridLayout());

		componentPanel1.add(info);
		componentPanel1.add(blankinfo);
		componentPanel1.setBackground(Color.GRAY);
		petriNetPanel.add(componentPanel1);

		JLabel info1 = new JLabel();
		info1.setText("then select NO MAPPING from combo box");
		info1.setFont(new Font("Calibri", Font.PLAIN, 12));
		info1.setForeground(Color.white);

		JPanel componentPanel2 = new JPanel();
		componentPanel2.setLayout(new GridLayout());

		componentPanel2.add(info1);
		componentPanel2.add(blankinfo);
		componentPanel2.setBackground(Color.GRAY);
		petriNetPanel.add(componentPanel2);

		JPanel noMappingPanel = new JPanel();
		noMappingPanel.setLayout(new GridLayout());

		defaultNoMapping = SlickerFactory.instance().createCheckBox("Default NO MAPPING", isNoMapping);
		defaultNoMapping.addActionListener(this);
		noMappingPanel.add(defaultNoMapping);
		noMappingPanel.add(blankinfo);
		noMappingPanel.setBackground(Color.GRAY);
		petriNetPanel.add(noMappingPanel);

		for (String value : history.getActivityNames()) {
			mappingPanel = new PetriGraphMapping<String, Transition>(value, net.getTransitions(), isNoMapping);
			petriNetPanel.add(mappingPanel.componentPanel);
			comboBoxes.putAll(mappingPanel.comboBoxes);
		}

		this.add(petriNetPanel);

	}

	public void actionPerformed(ActionEvent arg0) {
		comboBoxes.clear();
		this.petriNetPanel.removeAll();
		if (defaultNoMapping.isSelected()) {
			createPanel(true);
		} else {
			createPanel(false);
		}
		this.repaint();
		this.revalidate();
	}

	public HashMap<String, String> getXQueries() {
		HashMap<String, String> retVal = new LinkedHashMap<String, String>();

		try {
			layout = context.getConnectionManager().getFirstConnection(GraphLayoutConnection.class, context, net);
		} catch (ConnectionCannotBeObtained e) {
			layout = new GraphLayoutConnection(net);
		}

		visualizer.getLayoutConnection().updated();
		mappingPanel.addComboValues(comboBoxes);
		Map<String, Transition> map = mappingPanel.getMapping(false);

		for (Entry<String, Transition> entry : map.entrySet()) {
			Point2D p = visualizer.getLayoutConnection().getPosition(entry.getValue());
			Dimension dim = visualizer.getLayoutConnection().getSize(entry.getValue());
			double x = (p.getX() + dim.width / 2) * getZoom();
			double y = (p.getY() + dim.height / 2) * getZoom();

			StringBuffer xquery = new StringBuffer("<attributes>");

			xquery.append("\n\t").append("<attribute><x>").append(entry.getKey().toString());
			xquery.append("</x>").append("<y>").append(entry.getKey().toString()).append("</y>").append("\n");
			xquery.append("\t").append("<query>").append("\n\t<![CDATA[");
			xquery.append("<coordinate><x>").append(x).append("</x><y>").append(y);
			xquery.append("</y><z>0</z></coordinate>]]>").append("\n\t").append("</query>").append("\n");
			xquery.append("</attribute>").append("\n");
			xquery.append("</attributes>");

			retVal.put(entry.getKey().toString(), xquery.toString());
		}

		return retVal;
	}

	public double getZoom() {
		return visualizer.panel.getScale();
	}

	public Dimension getDimension() {
		return new Dimension((int) (visualizer.size_x * getZoom()), (int) (visualizer.size_y * getZoom()));
	}

	public ProMJGraph getGraph() {
		return visualizer.graph;
	}

}
