package org.processmining.plugins.mapvis;

import java.io.File;
import java.io.IOException;

import org.mapdb.DB;
import org.mapdb.DBMaker;
import org.mapdb.DBMaker.Maker;

public class MapDBDatabase {

	private File dbFile;
	private boolean isTemporary = true;

	public MapDBDatabase() {
		this(null, true);
	}

	public MapDBDatabase(File dbFile) {
		this(dbFile, false);
	}

	public MapDBDatabase(File dbFile, boolean isTemporary) {
		super();
		this.dbFile = dbFile;
		this.isTemporary = isTemporary;
	}
	
	@SuppressWarnings("deprecation")
	public synchronized DB createDB() throws IOException {
		 DB db;
		if (dbFile == null) {
			dbFile = File.createTempFile("LogReplay", ".db");
		}
		
		Maker dbMaker = DBMaker.fileDB(dbFile)
								.closeOnJvmShutdown()
								.cacheDisable()
								.compressionEnable()
								.fileMmapEnableIfSupported()
								.transactionDisable();
		if (isTemporary) {
			dbMaker.deleteFilesAfterClose();
		}
		db = dbMaker.make();
		return db;
	}

}
