package org.processmining.plugins.mapvis;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.WeakHashMap;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.mapdb.DB;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.util.ui.widgets.ProMCheckComboBox;
import org.processmining.framework.util.ui.widgets.ProMPropertiesPanel;
import org.processmining.plugins.mapvis.importdata.visualization.MapVisData;
import org.processmining.plugins.mapvis.replay.History;
import org.processmining.plugins.mapvis.replay.TimeStampArray;

import com.fluxicon.slickerbox.components.RoundedPanel;
import com.fluxicon.slickerbox.factory.SlickerFactory;

class MyPropertiesPanel extends ProMPropertiesPanel {
	private static final long serialVersionUID = 1L;

	public MyPropertiesPanel(String title) {
		super(title);
	}

	@Override
	protected RoundedPanel packInfo(final String name, final JComponent component) {
		RoundedPanel newPanel = super.packInfo(name, component);
		JLabel nameLabel = (JLabel) newPanel.getComponent(1);
		nameLabel.setMinimumSize(new Dimension(300, 20));
		nameLabel.setMaximumSize(new Dimension(300, 1000));
		nameLabel.setPreferredSize(new Dimension(300, 30));
		return newPanel;
	}
}

@Plugin(name = "Replay Log on Map", 
parameterLabels = { "Event log", "Visualization data", "Visualization data 2" }, returnLabels = { "Visualization data" }, returnTypes = { VisData.class }, userAccessible = true, help = "Replay XES log on maps")
public class MapVis {

	@SuppressWarnings("rawtypes")
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "Massimiliano de Leoni", email = "m.d.leoni@tue.nl")
	@PluginVariant(variantLabel = "One MapVisData parameter", requiredParameterLabels = { 0, 1 })
	public static VisData plugin(UIPluginContext context, XLog log, MapVisData mapVisData) throws InterruptedException,
			CloneNotSupportedException, IOException {
		MapVisData newMapVisData = (MapVisData) mapVisData.clone();

		// Show all map file names
		ProMPropertiesPanel mapConfigurationPanel = new MyPropertiesPanel("Map configuration");
		mapCheckBox = new JCheckBox[newMapVisData.getMapNames().size() + 1];
		Collection<String> mapNames = newMapVisData.getMapNames();

		int j = 0;
		Iterator<String> iter = mapNames.iterator();
		while (iter.hasNext()) {
			j++;
			mapCheckBox[j] = SlickerFactory.instance().createCheckBox("", true);
			mapCheckBox[j].setFont(new Font("Calibri", Font.BOLD, 20));
			mapConfigurationPanel.addProperty(iter.next().trim().toString().toUpperCase(), mapCheckBox[j]);
		}

		InteractionResult result = context.showConfiguration("Map configuration", mapConfigurationPanel);

		if (result == InteractionResult.CANCEL) {
			context.getFutureResult(0).cancel(true);
			return null;
		}

		SortedSet<String> setAttributeNames = getAttributesSet(log);
		ProMPropertiesPanel configurationPanel = new MyPropertiesPanel("Configuration");
		ArrayList<String> selectedAttribs = new ArrayList<String>();
		for (String attributeName : setAttributeNames) {
			if (!attributeName.contains(":"))
				selectedAttribs.add(attributeName);
		}

		SlickerFactory slickerFactory = SlickerFactory.instance();

		chooserButton = slickerFactory.createButton("Choose directory");
		chooserButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				chooser = new JFileChooser();
				chooser.setDialogTitle("Select output directory for images");
				chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				workingDirectory = new File(path);
				chooser.setCurrentDirectory(workingDirectory);
				if (chooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
					path = chooser.getSelectedFile().getPath();
					dirPath.setText(path);

				}
			}
		});

		dirPath = slickerFactory.createLabel(path);
		dirPath.setFont(new Font("Calibri", Font.BOLD, 20));

		ProMCheckComboBox discriminatingAttributes = new ProMCheckComboBox(setAttributeNames);
		discriminatingAttributes.addSelectedItems(selectedAttribs);

		configurationPanel.addProperty("Attributes to consider :", discriminatingAttributes);
		configurationPanel.addProperty("Are attributes value changing over-time", referCheckBox);
	//	configurationPanel.addProperty("Use database for processing events", useDataBase);
		configurationPanel.addProperty("Select database directory", chooserButton);
		configurationPanel.addProperty("Default path: ", dirPath);

		result = context.showConfiguration("Configuration", configurationPanel);
		if (result == InteractionResult.CANCEL) {
			context.getFutureResult(0).cancel(true);
			return null;
		}

		context.getProgress().setMinimum(0);
		context.getProgress().setMaximum(100);

		long start = System.currentTimeMillis();
		// replay log, build history
		History history = new History();
		context.log("Building the sequence of states (Execution History)");

		Map<String, Boolean> cBoxValue = new WeakHashMap<String, Boolean>();

		// check which values are selected by user
		j = 0;
		Iterator<String> iterMap = mapNames.iterator();
		while (iterMap.hasNext()) {
			j++;
			Boolean mapResult = mapCheckBox[j].isSelected();
			cBoxValue.put(iterMap.next().toString(), mapResult);
		}

		Collection attributeCollection = discriminatingAttributes.getSelectedItems();
		String[] attributes = new String[attributeCollection.size()];
		int i = 0;
		for (Object attribute : attributeCollection)
			attributes[i++] = (String) attribute;

		setAttributeNames.clear();
		selectedAttribs.clear();
		attributeCollection.clear();

		// create a movies for all maps 
		history.buildEventSet(context, log, attributes);
		context.log("Creatng the movie frames");
		//remove the unselected map data
		newMapVisData.getSelectedMapNames(cBoxValue);

		// create MapDb database 
		File pathName = new File(path);
		File fileName = null;
		fileName = File.createTempFile("LogReplay", ".db", pathName);
		MapDBDatabase mapDb = new MapDBDatabase(fileName);
		DB db = mapDb.createDB();

		history.build(context, newMapVisData, fileName, db, mapDb, referCheckBox.isSelected(), true);
		context.log("Processing completed!");
	//	context.log("committing the movie into database");
		//db.commit();
		
		history.setAttributes(attributes);
		VisData visData = new VisData(history, newMapVisData, attributes, TimeStampArray.getInstance());
		System.out.println(System.currentTimeMillis() - start);	
		return visData;
	}

	private static SortedSet<String> getAttributesSet(XLog log) {
		TreeSet<String> attributeNames = new TreeSet<String>();
		for (XTrace trace : log) {
			attributeNames.addAll(trace.getAttributes().keySet());
			for (XEvent event : trace)
				attributeNames.addAll(event.getAttributes().keySet());
		}
		return attributeNames;
	}

	private static JCheckBox[] mapCheckBox;
	private static JButton chooserButton;
	private static JFileChooser chooser;
	private static String path = System.getProperty("user.home") + "\\" + "Documents";
	private static JLabel dirPath;
	private static File workingDirectory = null;
	private static JCheckBox referCheckBox = SlickerFactory.instance().createCheckBox("", true);
	private static JCheckBox useDataBase = SlickerFactory.instance().createCheckBox("", true);

}
