package org.processmining.plugins.mapvis.exportdata;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.zip.GZIPOutputStream;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.mapvis.VisData;
import org.processmining.plugins.mapvis.replay.NameLookUp;
import org.processmining.plugins.mapvis.replay.ProcessIDData;
import org.processmining.plugins.mapvis.replay.TimeStampArray;
import org.processmining.plugins.mapvis.visualize.workitem.AbstractWorkDotColorSchema;

@Plugin(name = "Export replayed visualization", parameterLabels = { "VisData", "File" }, returnLabels = {}, returnTypes = {}, userAccessible = true)
@UIExportPlugin(description = "Replayed log files", extension = "xmrplgz")
public class VisDataExporter {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "A. Erdodi, M. de Leoni, N.Gupta", email = "atilla.erdodi@gmail.com; m.d.leoni@tue.nl; n.gupta@tue.nl")
	@PluginVariant(requiredParameterLabels = { 0, 1 }, variantLabel = "Export replayed visualization")
	public void export(UIPluginContext context, VisData visData, File file) throws IOException {

		ObjectOutputStream obj_out = null;
		visData.closeDb();
		FileOutputStream out = new FileOutputStream(file);
		GZIPOutputStream gz = new GZIPOutputStream(out);
		try {
			obj_out = new ObjectOutputStream(gz);
			obj_out.writeObject(visData);
			obj_out.writeObject(NameLookUp.getInstance());
			obj_out.writeObject(ProcessIDData.getInstance());
			obj_out.writeObject(AbstractWorkDotColorSchema.getInstance());
			obj_out.writeObject(TimeStampArray.getInstance());
			
		} finally {
			if (obj_out != null)
				try {
					obj_out.close();
					gz.finish();
					System.out.println("Zip Done");
				} catch (IOException ioe) {
					ioe.printStackTrace();
				}
		}
	}
}
