package org.processmining.plugins.mapvis.importdata.visualization;

import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.processmining.contexts.uitopia.annotations.UIImportPlugin;
import org.processmining.framework.abstractplugins.AbstractImportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

@Plugin(name = "Open Map Visualization File", parameterLabels = { "Filename" }, returnLabels = { "MapVis" }, returnTypes = { MapVisData.class })
@UIImportPlugin(description = "Map visualization specification files", extensions = { "xmvs" })
public class ImportMapVisData extends AbstractImportPlugin {

	protected MapVisData importFromStream(PluginContext context, InputStream input, String filename,
			long fileSizeInBytes) throws Exception {
		try {

			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document dom = db.parse(input);
			Element root = dom.getDocumentElement();

			MapVisData mapVisData = new MapVisData();

			Element taskVarsRoot = (Element) root.getElementsByTagName("task_variables").item(0);
			NodeList taskVars = taskVarsRoot.getElementsByTagName("variable");
			for (int i = 0; i < taskVars.getLength(); i++) {
				Element taskVar = (Element) taskVars.item(i);
				mapVisData.addTaskVarName(taskVar.getAttribute("name"));
			}

			Element mapsRoot = (Element) root.getElementsByTagName("maps").item(0);
			NodeList maps = mapsRoot.getElementsByTagName("map");
			for (int i = 0; i < maps.getLength(); i++) {
				Element map = (Element) maps.item(i);
				String mapName = map.getAttribute("name");
				String mapURL = map.getElementsByTagName("url").item(0).getTextContent();

				NodeList nl = map.getElementsByTagName("axis");

				if (nl.getLength() > 0) {
					for (int x = 0; x < nl.getLength(); x++) {
						Element axis = (Element) nl.item(x);
						String xAxis = axis.getElementsByTagName("xAxis").item(0).getTextContent();
						String yAxis = axis.getElementsByTagName("yAxis").item(0).getTextContent();
						mapVisData.addMap(mapName, mapURL, xAxis, yAxis);
						mapVisData.setOldMap(false);
					}
				} else{
					mapVisData.setOldMap(true);
					mapVisData.addOldMap(mapName, mapURL);
				}

				Element queriesRoot = (Element) map.getElementsByTagName("queries").item(0);
				NodeList queries = queriesRoot.getElementsByTagName("query");
				for (int j = 0; j < queries.getLength(); j++) {
					Element query = (Element) queries.item(j);
					String taskName = query.getAttribute("task");
					String xqueryString;

					if (nl.getLength() > 0) {
						NodeList nodes = query.getElementsByTagName("attributes");
						Element attributesNodes = (Element) nodes.item(0);
						NodeList attributes = attributesNodes.getElementsByTagName("attribute");

						for (int k = 0; k < attributes.getLength(); k++) {
							Element attribute = (Element) attributes.item(k);
							String x = attribute.getElementsByTagName("x").item(0).getTextContent();
							String y = attribute.getElementsByTagName("y").item(0).getTextContent();
							xqueryString = attribute.getElementsByTagName("xquery").item(0).getTextContent();
							mapVisData.getMap(mapName).addImportQuery(taskName, xqueryString, x, y);
						}
					} else {
						xqueryString = query.getElementsByTagName("xquery").item(0).getTextContent();
						mapVisData.getMap(mapName).addOldQuery(taskName, xqueryString);
					}
				}
			}

			return mapVisData;

		} catch (ParserConfigurationException pce) {
			pce.printStackTrace();
		} catch (SAXException se) {
			se.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}

		return null;
	}
}
