package org.processmining.plugins.mapvis.importdata.visualization;

import java.io.Serializable;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.TreeSet;

public class MapVisData implements Serializable, Cloneable {

	private static final long serialVersionUID = 1L;
	private HashMap<String, MapVisMap> maps;
	private Set<String> varNames;
	private boolean isOldMap = false;

	public String test() {
		return String.valueOf(maps.size());
	}

	public MapVisData() {
		maps = new HashMap<String, MapVisMap>();
		varNames = new TreeSet<String>();
	}

	public void addTaskVarName(String varName) {
		varNames.add(varName);
	}

	public Set<String> getTaskVarNames() {
		return varNames;
	}

	public void addMap(String name, String url, String xAxis, String yxis) {
		MapVisMap map = new MapVisMap(name, url, xAxis, yxis);
		map.setOldMap(false);
		maps.put(name, map);
	}

	public void addOldMap(String name, String url) {
		MapVisMap map = new MapVisMap(name, url);
		map.setOldMap(true);
		maps.put(name, map);
	}

	public MapVisMap getMap(String name) {
		return maps.get(name);
	}

	public Collection<String> getMapNames() {
		return maps.keySet();
	}

	public Collection<String> getSelectedMapNames(Map<String, Boolean> selectedMaps) {
		for (Entry<String, Boolean> entry : selectedMaps.entrySet()) {
			if (entry.getValue().equals(false)) {
				maps.remove(entry.getKey());
			}
		}
		return maps.keySet();
	}

	public void merge(MapVisData mapVisData2, String postfix) {
		for (String varName : mapVisData2.getTaskVarNames()) {
			varNames.add(varName);
		}
		for (String mapName : mapVisData2.getMapNames()) {
			MapVisMap newMap = mapVisData2.getMap(mapName);

			String x = newMap.getXAxis();
			String y = newMap.getYAxis();
			if (x == null && y == null) {
				newMap.setOldMap(true);
			} else {
				newMap.setOldMap(false);
			}
			newMap.setName(mapName + postfix);
			maps.put(mapName + postfix, newMap);
		}
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		MapVisData cloned = (MapVisData) super.clone();
		cloned.maps = new HashMap<String, MapVisMap>();
		for (Entry<String, MapVisMap> e : maps.entrySet()) {
			cloned.maps.put(e.getKey(), e.getValue());
		}

		for (String varName : varNames) {
			cloned.varNames.add(varName);
		}
		return cloned;
	}

	public String toString() {
		StringBuffer retVal = new StringBuffer();
		retVal.append("<specification>\n");
		retVal.append("  <task_variables></task_variables>\n");
		retVal.append("  <maps>\n");
		for (String mapName : getMapNames()) {
			retVal.append("    <map name=\"" + mapName + "\">\n");
			retVal.append("      <url>" + getMap(mapName).getUrl().trim() + "</url>\n");
			retVal.append("      <resources/>\n");
			if (!getMap(mapName).isOldMap()) {
				retVal.append("  	 <axis>\n");
				retVal.append("  	 	<xAxis>" + getMap(mapName).getXAxis() + "</xAxis>\n");
				retVal.append("   	 	<yAxis>" + getMap(mapName).getYAxis() + "</yAxis>\n");
				retVal.append("  	</axis>\n");
				retVal.append("      <queries>\n");
				for (String queryName : getMap(mapName).getQueryNames()) {
					retVal.append("        <query task=\"" + queryName + "\">\n");
					retVal.append("          <attributes>\n");
					for (Entry<AttributeKey, String> attribute : getMap(mapName).getAttributeQuery(queryName)
							.entrySet()) {
						retVal.append("          	<attribute>\n");
						retVal.append("          		<x>" + attribute.getKey().getxAxis() + "</x>\n");
						retVal.append("          		<y>" + attribute.getKey().getyAxis() + "</y>\n");
						retVal.append("          		<xquery>\n");
						retVal.append("           			<![CDATA[" + attribute.getValue().trim() + "]]>\n");
						retVal.append("          		</xquery>\n");
						retVal.append("          	</attribute>\n");
					}
					retVal.append("        </attributes>\n");
					retVal.append("       </query>\n");
				}
			} else {
				retVal.append("      <queries>\n");
				for (String queryName : getMap(mapName).getOldQueryNames()) {
					retVal.append("        <query task=\"" + queryName + "\">\n");
					retVal.append("          <xquery>\n");
					retVal.append("            <![CDATA[" + getMap(mapName).getQueryString(queryName).trim() + "]]>\n");
					retVal.append("          </xquery>\n");
					retVal.append("       </query>\n");
				}
			}
			retVal.append("      </queries>\n");
			retVal.append("    </map>\n");
		}
		retVal.append("  </maps>\n");
		retVal.append("</specification>\n");
		return retVal.toString();
	}

	public boolean isOldMap() {
		return isOldMap;
	}

	public void setOldMap(boolean isOldMap) {
		this.isOldMap = isOldMap;
	}
}
