package org.processmining.plugins.mapvis.importdata.visualization;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;

import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.util.ui.widgets.ProMSplitPane;

import com.fluxicon.slickerbox.factory.SlickerDecorator;

class MyTreeRenderer extends DefaultTreeCellRenderer {
	private static final long serialVersionUID = 1L;

	public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded,
			boolean leaf, int row, boolean hasFocus) {
		if (value instanceof LeafDefaultMutableTreeNode)
			return super.getTreeCellRendererComponent(tree, value, selected, expanded, true, row, hasFocus);
		else
			return super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);
	}
}

class LeafDefaultMutableTreeNode extends DefaultMutableTreeNode {
	private static final long serialVersionUID = 1L;

	public LeafDefaultMutableTreeNode(String queryName) {
		super(queryName);
	}

}

@Plugin(name = "Visualization of Map Data ", 
returnLabels = { "Visualized Log-Model Alignment Projection to Log" }, 
returnTypes = { JComponent.class }, parameterLabels = { "MapVisData" }, userAccessible = true)
@Visualizer
public class MapVisDataVisualizer {
	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualize(PluginContext context, MapVisData mapVis) {
		ProMSplitPane visPanel = new ProMSplitPane();
		Font orig = visPanel.getFont();
		DefaultMutableTreeNode root = new DefaultMutableTreeNode("Maps");
		DefaultTreeCellRenderer renderer = new MyTreeRenderer();
		JTree tree = new JTree(root);

		ImageIcon icon = new ImageIcon(this.getClass().getResource("map.png"));
		ImageIcon leafIcon = new ImageIcon(this.getClass().getResource("query.png"));
		
		renderer.setOpenIcon(scale(icon, 0.2, tree));
		renderer.setClosedIcon(scale(icon, 0.2, tree));

		renderer.setLeafIcon(scale(leafIcon, 0.2, tree));
		renderer.setFont(orig.deriveFont(orig.getSize() * 1.3F));

		tree.setCellRenderer(renderer);

		JScrollPane vis1 = new JScrollPane(tree);
		SlickerDecorator.instance().decorate(vis1, Color.WHITE, Color.BLACK, Color.BLACK);
		visPanel.add(vis1);
		
		for (String mapName : mapVis.getMapNames()) {
			MapVisMap policy = mapVis.getMap(mapName);
			DefaultMutableTreeNode mapNode = new DefaultMutableTreeNode(mapName + "(" + policy.getUrl() + ")");

			for (String queryName : policy.getQueryNames()) {
				DefaultMutableTreeNode queryNode = new DefaultMutableTreeNode(queryName);
				Map<AttributeKey, String> attrQuery = policy.getAttributeQuery(queryName);
				if (attrQuery==null)
				{
					String expr = policy.getQueryString(queryName);

					String queryString = "<html>" + expr + "</html>";
					queryString = queryString.replaceAll("\\{", "").replaceAll("\\}", "")
							.replaceAll("</?coordinate>", "");
					queryString = queryString.replaceAll("<z>.*</z>", "")
							.replaceAll("<x>", "<p><font color=\"#FF0000\">X</font>: ")
							.replaceAll("<y>", "<p><font color=\"#FF0000\">Y</font>: ").replaceAll("</[x-y]>", "</p>");
					String[] operators = { "if", "then", "else", "number", "div", "\\(", "\\)", "\\+", "\\-", "\\*" };
					for (String operator : operators) {
						queryString = queryString.replaceAll(operator + "(\\W)", "<b>" + operator + "$1 </b>");
					}
					queryNode.add(new LeafDefaultMutableTreeNode(queryString));
				
				}
				else
				{
					for (Entry<AttributeKey, String> attribute : attrQuery.entrySet()) {
						DefaultMutableTreeNode queryAttribute = new LeafDefaultMutableTreeNode("Attributes");

						String xAxis = attribute.getKey().getxAxis();
						String yAxis = attribute.getKey().getyAxis();

						String query = attribute.getValue();

						String queryString = "<html>" + query + "</html>";
						queryString = queryString.replaceAll("\\{", "").replaceAll("\\}", "")
								.replaceAll("</?coordinate>", "");
						queryString = queryString.replaceAll("<z>.*</z>", "")
								.replaceAll("<x>", "<p><font color=\"#FF0000\">X</font>: ")
								.replaceAll("<y>", "<p><font color=\"#FF0000\">Y</font>: ").replaceAll("</[x-y]>", "</p>");
						String[] operators = { "if", "then", "else", "number", "div", "\\(", "\\)", "\\+", "\\-", "\\*" };
						for (String operator : operators) {
							queryString = queryString.replaceAll(operator + "(\\W)", "<b>" + operator + "$1 </b>");
						}

						queryAttribute.add(new LeafDefaultMutableTreeNode(xAxis));
						queryAttribute.add(new LeafDefaultMutableTreeNode(yAxis));
						queryAttribute.add(new LeafDefaultMutableTreeNode(queryString));

						queryNode.add(queryAttribute);
					}
				}
				mapNode.add(queryNode);

			}
			root.add(mapNode);
		}
		expandJTreeNode(tree, tree.getModel(), root, 0, 0);
		return visPanel;
	}

	private Icon scale(ImageIcon icon, double scale, Component component) {
		Image src = icon.getImage();
		int w = (int) (scale * src.getWidth(component));
		int h = (int) (scale * src.getHeight(component));
		int type = BufferedImage.TYPE_INT_RGB;
		BufferedImage dst = new BufferedImage(w, h, type);
		Graphics2D g2 = dst.createGraphics();
		g2.drawImage(src, 0, 0, w, h, component.getBackground(), component);
		g2.dispose();
		return new ImageIcon(dst);
	}

	public static int expandJTreeNode(javax.swing.JTree tree, javax.swing.tree.TreeModel model, Object node, int row,
			int depth) {
		if (node != null && !model.isLeaf(node)) {
			tree.expandRow(row);
			if (depth != 0) {
				for (int index = 0; row + 1 < tree.getRowCount() && index < model.getChildCount(node); index++) {
					row++;
					Object child = model.getChild(node, index);
					if (child == null)
						break;
					javax.swing.tree.TreePath path;
					while ((path = tree.getPathForRow(row)) != null && path.getLastPathComponent() != child)
						row++;
					if (path == null)
						break;
					row = expandJTreeNode(tree, model, child, row, depth - 1);
				}
			}
		}
		return row;
	}
}
