package org.processmining.plugins.mapvis.replay;

import java.awt.Color;
import java.io.File;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension.StandardModel;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.mapdb.BTreeKeySerializer;
import org.mapdb.DB;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.plugins.mapvis.MapDBDatabase;
import org.processmining.plugins.mapvis.importdata.visualization.MapVisData;
import org.processmining.plugins.mapvis.visualize.workitem.AbstractWorkItemDot;
import org.processmining.plugins.mapvis.visualize.workitem.AbstractWorkItemDotSerializer;

public class History implements java.io.Serializable {

	private static final long serialVersionUID = 1L;
	public static int MAX_NUM_ATTR_OCCUR = 10;
	public static Color[] mainColors = { Color.GRAY, Color.YELLOW, Color.ORANGE, Color.RED, Color.GREEN, Color.PINK,
			new Color(160, 32, 240), Color.MAGENTA, Color.CYAN, Color.WHITE, new Color(147, 148, 19) };

	//Sort the main colour list
	static {
		Arrays.sort(mainColors, new Comparator<Color>() {
			public int compare(Color o1, Color o2) {
				return Integer.compare(o1.getRGB(), o2.getRGB());
			}

		});
	}

	public String[] attributes = null;
	private transient List<Timestamped> eventSet;
	private int numStates;
	private Map<String, Map<Integer, ArrayList<AbstractWorkItemDot>>> movies; // mapname -> (array of workitemlists)
	private HashMap<String, LinkedList<FrequentValueEntry>> mostFrequentPerAttribute = new HashMap<String, LinkedList<FrequentValueEntry>>();
	private long numConcludedActivity = 0;
	private int numTraces;
	private XLog log;
	private String[] maps = null;
	private File fileName;
	private boolean userDatabase;
	private transient DB db;
	private transient MapDBDatabase mapDb;

	public void reset() {
		mostFrequentPerAttribute.clear();
	}

	public static Object getValue(XAttribute attribute) {
		if (attribute instanceof XAttributeBoolean)
			return ((XAttributeBoolean) attribute).getValue();
		else if (attribute instanceof XAttributeContinuous)
			return ((XAttributeContinuous) attribute).getValue();
		else if (attribute instanceof XAttributeDiscrete)
			return ((XAttributeDiscrete) attribute).getValue();
		else if (attribute instanceof XAttributeLiteral)
			return ((XAttributeLiteral) attribute).getValue();
		else if (attribute instanceof XAttributeTimestamp)
			return ((XAttributeTimestamp) attribute).getValue().getTime();
		else
			return null;
	}

	public void buildEventSet(UIPluginContext context, XLog log, String attributeNames[]) {
		// build eventSet
		eventSet = new ArrayList<Timestamped>();
		numTraces = log.size();
		this.log = log;

		ProcessIDData.getInstance().initialProcessIdData();

		for (String attributeName : attributeNames) {
			ProcessIDData.getInstance().createNumOccurrenceAttributes(attributeName);
			ProcessIDData.getInstance().createProcessIdValuePerAttribute(attributeName);
			ProcessIDData.getInstance().createProcessIdColorPerAttribute(attributeName);
			mostFrequentPerAttribute.put(attributeName, new LinkedList<FrequentValueEntry>());
		}

		int numTrace = 1;
		// iterating through the traces in a log
		for (XTrace xTrace : log) {
			String processId = xTrace.getAttributes().get("concept:name").toString();
			context.getProgress().setValue(((numTrace++) * 10) / log.size());
			XAttributeMap attributes = (XAttributeMap) xTrace.getAttributes().clone();
			// iterating though the events in a trace
			Iterator<XEvent> iter = xTrace.iterator();
			while (iter.hasNext()) {

				// Max/min search for trace attributes
				XEvent xEvent = iter.next();
				for (XAttribute attr : xEvent.getAttributes().values()) {
					attributes.put(attr.getKey(), attr);
				}

				Date timestamp = XTimeExtension.instance().extractTimestamp(xEvent);
				String activity = XConceptExtension.instance().extractName(xEvent);
				// set sum of execution time per event

				//numConcludedInstancesPerActivity.put(activity, 0);
				ProcessIDData.getInstance().setNumConcludedInstancesPerActivity(activity, 0);
				ProcessIDData.getInstance().setMeanExecutionTimePerActivity(activity, 0L);
				//meanExecutionTimePerActivity.put(activity, 0L);
				StandardModel standardModel = XLifecycleExtension.instance().extractStandardTransition(xEvent);
				ActivityState activityState = new ActivityState();
				switch (standardModel) {
					case ASSIGN :
						activityState.setProgressState(ActivityState.ASSIGNED);
						break;
					case ATE_ABORT :
						activityState.setProgressState(ActivityState.ABORTED);
						break;
					case AUTOSKIP :
						activityState.setProgressState(ActivityState.ABORTED);
						break;
					case COMPLETE :
						activityState.setProgressState(ActivityState.CONCLUDED);
						break;
					case MANUALSKIP :
						activityState.setProgressState(ActivityState.ABORTED);
						break;
					case PI_ABORT :
						activityState.setProgressState(ActivityState.ABORTED);
						break;
					case REASSIGN :
						activityState.setProgressState(ActivityState.ASSIGNED);
						break;
					case RESUME :
						activityState.setProgressState(ActivityState.SCHEDULED);
						break;
					case SCHEDULE :
						activityState.setProgressState(ActivityState.SCHEDULED);
						break;
					case START :
						activityState.setProgressState(ActivityState.STARTED);
						break;
					case SUSPEND :
						activityState.setProgressState(ActivityState.SUSPENDED);
						break;
					case UNKNOWN :
						break;
					case WITHDRAW :
						activityState.setProgressState(ActivityState.ABORTED);
						break;
					default :
						break;
				}

				Event event = new Event();
				event.setTimestamp(timestamp);
				event.setActivityName(activity);
				event.setCaseId(processId);
				event.setState(activityState);
				event.setLastTraceIndex(!iter.hasNext());

				Boolean numResult;
				for (String attributeName : attributeNames) {
					XAttribute attrValueObject = attributes.get(attributeName);
					if (attrValueObject != null) {
						if (attrValueObject instanceof XAttributeContinuous
								|| attrValueObject instanceof XAttributeDiscrete)
							numResult = true;
						else
							numResult = false;

						ProcessIDData.getInstance().setProcessAttributeType(attributeName, numResult);

						String attrValue = getValue(attrValueObject).toString();

						int processIdVar = NameLookUp.getInstance().getValueByCaseId(processId);
						Map<Integer, String> processIdValue = ProcessIDData.getInstance().getProcessIdPerAttribute(
								attributeName);
						if (attrValue != null && processIdValue.get(processIdVar) == null) {

							ProcessIDData.getInstance().setProcessIdValuePerAttribute(attributeName, processIdVar,
									attrValue);
							Map<String, Integer> numOccurrenceAttribut = ProcessIDData.getInstance()
									.getNumOccurrenceAttributes(attributeName);
							Integer occurrence = numOccurrenceAttribut.get(attrValue);
							if (occurrence == null)
								occurrence = 1;
							else
								occurrence++;

							ProcessIDData.getInstance().createNumOccurrencePerAttribute(attributeName, attrValue,
									occurrence);
						}
					}
				}

				LinkedList<Object[]> vars = event.getVars();

				// process event attributes 
				for (String name : attributes.keySet()) {
					XAttribute attribute = attributes.get(name);
					String varName = name.replace(":", "_");
					Object value = getValue(attribute);
					vars.add(new Object[] { varName, value });
				}

				eventSet.add(event);
				attributes.clear();
			}
		}
		// get number of states present in the dataset
		numStates = eventSet.size();

		int index;
		for (String attributeName : attributeNames) {
			Map<String, Integer> numOccurrenceAttribut = ProcessIDData.getInstance().getNumOccurrenceAttributes(
					attributeName);
			LinkedList<FrequentValueEntry> mostFrequent = mostFrequentPerAttribute.get(attributeName);
			for (Entry<String, Integer> entr : numOccurrenceAttribut.entrySet()) {
				index = 0;
				ListIterator<FrequentValueEntry> iter = mostFrequent.listIterator();
				int size = mostFrequent.size();
				while (index < MAX_NUM_ATTR_OCCUR - 1) {
					if (iter.hasNext()) {
						if (iter.next().getOccurrence() <= entr.getValue()) {
							iter.previous();
							iter.add(new FrequentValueEntry(entr.getKey(), entr.getValue()));
							if (mostFrequent.size() > MAX_NUM_ATTR_OCCUR)
								mostFrequent.removeLast();
							break;
						}
					} else {
						if (size < MAX_NUM_ATTR_OCCUR) {
							iter.add(new FrequentValueEntry(entr.getKey(), entr.getValue()));
							break;
						}
					}
					index++;
				}
			}

			// sort the FrequentValueEntry list
			Collections.sort(mostFrequent, null);

			index = 0;
			Map<Integer, String> processIdValue = ProcessIDData.getInstance().getProcessIdPerAttribute(attributeName);
			for (Entry<Integer, String> entry : processIdValue.entrySet()) {
				int pos = 0;
				for (FrequentValueEntry item : mostFrequent) {
					if (entry.getValue().equals(item.getVarName()))
						break;
					else
						pos++;
				}

				ProcessIDData.getInstance().setProcessIdColorPerAttribute(attributeName, entry.getKey(),
						mainColors[pos]);
			}
		}

	}

	public void build(UIPluginContext context, MapVisData mapVisData, File fileName, DB db, MapDBDatabase mapDb,
			boolean referTime, boolean userDatabase) throws InterruptedException {
		numStates = eventSet.size();
		this.fileName = fileName;
		this.db = db;
		this.userDatabase = userDatabase;

		TimeStampArray.getInstance().setArraySize(numStates);
		maps = new String[mapVisData.getMapNames().size()];

		int i = 0;
		movies = new ConcurrentHashMap<String, Map<Integer, ArrayList<AbstractWorkItemDot>>>();
		if (userDatabase) {

			for (String mapName : mapVisData.getMapNames()) {
				Map<Integer, ArrayList<AbstractWorkItemDot>> movie = this.db.treeMapCreate(mapName)
						.keySerializer(BTreeKeySerializer.INTEGER).valuesOutsideNodesEnable()
						.valueSerializer(new AbstractWorkItemDotSerializer()).make();
				movies.put(mapName, movie);
				maps[i++] = mapName;
			}
		} else {

			for (String mapName : mapVisData.getMapNames()) {
				Map<Integer, ArrayList<AbstractWorkItemDot>> movie = new HashMap<Integer, ArrayList<AbstractWorkItemDot>>();
				movies.put(mapName, movie);
				maps[i++] = mapName;
			}

		}
		// sort events 
		Collections.sort(eventSet);

		int threadCount = 0;
		Thread[] threadPool = new Thread[mapVisData.getMapNames().size()];

		for (String mapName : mapVisData.getMapNames()) {
			Thread task = new ReplayTask(eventSet, mapName, movies.get(mapName), context, mapVisData.getTaskVarNames(),
					mapVisData.getMap(mapName), referTime);
			threadPool[threadCount++] = task;
			task.start();
		}

		for (threadCount = 0; threadCount < threadPool.length; threadCount++)
			threadPool[threadCount].join();
	}

	public List<AbstractWorkItemDot> getWorkItemDots(String mapName, int i) {
		try {
			if (userDatabase) {
				if (db.isClosed()) {
					MapDBDatabase mapDb = new MapDBDatabase(fileName);
					db = mapDb.createDB();
					this.movies = db.treeMap("movies");
					for (String name : maps) {
						Map<Integer, ArrayList<AbstractWorkItemDot>> movie = db.treeMap(name);
						movies.put(name, movie);
					}
				}
				if (!movies.get(mapName).get(i).isEmpty()) {
					return movies.get(mapName).get(i);
				}
			} else if (!movies.get(mapName).get(i).isEmpty()) {
				return movies.get(mapName).get(i);
			}
			return null;
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
			return null;
		}
	}

	public int getNumStates() {
		return numStates;
	}

	public List<FrequentValueEntry> getMostFrequentValue(String attribute) {
		return mostFrequentPerAttribute.get(attribute);
	}

	public String[] getAttributes() {
		return attributes;
	}

	public void setAttributes(String[] attributes) {
		this.attributes = attributes;
	}

	public int getNumTraces() {
		return numTraces;
	}

	public XLog getLogFile() {
		return this.log;
	}

	public void closeDB() {
		this.db.commit();
		this.db.close();
	}

	private void writeObject(ObjectOutputStream out) {
		try {
			String movieFile = fileName.getPath();
			out.writeInt(MAX_NUM_ATTR_OCCUR);
			out.writeInt(numStates);
			out.writeInt(numTraces);
			out.writeLong(numConcludedActivity);
			out.writeObject(mainColors);
			out.writeObject(mostFrequentPerAttribute);
			out.writeObject(attributes);
			out.writeUTF(movieFile);
			out.writeBoolean(userDatabase);
			out.writeObject(maps);
			if (!userDatabase) {
				out.writeObject(movies);
			}
		} catch (Exception e) {
			// TODO: handle exception
		}
	}

	@SuppressWarnings({ "unchecked" })
	private void readObject(ObjectInputStream in) {
		try {
			MAX_NUM_ATTR_OCCUR = in.readInt();
			numStates = in.readInt();
			numTraces = in.readInt();
			numConcludedActivity = in.readLong();
			mainColors = (Color[]) in.readObject();
			mostFrequentPerAttribute = (HashMap<String, LinkedList<FrequentValueEntry>>) in.readObject();
			attributes = (String[]) in.readObject();
			String movieFile = in.readUTF();
			fileName = new File(movieFile);

			userDatabase = in.readBoolean();
			maps = (String[]) in.readObject();
			if (userDatabase) {
				this.mapDb = new MapDBDatabase(fileName);
				this.db = mapDb.createDB();
				this.movies = db.treeMap("movies");
				for (String mapName : maps) {
					Map<Integer, ArrayList<AbstractWorkItemDot>> movie = db.treeMap(mapName);
					movies.put(mapName, movie);
				}
			} else {
				movies = (Map<String, Map<Integer, ArrayList<AbstractWorkItemDot>>>) in.readObject();
			}
		} catch (Exception e) {
			// TODO: handle exception
		}
	}
}
