package org.processmining.plugins.mapvis.replay;

import java.awt.Color;
import java.io.Serializable;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

public class ProcessIDData implements Serializable {

	private static final long serialVersionUID = 1L;
	private static ProcessIDData Instance = null;;
	private Map<String, Map<Integer, String>> processIdValuePerAttribute =null;
	private Map<String, Map<Integer, Color>> processIdColorPerAttribute =null;
	private Map<String, Map<String, Integer>> numOccurrenceAttributes = null;
	private Map<String, Long> meanExecutionTimePerActivity = new TreeMap<String, Long>();
	private Map<String, Boolean> processAttributeType = new HashMap<String, Boolean>();
	private Map<String, Integer> numConcludedInstancesPerActivity = new HashMap<String, Integer>();
	private long meanExecutionTime = 0;
	private long numConcludedActivity = 0;

	public static ProcessIDData getInstance() {
		if (Instance == null)
			Instance = new ProcessIDData();
		return Instance;
	}
	
	public static void setProcessIDData(ProcessIDData newInstance) {
		Instance = newInstance;
	}

	public Map<String, Boolean> getProcessAttributeType() {
		return this.processAttributeType;
	}
	
	public  Boolean getProcessAttributeNameType(String attributeName) {
		return this.processAttributeType.get(attributeName);
	}

	public void setProcessAttributeType(String attributeName, Boolean attributeType) {
		this.processAttributeType.put(attributeName, attributeType);
	}

	public  Long getMeanExecutionTimePerActivity(String activity) {
		return this.meanExecutionTimePerActivity.get(activity);
	}

	public void setMeanExecutionTimePerActivity(String key, Long value) {
		this.meanExecutionTimePerActivity.put(key, value);
	}

	public Map<Integer, String> getProcessIdAttribute(String attribute) {
		return this.processIdValuePerAttribute.get(attribute);
	}
	
	public String getProcessIdForAttribute(String attribute, Integer caseID) {
		return this.processIdValuePerAttribute.get(attribute).get(caseID);
	}

	public void setProcessIdValuePerAttribute(String attribute, Integer processIdVar, String attrValue) {
		this.processIdValuePerAttribute.get(attribute).put(processIdVar,attrValue);
	}
	
	public void createProcessIdValuePerAttribute(String attribute) {
		this.processIdValuePerAttribute.put(attribute, new HashMap<Integer, String>());
	}

	public void createNumOccurrenceAttributes(String attribute) {
		this.numOccurrenceAttributes.put(attribute, new HashMap<String, Integer>());
	}
	
	public void createNumOccurrencePerAttribute(String attribute, String attributeValue, Integer count) {
		this.numOccurrenceAttributes.get(attribute).put(attributeValue,count);
	}
	
	public Map<String, Integer> getNumOccurrenceAttributes(String attribute) {
		return this.numOccurrenceAttributes.get(attribute);
	}
	
	public Collection<String> getActivityNames() {
		return this.meanExecutionTimePerActivity.keySet();
	}

	public long getMeanExecutionTime() {
		return this.meanExecutionTime;
	}

	public void setMeanExecutionTime(long meanExecutionTime) {
		this.meanExecutionTime = meanExecutionTime;
	}

	public Map<Integer, Color> getProcessIdColor(String attribute) {
		return this.processIdColorPerAttribute.get(attribute);
	}

	public void initialProcessIdData() {
		processIdColorPerAttribute = new HashMap<String, Map<Integer, Color>>();
		processIdValuePerAttribute = new HashMap<String, Map<Integer, String>>();
		numOccurrenceAttributes = new HashMap<String, Map<String, Integer>>();
		//relativeAttributeData = new HashMap<Integer,Map<String,Integer>>();
	}
	
	public void createProcessIdColorPerAttribute(String attribute) {
		this.processIdColorPerAttribute.put(attribute, new HashMap<Integer, Color>());
	}
	
	public Map<Integer, Color> getProcessIdColorPerAttribute(String attribute) {
		return this.processIdColorPerAttribute.get(attribute);
	}
	
	public Map<Integer, String> getProcessIdPerAttribute(String attribute) {
		return this.processIdValuePerAttribute.get(attribute);
	}
	
	public void setProcessIdColorPerAttribute(String attribute, Integer processIdVar, Color color) {
		this.processIdColorPerAttribute.get(attribute).put(processIdVar,color);
	}

	public SortedSet<String> getAttributeFilterValues(String attributeFilter) {
		SortedSet<String> attrValue = new TreeSet<String>();
		if (attributeFilter != null) {
			for (Entry<Integer, String> entry : processIdValuePerAttribute.get(attributeFilter).entrySet()) {
				if (!attrValue.contains(entry.getValue())) {
					attrValue.add(entry.getValue());
				}
			}
		}
		return attrValue;
	}

	public Integer getNumConcludedInstancesPerActivity(String activity) {
		return this.numConcludedInstancesPerActivity.get(activity);
	}

	public void setNumConcludedInstancesPerActivity(String activity , int value) {
		this.numConcludedInstancesPerActivity.put(activity, value);
	}

	public long getNumConcludedActivity() {
		return this.numConcludedActivity;
	}

	public void setNumConcludedActivity(long numConcludedActivity) {
		this.numConcludedActivity = numConcludedActivity;
	}

	/*public void createTimeFramePerRelativeAttribute(int timeframe) {
		this.relativeAttributeData.put(timeframe, new HashMap<String,Integer>());
	}
	
	public void setRelativeAttributeData(int timeframe,String attributeName, Integer count) {
		if(relativeAttributeData.containsKey(timeframe)){
			this.relativeAttributeData.get(timeframe).put(attributeName, count);
		}else{
			this.relativeAttributeData.put(timeframe, new HashMap<String,Integer>());
			this.relativeAttributeData.get(timeframe).put(attributeName, count);
		}
	}
	
	public int getRelativeAttributeDataCount(int timeframe,String attributeName) {
		return this.relativeAttributeData.get(timeframe).get(attributeName);
	}
	
	public Map<String,Integer> getRelativeAttribute(int timeframe) {
		 Map<String,Integer> data = this.relativeAttributeData.get(timeframe);
		return data;
	}*/
	
	
}
