package org.processmining.plugins.mapvis.replay;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import net.sf.saxon.query.XQueryExpression;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.plugins.mapvis.importdata.visualization.AttributeKey;
import org.processmining.plugins.mapvis.importdata.visualization.MapVisMap;
import org.processmining.plugins.mapvis.visualize.workitem.AbstractWorkItemDot;
import org.processmining.plugins.mapvis.visualize.workitem.HiddenWorkItemDot;
import org.processmining.plugins.mapvis.visualize.workitem.InvalidWorkItemDot;
import org.processmining.plugins.mapvis.visualize.workitem.WorkItemDot;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class ReplayTask extends Thread {

	private org.processmining.plugins.mapvis.replay.State state;
	private static XQueryProcessor xp = XQueryProcessor.getInstance();
	private static DocumentBuilder builder;
	private String mapName;
	private Map<Integer, ArrayList<AbstractWorkItemDot>> movie;

	// it stores Abstract work item dot of previous frame of the movie for optimisation of the pick method 
	private Map<ActivityKey, AbstractWorkItemDot> preIndexmovies;
	private List<Timestamped> events;
	private UIPluginContext context;
	private Set<String> taskVarNames;
	private MapVisMap map;
	private boolean referTime;
	private boolean xAxis, yAxis;

	static {
		try {
			builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
	}

	public ReplayTask(List<Timestamped> events, String mapName, Map<Integer, ArrayList<AbstractWorkItemDot>> movie,
			UIPluginContext context, Set<String> taskVarNames, MapVisMap map, boolean referTime) {
		this.movie = movie;
		this.mapName = mapName;
		this.events = events;
		this.context = context;
		this.taskVarNames = taskVarNames;
		this.map = map;
		this.referTime = referTime;
		state = new org.processmining.plugins.mapvis.replay.State();
		preIndexmovies = new HashMap<ActivityKey, AbstractWorkItemDot>();
	}

	public void run() {
		Iterator<Timestamped> iter = events.iterator();
		int index = 0;

		//
		while (iter.hasNext()) {
			Timestamped timestamped = iter.next();
			// verify state of the current event
			applyEvent((AbsEvent) timestamped, state, taskVarNames, index);

			TimeStampArray.getInstance().setTime(index, timestamped.getTimestamp().getTime());
			TimeStampArray.getInstance().setNumActivityPerState(index, (short) state.domAlpha().size());

			ArrayList<AbstractWorkItemDot> workItemDotList = new ArrayList<AbstractWorkItemDot>(state.domAlpha().size());
			Map<ActivityKey, AbstractWorkItemDot> newPreviousFrame = new HashMap<ActivityKey, AbstractWorkItemDot>();

			for (ActivityKey key : state.domAlpha()) {

				AbstractWorkItemDot workItemDot = null;
				String caseID = key.getCaseId();
				String activityName = key.getActivityName();
				byte pState = state.alpha(key).getProgressState();

				WorkItemMeta metaData = state.getWorkItemMetaFor(key);
				long creationTime = metaData.getEnablementTime();

				if (creationTime == 0)
					creationTime = metaData.getAllocatedTime();
				if (creationTime == 0)
					creationTime = metaData.getStartedTime();

				//add trace id, event and time stamp
				workItemDot = pick(key, pState, creationTime);

				boolean isDynamicQuery = false;
				String queryString = null;
				AttributeKey attributeKey = null;

				Object xAxis = state.getVarName(caseID, map.getXAxis());
				Object yAxis = state.getVarName(caseID, map.getYAxis());
				if (xAxis != null && yAxis != null) {
					// only for Cartesian literal maps
					attributeKey = new AttributeKey(xAxis.toString(), yAxis.toString());
					queryString = map.getAttributeQueries(activityName, attributeKey);
				}

				XQueryExpression xQuery = null;
				// for rest of the maps
				if (!map.isOldMap() && (map.isTaskAvaiable(activityName) && queryString == null)) {
					attributeKey = new AttributeKey(activityName, activityName);
					queryString = map.getAttributeQueries(activityName, attributeKey);
					isDynamicQuery = true;
				}

				if (map.isOldMap()) {
					xQuery = map.getQuery(activityName);
					queryString = map.getQueryString(activityName);
					isDynamicQuery = true;
				}

				if (queryString != null) {
					try {
						boolean referAgain = referTime && timestamped.refer(caseID);
						if (workItemDot == null || referAgain || queryString.contains("/now")) {

							// check whether query is static or dynamic
							String result = "";
							if (queryString.contains("//")) {

								// when x  or y coordinates are present in a trace then run Xquery
								// when an event from same trace 
								if (isDynamicQuery && (this.xAxis || this.yAxis || referAgain)) {
									String stateXML = state.getXML(timestamped.getTimestamp(), activityName, caseID,
											key);
									if (!map.isOldMap()) {
										xQuery = map.getQuery(activityName, attributeKey);
									}
									result = xp.performXQuery(stateXML, xQuery);
								} else {
									result = getInvalidString();
								}
							} else {
								result = queryString;
							}

							workItemDot = createAbstractWorkItem(result, workItemDot, pState, creationTime, key);
						}
					} catch (Exception e) {
						e.printStackTrace();
						continue;
					}

				} else {
					if (workItemDot == null || !(workItemDot instanceof HiddenWorkItemDot)) {
						workItemDot = new HiddenWorkItemDot(pState, creationTime, key);
					}
				}
				assert (workItemDot != null);
				newPreviousFrame.put(key, workItemDot);
				workItemDotList.add(workItemDot);
			}

			preIndexmovies.putAll(newPreviousFrame);
			movie.put(index, workItemDotList);
			index++;

			synchronized (ReplayTask.class) {
				ReplayTask.class.notifyAll();
			}

			if (index % 1000 == 0) {

				context.log("Created movie frames for " + mapName + " state(s) " + index + " out of " + events.size()
						+ " states");
			}

			context.getProgress().setValue((index * 90) / events.size() + 10);
		}
	}

	/*
	 * create new work item dot for a corresponding event
	 */
	private AbstractWorkItemDot createAbstractWorkItem(String result, AbstractWorkItemDot workItemDot, byte pState,
			long creationTime, ActivityKey key) throws SAXException, IOException {
		StringReader sr = new StringReader(result);
		Document document = null;
		synchronized (builder) {
			document = builder.parse(new InputSource(sr));
		}

		Node node = document.getDocumentElement();
		short x = -1, y = -1;
		if (node != null) {
			if (node.getNodeName().equalsIgnoreCase("coordinate")) {
				Node child = node.getFirstChild();
				while (child != null) {
					if (child.getNodeName().equalsIgnoreCase("x")) {
						x = (short) Float.parseFloat(child.getFirstChild().getNodeValue());
					}
					if (child.getNodeName().equalsIgnoreCase("y")) {
						y = (short) Float.parseFloat(child.getFirstChild().getNodeValue());
					}
					child = child.getNextSibling();
				}
			}
		}

		if (x <= 0 || y <= 0) {
			if (workItemDot == null || !(workItemDot instanceof InvalidWorkItemDot)) {
				workItemDot = new InvalidWorkItemDot(pState, creationTime, key);
			}
		} else if (x > map.getWidth() || y > map.getHeight()) {
			if (workItemDot == null || !(workItemDot instanceof HiddenWorkItemDot)) {
				workItemDot = new HiddenWorkItemDot(pState, creationTime, key);
			}
		} else {
			if (workItemDot == null || !(workItemDot instanceof WorkItemDot) || ((WorkItemDot) workItemDot).getX() != x
					|| ((WorkItemDot) workItemDot).getY() != y) {
				workItemDot = new WorkItemDot(x, y, pState, creationTime, key);
			}
		}

		return workItemDot;
	}

	/*
	 * check the existing of dot into the previous frame of the movie
	 */
	private AbstractWorkItemDot pick(ActivityKey key, byte pState, long creationTime) {
		AbstractWorkItemDot workDot = preIndexmovies.get(key);
		if (workDot != null && workDot.getState() == pState && workDot.getCreationTime() == creationTime) {
			return workDot;
		} else {
			return null;
		}
	}

	private String getInvalidString() {
		return "<?xml version=\"1.0\" encoding=\"UTF-8\"?><coordinate><x>-1</x><y>-1</y><z>0</z></coordinate>";
	}

	private synchronized void applyEvent(AbsEvent event, org.processmining.plugins.mapvis.replay.State state,
			Set<String> taskVarNames, int index) {

		String activityName = event.getActivityName();
		String caseId = event.getCaseId();
		ActivityKey key = new ActivityKey(activityName, caseId);

		this.xAxis = false;
		this.yAxis = false;

		if (state.getWorkItemMetaFor(key) == null) {
			WorkItemMeta newMeta = new WorkItemMeta();
			state.setWorkItemMetaFor(key, newMeta);
		}

		if (event.getState().getProgressState() == ActivityState.SCHEDULED) {
			WorkItemMeta newMeta = state.getWorkItemMetaFor(key);
			newMeta.setEnablementTime(event.getTimestamp().getTime());
			state.setWorkItemMetaFor(key, newMeta);
		} else if (event.getState().getProgressState() == ActivityState.ASSIGNED) {
			WorkItemMeta newMeta = state.getWorkItemMetaFor(key);
			newMeta.setAllocatedTime(event.getTimestamp().getTime());
			state.setWorkItemMetaFor(key, newMeta);
		} else if (event.getState().getProgressState() == ActivityState.STARTED) {
			WorkItemMeta newMeta = state.getWorkItemMetaFor(key);
			newMeta.setStartedTime(event.getTimestamp().getTime());
			state.setWorkItemMetaFor(key, newMeta);
		}

		// set activity state
		ActivityState activityState = event.getState();
		boolean lastTrace = event.isLastTraceIndex();

		// set variables
		LinkedList<Object[]> vars = event.getVars();
		for (Object[] v : vars) {
			String varName = (String) v[0];
			if (!map.isOldMap()) {
				if (!xAxis)
					xAxis = map.getXAxis().equals(varName);
				if (!yAxis)
					yAxis = map.getYAxis().equals(varName);

				if (map.getXAxis().equals(varName) || map.getYAxis().equals(varName)) {
					Object varValue = v[1];
					if (taskVarNames.contains(varName)) {
						activityState.setVar(varName, varValue);
					} else {
						state.setNuAt(varName, caseId, varValue, lastTrace); // !
					}
				}

			} else {
				Object varValue = v[1];
				if (taskVarNames.contains(varName)) {
					activityState.setVar(varName, varValue);
				} else {
					state.setNuAt(varName, caseId, varValue, lastTrace);
				}

			}
		}

		long startTime = 0;

		if (event.getState().getProgressState() == ActivityState.CONCLUDED) {
			WorkItemMeta metaData = state.getWorkItemMetaFor(key);
			if (metaData != null) {
				startTime = metaData.getEnablementTime();
				if (startTime == 0)
					startTime = metaData.getAllocatedTime();
				if (startTime == 0)
					startTime = metaData.getStartedTime();
				if (startTime != 0) {
			
					long numConcludedActivity = ProcessIDData.getInstance().getNumConcludedActivity();
					
					Long meanExecutionTime = ((ProcessIDData.getInstance().getMeanExecutionTime() * numConcludedActivity) + (event
							.getTimestamp().getTime() - startTime)) / (numConcludedActivity + 1);
					
					numConcludedActivity++;
					ProcessIDData.getInstance().setNumConcludedActivity(numConcludedActivity);
					
					Long meanPerActivity = ProcessIDData.getInstance().getMeanExecutionTimePerActivity(activityName);
					Integer concludedInstances = ProcessIDData.getInstance().getNumConcludedInstancesPerActivity(
							activityName);

					meanPerActivity = ((meanPerActivity * concludedInstances) + (event.getTimestamp().getTime() - startTime))
							/ (concludedInstances + 1);

					ProcessIDData.getInstance().setMeanExecutionTime(meanExecutionTime);

					ProcessIDData.getInstance().setMeanExecutionTimePerActivity(activityName, meanPerActivity);

					ProcessIDData.getInstance().setNumConcludedInstancesPerActivity(activityName,
							concludedInstances + 1);
				}
			}
		}

		state.setAlphaAt(key, activityState);
	}

}
