package org.processmining.plugins.mapvis.replay;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.NavigableMap;
import java.util.Set;
import java.util.TreeMap;

public class State implements Serializable {

	private static final long serialVersionUID = 1L;
	private Map<ActivityKey, ActivityState> alpha_values;
	private Map<String, Map<String, Object>> nu_values;
	private NavigableMap<ActivityKey, WorkItemMeta> workItemMetaMap;

	public State() {
		alpha_values = new HashMap<ActivityKey, ActivityState>();
		nu_values = new HashMap<String, Map<String, Object>>();
		workItemMetaMap = new TreeMap<ActivityKey, WorkItemMeta>();
	}

	public ActivityState alpha(ActivityKey key) { // returns the activity state
		return alpha_values.get(key);
	}

	public Object nu(String varName, String caseID) { // returns the value of a trace variable
		return nu_values.get(caseID).get(varName);
	}

	public Set<ActivityKey> domAlpha() {
		return alpha_values.keySet();
	}

	public Set<String> domNu(String cid) {
		if (!nu_values.containsKey(cid))
			return new HashSet<String>();
		else
			return nu_values.get(cid).keySet();
	}

	/*
	 * contains events
	 */
	public void setAlphaAt(ActivityKey key, ActivityState activityState) {
		assert (activityState != null);
		if (alpha_values.containsKey(key) && !(activityState.getProgressState() == ActivityState.CONCLUDED)) {
			alpha_values.put(key, activityState);
		} else {
			if (!(activityState.getProgressState() == ActivityState.CONCLUDED)) {
				alpha_values.put(key, activityState);
			} else {
				alpha_values.remove(key);
				workItemMetaMap.remove(key);
			}
		}
	}

	/*
	 * contain event attribute values
	 */
	public void setNuAt(String varName, String caseId, Object value, boolean lastTrace) {
		if (!lastTrace) {
			if (!nu_values.containsKey(caseId)) {
				Map<String, Object> nuValues = new HashMap<String, Object>();
				nuValues.put(varName, value);
				nu_values.put(caseId, nuValues);
			} else {
				Object entry = nu_values.get(caseId).get(varName);
				if (entry == null) {
					nu_values.get(caseId).put(varName, value);
				} else {
					if (!entry.equals(value)) {
						nu_values.get(caseId).put(varName, value);
					}
				}
			}
		} else {
			// remove variable key based on last trace of the dataset
			nu_values.remove(caseId);
		}
	}

	public WorkItemMeta getWorkItemMetaFor(ActivityKey key) {
		return workItemMetaMap.get(key);
	}

	public void setWorkItemMetaFor(ActivityKey key, WorkItemMeta workItemMeta) {
		Entry<ActivityKey, WorkItemMeta> entry = workItemMetaMap.floorEntry(key);
		if (entry == null || !entry.getKey().equals(key))
			workItemMetaMap.put(key, workItemMeta);
		else {
			if (!entry.getValue().equals(workItemMeta))
				workItemMetaMap.put(entry.getKey(), workItemMeta);
		}
	}

	/*
	 * create tag of an event for performing an query
	 */
	public String getXML(Date timestamp, String activity, String cid, ActivityKey activityKey) {
		StringBuilder returnXML = new StringBuilder("<state>");

		for (String varName : domNu(cid)) {
			Object value = nu(varName, cid);
			returnXML.append("<").append(varName).append(">").append(value).append("</").append(varName).append(">");
		}

		for (String varName : alpha(activityKey).getAllVarName()) {
			String value = (String) alpha(activityKey).getVar(varName);
			returnXML.append("<").append(varName).append(">").append(Encode.encodeText(value)).append("</")
					.append(varName).append(">");
		}

		returnXML.append("<caseID>").append(cid).append("</caseID>"); // "global" == per case		
		returnXML.append("<enablementTime>").append(getWorkItemMetaFor(activityKey).getEnablementTime());
		returnXML.append("</enablementTime>"); // "global" == per case
		returnXML.append("<startedTime>").append(getWorkItemMetaFor(activityKey).getStartedTime());
		returnXML.append("</startedTime>"); // "global" == per case
		returnXML.append("<allocatedTime>").append(getWorkItemMetaFor(activityKey).getAllocatedTime());
		returnXML.append("</allocatedTime>"); // "global" == per case
		returnXML.append("<now>").append(timestamp.getTime()).append("</now>"); // local == per workitem			
		returnXML.append("</state>");
		return returnXML.toString();
	}

	public Object getVarName(String cid, String varName) {
		if (nu_values.containsKey(cid)) {
			Object value = nu_values.get(cid).get(varName);
			return value;
		} else
			return null;
	}

}
