package org.processmining.plugins.mapvis.replay;

import java.io.Serializable;
import java.util.Date;

/***
 * get time stamps while playing visualisation movie
 * 
 * @author ngupta
 *
 */
public class TimeStampArray implements Serializable{

	private static final long serialVersionUID = 1L;
	private static TimeStampArray Instance = null;
	private long[] tsArray; // Time array .. contains times from data set file
	private int size;
	private transient int counter;
	private long modelTime;// it contains the current time-stamp while playing the replay log animation
	private short[] numActivityPerState;

	
	public static TimeStampArray getInstance() {
		if (Instance == null)
			Instance = new TimeStampArray();
		return Instance;
	}

	public static void setTimeStampArray(TimeStampArray newInstance) {
		Instance = newInstance;
	}
	
	public void setArraySize(int size) {
		this.size = size;
		this.tsArray = new long[size];
		this.numActivityPerState = new short[size];
	}
	
	public void setTime(int index, long timestamp) {
		this.tsArray[index] = timestamp;
	}
	
	public long[] getTsArray() {
		return this.tsArray;
	}

	public void setTsArray(long[] tsArray) {
		this.tsArray = tsArray;
	}
	
	public long getStart() {
		return this.tsArray[0];
	}

	public long getEnd() {
		return this.tsArray[this.size - 1];
	}

	public Date getTimeForCounter(int counter) {
		return new Date(this.tsArray[counter]);
	}
	
	public int getCounter() {
		return this.counter;
	}

	public void updateCounter(long timestamp) {
		while ((this.counter + 1 < this.size) && (timestamp > this.tsArray[this.counter + 1]))
			this.counter++;
	}
	public void seekCounter(long timestamp) {
		int i = this.size / 2;
		int interval = this.size / 4;
		while (!(i == 0 || i == (this.size - 1) || this.tsArray[i] == timestamp || this.tsArray[i + 1] > timestamp
				&& this.tsArray[i] < timestamp)) {
			if (this.tsArray[i] < timestamp) {
				i = i + interval;
			} else {
				i = i - interval;
			}
			interval = interval / 2;
			if (interval == 0)
				interval = 1;
		}
		this.counter = i;
	}

	
	public long getModelingTime() {
		return this.modelTime;
	}

	public void updateModelingTime(long modelTime) {
		this.modelTime = modelTime;

	}

	public short getNumberActivity(int state) {
		return numActivityPerState[state];
	}

	public void setNumActivityPerState(int index ,short numActivityPerState) {
		this.numActivityPerState[index] = numActivityPerState;
	}
	
	
	public long getTimeFromIndex(int index){
		return this.tsArray[index];
	}
}
