package org.processmining.plugins.mapvis.visualize;

import java.awt.Color;
import java.awt.GridLayout;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.processmining.framework.util.ui.scalableview.ScalableComponent;
import org.processmining.framework.util.ui.scalableview.ScalableViewPanel;
import org.processmining.framework.util.ui.scalableview.interaction.ViewInteractionPanel;
import org.processmining.plugins.mapvis.importdata.visualization.MapVisMap;
import org.processmining.plugins.mapvis.replay.FrequentValueEntry;
import org.processmining.plugins.mapvis.replay.History;
import org.processmining.plugins.mapvis.replay.TimeStampArray;
import org.processmining.plugins.mapvis.visualize.workitem.AbstractWorkDotColorSchema;
import org.processmining.plugins.mapvis.visualize.workitem.AbstractWorkDotColorSchema.ColoringSchema;

public class ColourLegendPanel extends JPanel implements ViewInteractionPanel {

	private static final long serialVersionUID = 1L;
	private MapVisMap map;
	private History instance;
	private Map<String, Map<Color, Integer>> colorData;
	long lastcounter = -1;

	public ColourLegendPanel(MapVisMap map, History instance) {
		this.map = map;
		this.instance = instance;
		recompute();
	}

	public void recompute() {
		Boolean showNumbers = AbstractWorkDotColorSchema.showNumberPerMap.get(map);
		if (showNumbers == null)
			showNumbers = true;
		AbstractWorkDotColorSchema.ColoringSchema schema = AbstractWorkDotColorSchema.schemaMap.get(map);
		if (schema == null)
			schema = ColoringSchema.STATEBASED;
		switch (schema) {
			case AGEDONACTIVITYMEANTIME :
				recomputeAge();
				break;
			case PROPERTYBASED :
				recomputeDiscriminating(showNumbers);
				break;
			case STATEBASED :
				recomputeOnState();
				break;
			case INDEX :
				recomputeOnIndex();
				break;
			default :
				break;

		}
	}

	private void addLabelRow(String name, Color color) {
		JLabel aLabel = new JLabel(name);
		aLabel.setBackground(color);
		aLabel.setOpaque(true);
		setForegroundForLabels(color, aLabel);
		this.add(aLabel);
	}

	private void recomputeOnState() {
		this.setLayout(new GridLayout(4, 2));
		this.removeAll();
		addLabelRow("SCHEDULED", Color.WHITE);
		addLabelRow("ASSIGNED", Color.CYAN);
		addLabelRow("STARTED", Color.GREEN);
		addLabelRow("SUSPENDED", Color.YELLOW);
		addLabelRow("ABORTED", Color.RED);
	}

	private void recomputeOnIndex() {
		this.setLayout(new GridLayout(2, 2));
		this.removeAll();
		addLabelRow("Negative", Color.YELLOW);
		addLabelRow("Positive", Color.BLACK);

	}

	public void recomputeDiscriminating(Boolean showNumbers) {
		String discriminating = AbstractWorkDotColorSchema.discriminatingAttribute.get(map);
		int rows = Math.min(History.MAX_NUM_ATTR_OCCUR, instance.getMostFrequentValue(discriminating).size());
		this.setLayout(new GridLayout(rows + 3, 2));
		this.removeAll();
		int i = 0;
		this.add(new JLabel("Value"));
		this.add(new JLabel("Color/Occurrences"));
		int totalOccurrenceMostFrequent = 0;
		String strLabel;

		for (FrequentValueEntry value : instance.getMostFrequentValue(discriminating)) {
			this.add(new JLabel(value.getVarName()));
			totalOccurrenceMostFrequent += value.getOccurrence();

			if (showNumbers)
				strLabel = value.getOccurrence() + "";
			else
				strLabel = "N.A.";
			JLabel label = new JLabel(strLabel);

			Color aColor = History.mainColors[i++];
			setForegroundForLabels(aColor, label);
			this.add(label);
		}
		this.add(new JLabel("Others"));

		if (showNumbers)
			strLabel = (instance.getNumTraces() - totalOccurrenceMostFrequent) + "";
		else
			strLabel = "N.A.";
		JLabel label = new JLabel(strLabel);
		Color aColor = History.mainColors[History.MAX_NUM_ATTR_OCCUR];
		setForegroundForLabels(aColor, label);
		this.add(label);

		this.add(new JLabel("No Key Info"));
		JLabel label1 = new JLabel("N.A.");
		Color color = Color.blue;
		setForegroundForLabels(color, label1);
		this.add(label1);
	}

	public void recomputeAge() {
		long counter = TimeStampArray.getInstance().getModelingTime();
		if (counter != lastcounter) {
			this.removeAll();
			lastcounter = counter;
			int rows = colorData.size();
			this.setLayout(new GridLayout(rows, 2));
			Map<Color, Integer> colvalue;
			for (Entry<String, Map<Color, Integer>> col : colorData.entrySet()) {
				colvalue = new HashMap<Color, Integer>();
				colvalue.putAll(col.getValue());
				for (Entry<Color, Integer> colInt : colvalue.entrySet()) {
					String age = col.getKey();
					Color color = colInt.getKey();
					JLabel label = new JLabel(age);
					setForegroundForLabels(color, label);
					this.add(label);
				}
			}
		}
	}

	protected void getAgedata(Map<String, Map<Color, Integer>> colorData1) {
		colorData = new TreeMap<String, Map<Color, Integer>>();
		this.colorData.putAll(colorData1);
	}

	// find the best colour contracts for the text according to its respective colour
	private void setForegroundForLabels(Color aColor, JLabel label) {
		double y = 0.2126 * Math.pow(aColor.getRed() / 255F, 2.2) + 0.7151 * Math.pow(aColor.getGreen() / 255F, 2.2)
				+ 0.0721 * Math.pow(aColor.getBlue() / 255F, 2.2);
		if (y < 0.18)
			label.setForeground(Color.white);
		else
			label.setForeground(Color.BLACK);

		label.setOpaque(true);
		label.setBackground(aColor);
	}

	public JComponent getComponent() {
		return this;
	}

	public double getHeightInView() {
		return this.getPreferredSize().getHeight();
	}

	public String getPanelName() {
		return "Legend";
	}

	public double getWidthInView() {
		return this.getPreferredSize().getWidth();
	}

	public void setParent(ScalableViewPanel viewPanel) {
	}

	public void setScalableComponent(ScalableComponent scalable) {
	}

	public void willChangeVisibility(boolean to) {
	}

	public void updated() {
	}

}
