package org.processmining.plugins.mapvis.visualize;

import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.fluxicon.slickerbox.components.InspectorButton;
import com.fluxicon.slickerbox.components.PlayButton;
import com.fluxicon.slickerbox.components.SlickerTabbedPane;
import com.fluxicon.slickerbox.ui.SlickerSliderUI;

public class ControlPanel extends JPanel {

	private static final long serialVersionUID = 918587885234059406L;
	
	protected final SlickerTabbedPane canvas;
	protected final AnimationTimer timer;
	protected boolean running;	
	protected TimeDisplay timeDisplay;
	protected SeekPane seekPane;	
	protected JSlider mergeSlider; 
	protected JSlider speedSlider;	
	protected InspectorButton fullScreenButton;
	
	public ControlPanel(final org.processmining.plugins.mapvis.VisData visData, final SlickerTabbedPane mapTabbedPane, final AnimationTimer timer) {
		this.canvas = mapTabbedPane;
		this.timer = timer;
		this.seekPane = new SeekPane(visData, timer);
		this.timeDisplay = new TimeDisplay();
		this.timer.addListener(this.timeDisplay);
		running = false;
		final PlayButton playButton = new PlayButton();
		playButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(running == false) {
					timer.start();
					running = true;
				} else {
					timer.stop();
					running = false;
				}
			}
		});
		this.timer.addListener(new AnimationListener() {
			public void started() {
				playButton.setPlay(false);
				running = true;
			}
			public void stopped() {
				playButton.setPlay(true);
				running = false;
			}
			public void updateModelTime(long modelTime) {
			}
		});
		final double speedRef = timer.getSpeedFactor();
		double maxValue=(visData.getEnd()-visData.getStart())/10E2;
		if (maxValue>Integer.MAX_VALUE)
			maxValue=Integer.MAX_VALUE;
		speedSlider = new JSlider(Math.min(10, (int)maxValue/100), (int)maxValue, (int)maxValue/100);
		speedSlider.setUI(new SlickerSliderUI(speedSlider, 
				new Color(40, 40, 40),
				new Color(70, 70, 70),
				new Color(0, 0, 0),
				new Color(80, 80, 80),
				new Color(0, 0, 0),
				new Color(40, 40, 40), 8, 14));
		speedSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {

				double factor = speedSlider.getValue() / 30.0F;
				timer.setSpeedFactor(speedRef * factor);
			}
		});

		this.setBackground(new Color(30, 30, 30));
		this.setBorder(BorderFactory.createEmptyBorder(11, 15, 11, 15));
		this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

		this.add(playButton);
		this.add(Box.createHorizontalStrut(13));
		this.add(seekPane);
		this.add(Box.createHorizontalStrut(13));
		this.add(timeDisplay);
		this.add(Box.createHorizontalStrut(10));
		this.add(packSlider(speedSlider, "playback speed"));
		this.add(Box.createHorizontalStrut(10));
	}
	
	protected void paintComponent(Graphics g) {
		Graphics2D g2d = (Graphics2D)g;
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		int width = getWidth();
		int height = getHeight();
		GradientPaint gradient = new GradientPaint(0, 0, new Color(70, 70, 70), 
				0, height - (height / 3), new Color(60, 60, 60), false);
		g2d.setPaint(gradient);
		g2d.fillRect(0, 0, width + 1, height + 1);
	}
	
	protected JPanel packSlider(JSlider slider, String name) {
		JPanel panel = new JPanel();
		panel.setBorder(BorderFactory.createEmptyBorder());
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		panel.setOpaque(false);
		JLabel label = new JLabel(name);
		label.setForeground(new Color(13, 13, 13));
		label.setOpaque(false);
		label.setFont(label.getFont().deriveFont(10f));
		label.setAlignmentX(JLabel.LEFT_ALIGNMENT);
		label.setHorizontalTextPosition(JLabel.LEFT);
		slider.setAlignmentX(JSlider.CENTER_ALIGNMENT);
		panel.add(Box.createVerticalGlue());
		panel.add(slider);
		panel.add(label);
		panel.add(Box.createVerticalGlue());
		return panel;
	}
}
