package org.processmining.plugins.mapvis.visualize;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import org.processmining.framework.util.ui.scalableview.ScalableComponent;
import org.processmining.framework.util.ui.scalableview.ScalableViewPanel;
import org.processmining.framework.util.ui.scalableview.interaction.ViewInteractionPanel;
import org.processmining.plugins.mapvis.replay.TimeStampArray;

import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * Export Map panel image as a .png file
 * 
 * @author ngupta
 * 
 */
public class ExportPanel extends JPanel implements ViewInteractionPanel, ActionListener {

	private static final long serialVersionUID = 1L;
	private JButton jbnExport;
	private JFileChooser chooser;
	private MapImagePanel mapPanel;

	public ExportPanel() {
	}

	public ExportPanel(MapImagePanel mapPanel) {
		super(new BorderLayout());
		this.mapPanel = mapPanel;
		initComponent();
		this.setBorder(BorderFactory.createEmptyBorder());
		this.setOpaque(true);
	}

	//Create initial components for the export panel
	private void initComponent() {
		SlickerFactory slickerFactory = SlickerFactory.instance();
		//Export button
		jbnExport = slickerFactory.createButton("Export image");
		jbnExport.setVerticalAlignment(SwingConstants.CENTER);
		jbnExport.addActionListener(this);
		this.add(jbnExport);
	}

	//save map panel image into the choose directory
	private void saveMapPanelImage(String path) {
		try {
				
			File file = new File(path);
			BufferedImage image = new BufferedImage(mapPanel.getWidth(), mapPanel.getHeight(),
					BufferedImage.TYPE_INT_RGB);
			// call the Component's paint method,
			// using the Graphics object of the image.

			mapPanel.paint(image.getGraphics());
			ImageIO.write(image, "png", file);

		} catch (Exception exception) {
			System.out.print("Exception to write image");
		}

	}

	public void actionPerformed(ActionEvent arg0) {
		chooser = new JFileChooser();
		chooser.setDialogTitle("Select output directory for images");
		int status = chooser.showOpenDialog(null);
		if (status == JFileChooser.APPROVE_OPTION) {
			File selectedFile = chooser.getSelectedFile();
			if (!selectedFile.getName().isEmpty()) {
				if ("Export image".equals(arg0.getActionCommand())) {
					DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd_HHmmss");
					long modelTime = TimeStampArray.getInstance().getModelingTime();
					Date date = new Date(modelTime);
					String mapName = mapPanel.getMapVisMap().getName();
					String path = selectedFile.getParent() + "/" + selectedFile.getName()+dateFormat.format(date) +"_" + mapName + ".png"; ;
					saveMapPanelImage(path);
					JOptionPane.showMessageDialog(null, "Saved map image", "Message", JOptionPane.INFORMATION_MESSAGE);
				}
			}
		} else if (status == JFileChooser.CANCEL_OPTION)
			System.out.println("canceled");
	}

	public String getPanelName() {
		return "Export";
	}

	public JComponent getComponent() {
		return this;
	}

	public int getPosition() {
		return SwingConstants.NORTH;
	}

	public void setScalableComponent(ScalableComponent scalable) {
	}

	public void setParent(ScalableViewPanel viewPanel) {
	}

	public double getHeightInView() {
		return this.getPreferredSize().getHeight();
	}

	public double getWidthInView() {
		return this.getPreferredSize().getWidth();
	}

	public void willChangeVisibility(boolean to) {
	}

	public void updated() {
	}

}
