package org.processmining.plugins.mapvis.visualize;

import java.lang.reflect.Field;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.swing.JOptionPane;

import org.deckfour.uitopia.api.model.ResourceType;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIContext;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.model.ProMPOResource;
import org.processmining.contexts.uitopia.model.ProMTask;
import org.processmining.framework.plugin.GlobalContext;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.providedobjects.ProvidedObjectID;
import org.processmining.plugins.mapvis.replay.History;
import org.processmining.plugins.utils.ProvidedObjectHelper;

/*
 * @ngupta Export sub-log file
 */
public class ExportSubLog {

	private History instHistory;
	private PluginContext context;

	public ExportSubLog(History instHistory, PluginContext context) {
		this.instHistory = instHistory;
		this.context = context;
	}

	/*
	 * It will analyse the data of the current log file and publish only
	 * selected working dot data as a sub-log
	 */

	protected void publishSubLog(ArrayList<String> workDots) {
		// create a new log file
		try {
			if (instHistory.getLogFile() != null) {
				XLog newLog = XFactoryRegistry.instance().currentDefault()
						.createLog(instHistory.getLogFile().getAttributes());
				for (XTrace trace : instHistory.getLogFile()) {
					String caseId = XConceptExtension.instance().extractName(trace);
					if (workDots.contains(caseId)) {
						XTrace newTrace = XFactoryRegistry.instance().currentDefault()
								.createTrace(trace.getAttributes());
						for (XEvent event : trace) {
							XEvent newEvent = XFactoryRegistry.instance().currentDefault()
									.createEvent(event.getAttributes());
							newTrace.add(newEvent);
						}
						newLog.add(newTrace);
					}
				}

				Date date = new Date();
				DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd_HHmmss");
				String fileName = "SubLog_" + dateFormat.format(date);

				publish(context, fileName + "", newLog, XLog.class, true);
				JOptionPane.showMessageDialog(null, "A Sub-log has been created for the clicked dot", "Message",
						JOptionPane.INFORMATION_MESSAGE);
			}else{
				JOptionPane.showMessageDialog(null, "Sorry!! The framework does not have an event log", "Message",
						JOptionPane.INFORMATION_MESSAGE);
			}
			
		} catch (Exception e) {
			// TODO: handle exception
		}
	}

	/**
	 * Will publish object as a provided object with the class specified by
	 * class. If the context is a UIPluginContext and favourite is true, it will
	 * be marked as a favourite object as well. Finally, if it is marked as
	 * Favourite, it will also be show and ProM will switch to the Views tab.
	 * 
	 * @param <T>
	 * @param context
	 * @param name
	 * @param object
	 * @param clazz
	 * @param favorite
	 */
	public static <T> void publish(final PluginContext context, final String name, final T object,
			final Class<? super T> clazz, final boolean favorite) {
		final ProvidedObjectID id = context.getProvidedObjectManager().createProvidedObject(name, object, clazz,
				context);

		if (context instanceof UIPluginContext) {
			final GlobalContext gcontext = ((UIPluginContext) context).getGlobalContext();
			if (gcontext instanceof UIContext) {
				final UIContext uicontext = (UIContext) gcontext;
				final ResourceType resType = uicontext.getResourceManager().getResourceTypeFor(clazz);
				if (resType != null) {
					ProMTask task = null;
					try {
						final Field taskField = context.getClass().getDeclaredField("task");
						taskField.setAccessible(true);
						task = (ProMTask) taskField.get(context);
					} catch (final Exception _) {
					}
					final List<Collection<ProMPOResource>> lst = Collections.emptyList();
					ProMPOResource res = new ProMPOResource(uicontext, task == null ? null : task.getAction(), resType,
							id, lst);
					res = uicontext.getResourceManager().addResource(id, res);
				}
			}
		}
		if (favorite) {
			ProvidedObjectHelper.setFavorite(context, object);
		}
	}

}
