package org.processmining.plugins.mapvis.visualize;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.regex.Pattern;

import org.processmining.plugins.mapvis.replay.NameLookUp;
import org.processmining.plugins.mapvis.replay.ProcessIDData;
import org.processmining.plugins.mapvis.visualize.workitem.AbstractWorkItemDot;
/*
 * @ngupta
 * Filter an event or attribute values for AbstractWorkDotItems
 */
public class FilterEventAttribute {
	
	private Collection<String> eventFilter;
	private MapPanel parentPanel;
	
	public FilterEventAttribute(Collection<String> eventFilter, MapPanel parentPanel) {
		this.eventFilter = eventFilter;
		this.parentPanel = parentPanel;
	}

	/*
	 * find whether the selected filter event is matching the current working
	 * item dot. if selected event and working dot matches then it will get
	 * displayed on map and side bar panels
	 */
	protected void eventFilter(AbstractWorkItemDot workItem) {
		Iterator<?> eventIter;
		if (eventFilter != null) {
			eventIter = eventFilter.iterator();
			while (eventIter.hasNext()) {
				String event = eventIter.next().toString();
				String workItemValue = workItem.getKey().getActivityName().toString();
				if (workItemValue.equals(event)) {
					parentPanel.selectWorkItemDotLocation(workItem);
				}
			}
		}
	}

	
	protected boolean eventAttributeFilter(AbstractWorkItemDot workItem) {
		Iterator<?> eventIter;
		boolean eventResult = false;
		if (eventFilter != null) {
			eventIter = eventFilter.iterator();
			while (eventIter.hasNext()) {
				String event = eventIter.next().toString();
				String workItemValue = workItem.getKey().getActivityName().toString();
				if (workItemValue.equals(event)) {
					eventResult = true;
				}
			}
		}
		return eventResult;
	}


	private float getValue(String val) {
		float value = 0f;
		if (val != null) {
			String decimalPattern = "([0-9]*)\\.([0-9]*)";
			boolean match = Pattern.matches(decimalPattern, val);

			if (match) {
				value = Float.valueOf(val);
			} else {
				value = Integer.parseInt(val);
			}
		}
		return value;
	}

	/*
	 * continuous attribute
	 */
	private boolean continousAttributeSelection(ArrayList<?> t, String entry, String key) {
		//int i = Integer.parseInt(t.toArray()[0].toString());
		//int j = Integer.parseInt(t.toArray()[1].toString());
		//int entryValue = Integer.parseInt(entry);
		
		float i = getValue(t.toArray()[0].toString());
		float j = getValue(t.toArray()[1].toString());
		float entryValue = getValue(entry);
		// check whether value is between the given range of text boxes
		if (entryValue >= i && entryValue <= j)
			return true;
		else
			return false;
	}

	/*
	 * find whether the selected filter attributes is matching the current
	 * working item dot. if selected attribute value and working dot attribute
	 * value matches then it will get displayed on map and side bar panels
	 */
	protected void attributeFilter(AbstractWorkItemDot workItem) {
		Map<String, Boolean> select = new HashMap<String, Boolean>();
		for (Entry<String, ArrayList<?>> t : parentPanel.selectedAttributeValues.entrySet()) {
			String key = t.getKey().toString();
			Boolean value = ProcessIDData.getInstance().getProcessAttributeType().get(key);

			// get all process id and their values corresponding to the selected attribute
			Map<Integer, String> processIdValue = ProcessIDData.getInstance().getProcessIdAttribute(key);
			int processIdValueForWorkItemDot = NameLookUp.getInstance().getValueByCaseId(workItem.getKey().getCaseId());

			if (value && processIdValue.containsKey(processIdValueForWorkItemDot)) {
				boolean result = continousAttributeSelection(t.getValue(),
						processIdValue.get(processIdValueForWorkItemDot), key);
				select.put(key, result);
			}

			// for literal values 
			if (!value && processIdValue.containsKey(processIdValueForWorkItemDot)) {
				String workDotValue = processIdValue.get(processIdValueForWorkItemDot);
				// get all id from the selected attributes
				for (Object aString : t.getValue()) {
					String attrValue = aString.toString();
					if (attrValue.equals(workDotValue)) {
						select.put(key, true);
						break;
					}
					if (!attrValue.equals(workDotValue))
						select.put(key, false);
				}
			}

			// when event does not have attribute value to corresponding selected attribute  
			// if "No Key" combo value item has been selected then it will display the working dots
			if (!value && t.getValue().contains("No Key") && !select.containsKey(key))
				select.put(key, true);

			// when no values with the corresponding selected attributes on the filter panel
			if (!select.containsKey(t.getKey().toString())) {
				select.put(t.getKey().toString(), false);
			}
		}

		if (select.containsValue(false)) {
			// do nothing
		} else if (select != null && !select.isEmpty())
			parentPanel.selectWorkItemDotLocation(workItem);
	}

}
