package org.processmining.plugins.mapvis.visualize;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Pattern;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.processmining.framework.util.ui.scalableview.ScalableComponent;
import org.processmining.framework.util.ui.scalableview.ScalableViewPanel;
import org.processmining.framework.util.ui.scalableview.interaction.ViewInteractionPanel;
import org.processmining.framework.util.ui.widgets.ProMCheckComboBox;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.plugins.mapvis.replay.ProcessIDData;
import org.processmining.plugins.mapvis.visualize.workitem.AbstractWorkDotColorSchema;

import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * @author ngupta Create Filters on Event and attribute level for the data set
 *         file
 */
public class FilterPanel extends JPanel implements ViewInteractionPanel{

	private static final long serialVersionUID = 1L;
	private MapImagePanel imagePanel;
	// for min and max text boxes
	private JPanel[] panel = new JPanel[100];
	private JPanel mainPanel, eventPanel, subEventPanel, buttonPanel;
	protected JPanel listPanel;
	protected ProMCheckComboBox eventAttributeCombo = null;
	protected ProMCheckComboBox attributeCombo = null;
	private JButton addButton;
	protected JCheckBox eventSelection = SlickerFactory.instance().createCheckBox(
			"Filter events for plotting on a map panel", false);
	protected JCheckBox attributeSelection = SlickerFactory.instance().createCheckBox(
			"Filter attributes for plotting on a map panel ", false);

	protected Map<String, ProMCheckComboBox> comboBoxes = new HashMap<String, ProMCheckComboBox>();
	protected Map<String, ProMTextField> mintextboxes = new ConcurrentHashMap<String, ProMTextField>();
	protected Map<String, ProMTextField> maxtextboxes = new ConcurrentHashMap<String, ProMTextField>();
	private Map<String, Boolean> values = new HashMap<String, Boolean>();
	private String[] attributes;
	private PanelComponents components;
	private final FilterListner listener;
	

	public FilterPanel(MapImagePanel imagePanel, String[] attributes) {
		this.attributes =attributes;
		this.imagePanel = imagePanel;
		components = new PanelComponents();
		listener = new FilterListner(this);
		
		values = ProcessIDData.getInstance().getProcessAttributeType();
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		initComopent();
	}

	private void initComopent() {
		mainPanel = new JPanel();
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
		mainPanel.add(Box.createVerticalStrut(10));

		eventAttributeCombo = new ProMCheckComboBox(ProcessIDData.getInstance().getActivityNames());
		attributeCombo = new ProMCheckComboBox(attributes);

		// event combo box panel
		eventPanel = new JPanel();
		eventPanel.setLayout(new BoxLayout(eventPanel, BoxLayout.Y_AXIS));
		eventPanel.add(eventSelection);
		subEventPanel = new JPanel();
		subEventPanel.setSize(400, 30);
		//Event check box
		subEventPanel = EventPanel();
		eventPanel.add(subEventPanel);
		subEventPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		eventPanel.add(Box.createVerticalStrut(10));

		eventSelection.addActionListener(listener);
		eventSelection.setActionCommand(FilterListner.EVENT);
		eventAttributeCombo.addItemListener(listener);
		eventAttributeCombo.setActionCommand(FilterListner.EVENTITEM);

		// attribute combo or text box panel
		listPanel = new JPanel();
		listPanel.add(attributeSelection);
		attributeSelection.addActionListener(listener);
		attributeSelection.setActionCommand(FilterListner.ATTRIBUTE);
		listPanel.setLayout(new BoxLayout(listPanel, BoxLayout.Y_AXIS));
		listPanel.add(Box.createVerticalStrut(10));

		// add and delete buttons
		buttonPanel = new JPanel();
		addButton = components.createButton("Select attribute");
		addButton.addActionListener(listener);
		addButton.setActionCommand(FilterListner.ADDATTRIBUTE);

		buttonPanel = components.createPanel(addButton);
		buttonPanel.add(Box.createHorizontalStrut(50));

		eventPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		listPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		buttonPanel.setAlignmentX(Component.LEFT_ALIGNMENT);

		// add all panels to the main panel
		mainPanel.add(eventPanel);
		mainPanel.add(listPanel);
		mainPanel.add(buttonPanel);
		this.add(mainPanel);

		// scroll able panel
		JScrollPane panelPane = new JScrollPane(mainPanel);
		mainPanel.setAutoscrolls(true);
		this.add(panelPane);
	}

	// it create  a check combo box on filter panel
	private void createComboPanel(String attr, int i) {
		for (Entry<String, Boolean> value : values.entrySet()) {
			if (value.getKey().equals(attr) && value.getValue() == false) {
				FilterGraphSelection(attr, ProcessIDData.getInstance().getAttributeFilterValues(attr), i);
			}
		}
	}

	// it create text field on filter panel
	private void createTextPanel(String attr, int i) {
		for (Entry<String, Boolean> value : values.entrySet()) {
			if (value.getKey().equals(attr) && value.getValue() == true) {
				SortedSet<String> attrValue = ProcessIDData.getInstance().getAttributeFilterValues(attr);
				numberSelection(attr, attrValue, i);
			}
		}
	}

	// create combo boxes for literal values on filer panel
	private void FilterGraphSelection(final String value, Collection<?> coll2, final int i) {
		Set<Object> internalSet;
		try {
			internalSet = new TreeSet<Object>();
			internalSet.addAll(coll2);
			internalSet.add("No Key");
		} catch (ClassCastException err) {
			internalSet = new HashSet<Object>();
			internalSet.addAll(coll2);
			internalSet.add("No Key");
		}

		ProMCheckComboBox component = new ProMCheckComboBox(internalSet);
		component.setMaximumSize(new Dimension(300, 30));
		component.setPreferredSize(new Dimension(300, 30));
		comboBoxes.put(value, component);

		JPanel buttonPanel = new JPanel();
		buttonPanel.setSize(new Dimension(30, 30));
		JButton remove = components.createButton("Remove");
		remove.setMaximumSize(new Dimension(10, 30));
		buttonPanel.add(remove);

		JLabel label = new JLabel(panelName(value));
		label.setMaximumSize(new Dimension(10, 30));

		JPanel componentPanel = new JPanel();
		componentPanel.setLayout(new GridLayout());
		componentPanel.add(label);
		componentPanel.add(component);

		remove.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				listPanel.remove(panel[i]);
				comboBoxes.remove(value);
				mainPanelUpdate();
			}
		});

		component.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				comboAction();
			}
		});

		panel[i] = new JPanel();
		panel[i].setSize(400, 30);
		panel[i].add(componentPanel);
		panel[i].add(buttonPanel);
		panel[i].add(Box.createVerticalStrut(33));
		panel[i].setAlignmentX(Component.LEFT_ALIGNMENT);

		listPanel.add(panel[i]);
		listPanel.updateUI();
		listPanel.repaint();
		mainPanel.updateUI();
	}

	private void comboAction() {
		if (attributeSelection.isSelected()) {
			imagePanel.getParentPane().toggleAttributeFilterOn(true);
			imagePanel.getParentPane().setAttributeFilterValue(getSelectedAttribute());
			imagePanel.getParentPane().setAttributeValues(mergeMappings());
		} else {
			imagePanel.getParentPane().toggleAttributeFilterOn(false);
		}
		imagePanel.getParentPane().recalc();
		
		Boolean showNumbers = AbstractWorkDotColorSchema.showNumberPerMap.get(imagePanel.getMapVisMap());
		if (showNumbers == null)
			showNumbers = true;
		imagePanel.getParentPane().recomputeValidChart(showNumbers);
		imagePanel.getParentPane().recomputeColorChart(showNumbers);
		imagePanel.getParentPane().addCharts();
	}

	private String panelName(String attr) {
		String panelLabel = " " + attr.trim().replaceAll("[^a-zA-Z]", " ");
		return panelLabel;
	}
	private void onlyNumeric(ProMTextField text) {
		text.addKeyListener(new KeyAdapter() {
			public void keyTyped(KeyEvent e) {
				char c = e.getKeyChar();
				if (!((c >= '0') && (c <= '9') || (c == KeyEvent.VK_BACK_SPACE) || (c == KeyEvent.VK_DELETE) || (c == '.') || (c == KeyEvent.VK_ENTER))) {
					getToolkit().beep();
					e.consume();
				}
			}
		});
	}

	private void numberSelection(final String attr, SortedSet<String> attrValues, final int i) {
		final ProMTextField mintxt = new ProMTextField();
		onlyNumeric(mintxt);
		final ProMTextField maxtxt = new ProMTextField();
		onlyNumeric(maxtxt);

		mintxt.setMaximumSize(new Dimension(150, 30));
		mintxt.setPreferredSize(new Dimension(150, 30));

		maxtxt.setMaximumSize(new Dimension(150, 30));
		maxtxt.setPreferredSize(new Dimension(150, 30));

		mintxt.setText(attrValues.first().toString());
		maxtxt.setText(attrValues.last().toString());

		mintextboxes.put(attr, mintxt);
		maxtextboxes.put(attr, maxtxt);

		JPanel buttonPanel = new JPanel();
		buttonPanel.setSize(new Dimension(30, 30));
		JButton remove = components.createButton("Remove");
		remove.setMaximumSize(new Dimension(10, 30));
		buttonPanel.add(remove);

		JLabel label = new JLabel(panelName(panelName(attr)));
		label.setMaximumSize(new Dimension(10, 30));
		JPanel componentPanel = new JPanel();
		componentPanel.setLayout(new GridLayout());
		componentPanel.add(label);
		componentPanel.add(components.createPanel(mintxt, maxtxt));

		panel[i] = new JPanel();
		panel[i].add(componentPanel);
		panel[i].add(buttonPanel);
		remove.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				listPanel.remove(panel[i]);
				mintextboxes.remove(attr);
				maxtextboxes.remove(attr);
				mainPanelUpdate();
			}
		});

		mintxt.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				comboAction();
			}
		});

		maxtxt.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				comboAction();
			}
		});

		panel[i].add(Box.createVerticalStrut(33));
		panel[i].setAlignmentX(Component.LEFT_ALIGNMENT);
		listPanel.add(panel[i]);

		listPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		listPanel.repaint();
		mainPanel.updateUI();

	}
	
	private void mainPanelUpdate() {
		listPanel.updateUI();
		listPanel.repaint();
		mainPanel.updateUI();
		comboAction();
	}

	protected void getCompoentsOnPanel() {
		Collection<String> selectAttr = getSelectedAttribute();
		if (selectAttr != null) {
			int i = 1;
			for (String attr : selectAttr) {
				// check whether a combo and text values are already available on filter panel or not
				if (!comboBoxes.containsKey(attr))
					createComboPanel(attr, i++);

				if (!mintextboxes.containsKey(attr))
					createTextPanel(attr, i++);
			}
		}
	}

	private Map<String, ArrayList<String>> getMappingForCombo() {
		Map<String, ArrayList<String>> retValue = new HashMap<String, ArrayList<String>>();
		for (String node : comboBoxes.keySet()) {
			retValue.put(node, new ArrayList<String>());

			@SuppressWarnings("unchecked")
			Collection<String> selection = comboBoxes.get(node).getSelectedItems();
			if (selection != null) {
				for (Object setValue : selection) {
					if (setValue.toString() != null && !retValue.containsKey(setValue)) {
						retValue.get(node).add(setValue.toString());
					}
				}
			}
		}
		return retValue;
	}

	private Map<String, ArrayList<Integer>> getMappingForTextbox() {
		Map<String, ArrayList<Integer>> value = new HashMap<String, ArrayList<Integer>>();

		for (String node : mintextboxes.keySet()) {
			value.put(node, new ArrayList<Integer>());
			value.get(node).add(getValue(mintextboxes.get(node).getText()));

			for (String node1 : maxtextboxes.keySet()) {
				if (node.equals(node1)) {
					value.get(node1).add(getValue(maxtextboxes.get(node1).getText()));
				}
			}
		}
		return value;
	}
	
	private int getValue(String val) {
		int value = 0;
		if (val != null) {
			String decimalPattern = "([0-9]*)\\.([0-9]*)";
			boolean match = Pattern.matches(decimalPattern, val);

			if (match) {
				float value1 = Float.valueOf(val);
				value = (int) value1;
			} else {
				value = Integer.parseInt(val);
			}
		}
		return value;
	}

	// combine combo and text attributes selected values
	public Map<String, ArrayList<?>> mergeMappings() {
		Map<String, ArrayList<?>> value = new HashMap<String, ArrayList<?>>();
		value.putAll(getMappingForTextbox());
		value.putAll(getMappingForCombo());
		return value;
	}

	private JPanel EventPanel() {
		eventAttributeCombo.setMaximumSize(new Dimension(300, 30));
		eventAttributeCombo.setPreferredSize(new Dimension(300, 30));

		JLabel label = new JLabel("Events");
		label.setMaximumSize(new Dimension(10, 30));

		JPanel componentPanel = new JPanel();
		componentPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		componentPanel.setLayout(new GridLayout());
		componentPanel.add(label);
		componentPanel.add(eventAttributeCombo);

		JPanel buttonPanel = new JPanel();
		buttonPanel.setSize(new Dimension(50, 30));
		JButton remove = components.createButton("Remove");
		remove.setMaximumSize(new Dimension(10, 30));
		buttonPanel.add(remove);
		//buttonPanel.setVisible(false);
		remove.setVisible(false);

		JPanel eventPanel = new JPanel();
		eventPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		eventPanel.add(componentPanel);
		eventPanel.add(buttonPanel);
		return eventPanel;
	}

	@SuppressWarnings("unchecked")
	public Collection<String> getSelectedEvent() {
		return eventAttributeCombo.getSelectedItems();
	}

	@SuppressWarnings("unchecked")
	public Collection<String> getSelectedAttribute() {
		return attributeCombo.getSelectedItems();
	}

	public String getPanelName() {
		return "Filter";
	}

	public JComponent getComponent() {
		return this;
	}

	public double getHeightInView() {
		return this.getPreferredSize().getHeight();
	}

	public double getWidthInView() {
		return this.getPreferredSize().getWidth();
	}

	public MapImagePanel getImagePanel() {
		return imagePanel;
	}

	public void setScalableComponent(ScalableComponent scalable) {
	}

	public void setParent(ScalableViewPanel viewPanel) {
	}

	public void updated() {
	}

	public void willChangeVisibility(boolean to) {
	}
}
