package org.processmining.plugins.mapvis.visualize;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JPanel;

import org.processmining.plugins.mapvis.visualize.chart.WorkItemChartPanel;
import org.processmining.plugins.mapvis.visualize.workitem.WorkItemDot;

public class MapImageRolloverListener extends MouseMotionAdapter implements MouseListener, MouseWheelListener {

	private MapImagePanel imagePanel;
	private WorkItemDot wDot = null;
	protected short x;
	protected short y;

	public MapImageRolloverListener(MapImagePanel imagePanel) {
		this.imagePanel = imagePanel;
	}

	/*
	 * show Panel and dot data on mouse hover of the dot on the Map
	 * panel(non-Javadoc)
	 */
	public void mouseMoved(MouseEvent e) {
		short x = (short) e.getX();
		short y = (short) e.getY();
		short x_border = (short) imagePanel.getScalablePanel().getVisibleRect().getMaxX();
		short y_border = (short) imagePanel.getScalablePanel().getVisibleRect().getMaxY();

		if (wDot != null) {
			JPanel panel = (JPanel) e.getSource();
			wDot.showInfo(false);
			panel.repaint();
		}
		if (imagePanel.isMergeWithPartialOverlap()) {
			wDot = imagePanel.getMergeWorkItemAt(x, y);
		} else {
			wDot = imagePanel.getWorkItemAt(x, y);
		}

		if (wDot != null) {
			JPanel panel = (JPanel) e.getSource();

			if ((x_border - x < 150) && (y_border - y < 50)) {
				wDot.setInfoOffset((short) -150, (short) -50);
			} else if (x_border - x < 150) {
				wDot.setInfoOffset((short) -150, (short) 0);
			} else if (y_border - y < 50) {
				wDot.setInfoOffset((short) 0, (short) -50);
			} else {
				wDot.setInfoOffset((short) 0, (short) 0);
			}

			wDot.showInfo(true);
			panel.repaint();
		}
	}

	/*
	 * zoom in and out on mouse wheels
	 */
	public void mouseWheelMoved(MouseWheelEvent e) {
		try {
			int notches = e.getWheelRotation();
			double temp = imagePanel.getScale() - (notches * 0.2);
			if (temp > 0.0) {
				imagePanel.setScale(temp);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public void mouseClicked(MouseEvent e) {
		short x = (short) e.getX();
		short y = (short) e.getY();
		wDot = imagePanel.getWorkItemAt(x, y);
		if (wDot != null) {
			ArrayList<String> workDotList = new ArrayList<String>();
			workDotList = wDot.publishLog();
			// Create a sub-log file
			if (e.getClickCount() == 2 && !e.isConsumed()) {
				if (!imagePanel.getParentPane().isIndextation) {
					Map<String, Map<String, String>> workDots = new HashMap<String, Map<String, String>>();
					workDots = wDot.publishTable();
					new WorkItemChartPanel("WorkDot Information", imagePanel, workDots, workDotList, wDot.colorData());
				}
			}
		}
	}

	public void mouseReleased(MouseEvent e) {
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mousePressed(MouseEvent arg0) {
	}
}
