package org.processmining.plugins.mapvis.visualize;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.ui.scalableview.ScalableViewPanel;
import org.processmining.framework.util.ui.scalableview.interaction.PIPInteractionPanel;
import org.processmining.framework.util.ui.scalableview.interaction.ZoomInteractionPanel;
import org.processmining.plugins.mapvis.importdata.visualization.MapVisMap;
import org.processmining.plugins.mapvis.replay.History;
import org.processmining.plugins.mapvis.replay.TimeStampArray;
import org.processmining.plugins.mapvis.visualize.workitem.AbstractWorkDotColorSchema;
import org.processmining.plugins.mapvis.visualize.workitem.AbstractWorkItemDot;
import org.processmining.plugins.mapvis.visualize.workitem.HiddenWorkItemDot;
import org.processmining.plugins.mapvis.visualize.workitem.InvalidWorkItemDot;
import org.processmining.plugins.mapvis.visualize.workitem.WorkItemDot;

public class MapPanel extends JPanel implements AnimationListener {

	private static final long serialVersionUID = -1196292064681907595L;
	private int fadingTimeWindow = 15;
	private MapImagePanel imagePanel;
	private MapVisMap map;
	private MapSidebar sidebar;
	private PIPInteractionPanel pipPanel;
	private final ScalableViewPanel scalablePanel;
	private int counter = 0;
	private boolean isActive = false;
	private boolean fadingOn;
	private boolean eventFilterOn = false;
	private boolean attributeFilterOn = false;
	private Collection<String> eventFilter;
	private Collection<?> attributeFilter;
	public ColourLegendPanel legend;
	private ExportPanel export;
	private AnimationTimer timer;
	private TimeChartPanel colorChart;
	private TimeChartPanel timeChart;
	private History instHistory;
	boolean isBlackNWhite = false;
	protected Map<String, ArrayList<?>> selectedAttributeValues = new HashMap<String, ArrayList<?>>();
	private List<WorkItemDot> dotList = new LinkedList<WorkItemDot>();
	protected boolean isIndextation = false;
	protected boolean getFreezeTime = false;
	protected boolean isRelativeToAttributeOn = false;
	private Map<Coordinate, WorkItemDot> myWorkDots = new HashMap<Coordinate, WorkItemDot>();
	private Map<Coordinate, WorkItemDot> freezeDot = null;
	private ComparativeAnalysis comparative;
	private int hiddenCount, invalidCount;
	private List<AbstractWorkItemDot>[] arrayOfDotList = null;
	protected ParameterPanel paraPanel = null;

	public MapPanel(MapVisMap map, AnimationTimer timer, History instHistory, PluginContext context) {

		this.map = map;
		this.timer = timer;
		this.instHistory = instHistory;

		imagePanel = new MapImagePanel(map, instHistory.getAttributes()[0], instHistory, context);
		sidebar = new MapSidebar(imagePanel);
		legend = new ColourLegendPanel(map, instHistory);
		scalablePanel = new ScalableViewPanel(imagePanel);
		imagePanel.setScalablePanel(scalablePanel);
		pipPanel = new PIPInteractionPanel(scalablePanel);
		export = new ExportPanel(imagePanel);

		comparative = new ComparativeAnalysis(imagePanel, instHistory.getAttributes());
		paraPanel = new ParameterPanel(imagePanel, instHistory.getAttributes());

		scalablePanel.addViewInteractionPanel(pipPanel, SwingConstants.NORTH);
		scalablePanel.addViewInteractionPanel(new ZoomInteractionPanel(scalablePanel, ScalableViewPanel.MAX_ZOOM),
				SwingConstants.WEST);
		scalablePanel.addViewInteractionPanel(paraPanel, SwingConstants.EAST);
		scalablePanel.addViewInteractionPanel(new FilterPanel(imagePanel, instHistory.getAttributes()),
				SwingConstants.EAST);

		timeChart = ValidChartPanel.createValidChartPanel(imagePanel, timer, instHistory);
		scalablePanel.addViewInteractionPanel(timeChart, SwingConstants.SOUTH);
		colorChart = ValidChartPanel.createColorPanel(imagePanel, timer, true, instHistory);
		scalablePanel.addViewInteractionPanel(colorChart, SwingConstants.SOUTH);

		scalablePanel.addViewInteractionPanel(legend, SwingConstants.EAST);
		scalablePanel.addViewInteractionPanel(export, SwingConstants.NORTH);
		scalablePanel.addViewInteractionPanel(comparative, SwingConstants.EAST);
		timer.addListener(timeChart);
		timer.addListener(colorChart);

		this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		this.add(sidebar);
		this.add(scalablePanel);

		imagePanel.setAutoscrolls(true);
		imagePanel.setParentPane(this);
	}

	public void setActive(boolean isActive) {
		this.isActive = isActive;
	}

	public void recomputeLegend() {
		legend.recompute();
		legend.revalidate();
		legend.repaint();
	}

	public void recalc() {
		recalc(TimeStampArray.getInstance().getModelingTime());
	}

	protected void selectWorkItemDotLocation(AbstractWorkItemDot workItem) {
		if (workItem instanceof InvalidWorkItemDot) {
			sidebar.addWorkItemDot((InvalidWorkItemDot) workItem);
		} else if (workItem instanceof HiddenWorkItemDot) {
			sidebar.addWorkItemDot((HiddenWorkItemDot) workItem);
		} else if (workItem instanceof WorkItemDot) {
			if (imagePanel.isMergeWithPartialOverlap()) {
				dotList.add((WorkItemDot) workItem);
			} else {
				createMap((WorkItemDot) workItem);
				if (fadingOn) {
					if (arrayOfDotList != null) {
						((WorkItemDot) workItem).setTransparency((byte) (percentageOfAppearance(arrayOfDotList,
								workItem) * 127));
					}
				}
			}
		}
	}

	private void createMap(WorkItemDot workItem) {
		Coordinate coordinate = new Coordinate(workItem.getX(), workItem.getY());
		if (!myWorkDots.containsKey(coordinate)) {
			myWorkDots.put(coordinate, workItem);
		} else {
			WorkItemDot dot = myWorkDots.get(coordinate);
			WorkItemDot merged = dot.joinTo(workItem, true);
			myWorkDots.put(coordinate, merged);
		}
	}

	protected Map<Coordinate, WorkItemDot> getMaps() {
		return this.myWorkDots;
	}

	private List<WorkItemDot> getWorkItemDotForMap() {
		return dotList;
	}

	public synchronized void recalc(long timestamp) {
		if (isActive && !imagePanel.isPainting) {
			TimeStampArray.getInstance().seekCounter(timestamp);
			counter = TimeStampArray.getInstance().getCounter();

			Boolean showNumber = AbstractWorkDotColorSchema.showNumberPerMap.get(AbstractWorkDotColorSchema
					.getInstance().getMapVisMap());
			if (showNumber == null) {
				showNumber = false;
			}

			// clear data from all panels(map and side bars of map panel)

			if (imagePanel.isMergeWithPartialOverlap()) {
				dotList.clear();
				imagePanel.clearMergeWorkItemDots();
			} else {
				myWorkDots = new HashMap<Coordinate, WorkItemDot>();
				imagePanel.clearWorkItemDots();
			}

			sidebar.clearWorkItemDots();
			if (instHistory.getWorkItemDots(map.getName(), counter) != null) {

				if (fadingOn) {
					arrayOfDotList = disappearingDots(map.getName(), counter, fadingTimeWindow);
				}

				for (AbstractWorkItemDot workItem : instHistory.getWorkItemDots(map.getName(), counter)) {
					// When both event and attribute filters are active
					if (eventFilterOn && attributeFilterOn) {
						FilterEventAttribute filter = new FilterEventAttribute(eventFilter, this);
						boolean eventResult = filter.eventAttributeFilter(workItem);
						if (eventResult) {
							filter.attributeFilter(workItem);
						}
					} else if (eventFilterOn) {
						FilterEventAttribute filter = new FilterEventAttribute(eventFilter, this);
						// When event filter is active
						filter.eventFilter(workItem);

					} else if (attributeFilterOn) {
						// when attribute filter is active
						if (attributeFilter != null) {
							FilterEventAttribute filter = new FilterEventAttribute(eventFilter, this);
							filter.attributeFilter(workItem);
						}
					} else {
						// when none of the filter is active
						selectWorkItemDotLocation(workItem);
					}

				}

			}
		}

		if (imagePanel.isMergeWithPartialOverlap()) {
			List<WorkItemDot> reDotList = new LinkedList<WorkItemDot>();
			reDotList = getWorkItemDotForMap();
			imagePanel.setMergeWorkItemDots(reDotList);
			imagePanel.joinWorkItemDots();
		} else {
			imagePanel.setWorkItemDots(myWorkDots);
		}
		if (isIndextation && getFreezeTime) {
			imagePanel.findDiffBetweenFrame(myWorkDots, freezeDot, timestamp);
			sidebar.findDiffBetweenSideFrame(invalidCount, hiddenCount, timestamp);
		}

		sidebar.revalidate();
		sidebar.paintWorkItemDots();
		imagePanel.revalidate();
		repaintAll();
	}

	public Map<Coordinate, WorkItemDot> getFreezeDot() {
		this.freezeDot = new HashMap<Coordinate, WorkItemDot>();
		this.freezeDot.putAll(myWorkDots);
		return this.freezeDot;
	}

	void setAgeDataForLegend(Map<String, Map<Color, Integer>> colorData) {
		legend.getAgedata(colorData);
	}

	public void getFreezeInvalidCount() {
		invalidCount = sidebar.getFreezeInvalidCount();
	}

	public void getFreezeHiddenCount() {
		hiddenCount = sidebar.getFreezeHiddenCount();
	}

	private float percentageOfAppearance(List<AbstractWorkItemDot>[] arrayOfDotList, AbstractWorkItemDot workItem) {
		int i = 0;
		while (i < arrayOfDotList.length) {
			if (arrayOfDotList[i].contains(workItem))
				return ((float) i) / arrayOfDotList.length;
			i++;
		}
		return 1;
	}

	@SuppressWarnings("unchecked")
	private List<AbstractWorkItemDot>[] disappearingDots(String mapName, int counter, int timeWindow) {
		int arrayLength = Math.min(instHistory.getNumStates(), timeWindow);
		List<AbstractWorkItemDot>[] remainingDots = new List[arrayLength];
		History instance = instHistory;
		LinkedList<AbstractWorkItemDot> listOfDots = new LinkedList<AbstractWorkItemDot>(instance.getWorkItemDots(
				mapName, counter));

		for (int i = 0; i < arrayLength; i++) {
			List<AbstractWorkItemDot> aListOfDots = instHistory.getWorkItemDots(mapName, counter + 1 + i);
			if (aListOfDots == null) {
				remainingDots[i] = new LinkedList<AbstractWorkItemDot>();
				break;
			}
			remainingDots[i] = (List<AbstractWorkItemDot>) listOfDots.clone();
			remainingDots[i].removeAll(aListOfDots);
			listOfDots.retainAll(aListOfDots);
		}
		return remainingDots;
	}

	public void updateModelTime(long modelTime) {
		recalc(modelTime);
	}

	private void repaintAll() {
		pipPanel.updated();
		imagePanel.repaint();
		sidebar.repaint();
	}

	public void toggleFadingOn() {
		this.fadingOn = !fadingOn;
	}

	public void toggleEventFilterOn(boolean value) {
		this.eventFilterOn = value;
	}

	public boolean getEventFilterOn() {
		return this.eventFilterOn;
	}

	public Collection<String> getEventFilterValue() {
		return this.eventFilter;
	}

	public Collection<String> setEventFilterValue(Collection<String> event) {
		return this.eventFilter = event;
	}

	public void toggleAttributeFilterOn(boolean value) {
		this.attributeFilterOn = value;
	}

	public boolean getattributeFilterOn() {
		return this.attributeFilterOn;
	}

	public Collection<?> setAttributeFilterValue(Collection<?> value) {
		return this.attributeFilter = value;
	}

	public Collection<?> getAttributeFilterValue() {
		return this.attributeFilter;
	}

	public Map<String, ArrayList<?>> setAttributeValues(Map<String, ArrayList<?>> attributeValues) {
		return this.selectedAttributeValues = attributeValues;
	}

	public Map<String, ArrayList<?>> getAttributeValues() {
		return this.selectedAttributeValues;
	}

	public int getFadingTimeWindow() {
		return fadingTimeWindow;
	}

	public void setFadingTimeWindow(int fadingTimeWindow) {
		this.fadingTimeWindow = fadingTimeWindow;
	}

	public void recomputeColorChart(boolean showNumbers) {
		colorChart.setLabelPosChart(showNumbers);
		timer.removeListener(colorChart);
		scalablePanel.removeViewInteractionPanel(colorChart);
		colorChart = ValidChartPanel.createColorPanel(imagePanel, timer, showNumbers, instHistory);
		timer.addListener(colorChart);
	}

	public void recomputeValidChart(boolean showNumbers) {
		timeChart.setLabelPosChart(showNumbers);
		timer.removeListener(timeChart);
		scalablePanel.removeViewInteractionPanel(timeChart);
		timeChart = ValidChartPanel.createValidChartPanel(imagePanel, timer, instHistory);
		timer.addListener(timeChart);
	}

	public void addCharts() {
		scalablePanel.addViewInteractionPanel(timeChart, SwingConstants.SOUTH);
		scalablePanel.addViewInteractionPanel(colorChart, SwingConstants.SOUTH);
	}

	public void addColorChart() {
		scalablePanel.addViewInteractionPanel(colorChart, SwingConstants.SOUTH);
	}

	public void toggleBlackNWhiteOn() {
		this.isBlackNWhite = !isBlackNWhite;
	}

	public boolean getBlackWhite() {
		return isBlackNWhite;
	}

	public boolean isIndextation() {
		return isIndextation;
	}

	public void setIndextation(boolean isIndextation) {
		this.isIndextation = isIndextation;
		sidebar.setSidebarIndex(isIndextation);
	}

	public boolean isFreezeTimeIndextation() {
		return getFreezeTime;
	}

	public void setFreezeIndextation(boolean isIndextation) {
		this.getFreezeTime = isIndextation;
		sidebar.setFreezeIndextation(isIndextation);
	}

	public void setRelativeToAttributeOn(boolean isRelative) {
		this.isRelativeToAttributeOn = isRelative;
	}

	public void started() {
	}

	public void stopped() {
	}
}
