package org.processmining.plugins.mapvis.visualize;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.util.TreeSet;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.ListCellRenderer;
import javax.swing.SwingUtilities;

import org.processmining.framework.util.ui.ExpandButton;
import org.processmining.plugins.mapvis.visualize.workitem.AbstractWorkItemDot;
import org.processmining.plugins.mapvis.visualize.workitem.HiddenWorkItemDot;
import org.processmining.plugins.mapvis.visualize.workitem.InvalidWorkItemDot;

import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

@SuppressWarnings("rawtypes")
class ListRenderer extends JLabel implements ListCellRenderer {
	private static final long serialVersionUID = 1L;

	protected boolean isIndexation;
	protected boolean isFreezeIndextation;

	public ListRenderer() {
		setOpaque(false);
	}

	public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected,
			boolean cellHasFocus) {

		if (!isFreezeIndextation) {
			try{
				AbstractWorkItemDot dot = (AbstractWorkItemDot) value;
				Color aColor = dot.getColor();
				setOpaque(true);
				setBackground(aColor);
				double y = 0.2126 * Math.pow(aColor.getRed() / 255F, 2.2) + 0.7151
						* Math.pow(aColor.getGreen() / 255F, 2.2) + 0.0721 * Math.pow(aColor.getBlue() / 255F, 2.2);
				if (y < 0.18)
					setForeground(Color.white);
				else
					setForeground(Color.BLACK);

				setFont(list.getFont());
				setText(dot.getInfo());
			}
			catch(Exception ex){
				
			}
		}

		if (isIndexation & isFreezeIndextation) {
			if (value instanceof Integer) {
				String val = value.toString();
				int count = Integer.parseInt(val);
				Color aColor = null;
				if (count > 0) {
					aColor = Color.BLACK;
					setForeground(Color.white);
				}
				if (count < 0) {
					aColor = Color.YELLOW;
					setForeground(Color.BLACK);
				}
				setBackground(aColor);
				setText(val);
			}
		}
		return this;
	}

}

public class MapSidebar extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -8041446574539590035L;

	private Color colorLeft = new Color(40, 40, 40);
	private Color colorRight = new Color(20, 20, 20);
	private int width = 190;

	private boolean visible = true;
	protected AnimationTimer timer;

	private ExpandButton expandButton;
	private JPanel expandPanel;

	private JPanel contentPanel;
	@SuppressWarnings("rawtypes")
	private JList listHiddenWorkItemDot;
	@SuppressWarnings("rawtypes")
	private JList listInvalidWorkItemDot;
	private JLabel numberInvalidDotLbl = null;
	private JLabel numberHiddenDotLbl = null;

	private TreeSet<InvalidWorkItemDot> invalidWorkItemDots = new TreeSet<InvalidWorkItemDot>();
	private TreeSet<HiddenWorkItemDot> hiddenWorkItemDots = new TreeSet<HiddenWorkItemDot>();

	@SuppressWarnings("rawtypes")
	private DefaultListModel invalidWorkItemModel = new DefaultListModel();
	@SuppressWarnings("rawtypes")
	private DefaultListModel hiddenWorkItemModel = new DefaultListModel();

	protected boolean isIndextation = false;
	protected boolean isFreezeIndextation = false;
	private int diffHiddenCount, diffInvalidCount;
	private ListRenderer hidden = new ListRenderer();
	private ListRenderer invalid = new ListRenderer();

	@SuppressWarnings({ "unchecked", "rawtypes" })
	public MapSidebar(final MapImagePanel imagePanel) {
		this.setOpaque(true);
		contentPanel = new JPanel();
		contentPanel.setOpaque(false);
		contentPanel.setMinimumSize(new Dimension(width, 200));
		contentPanel.setMaximumSize(new Dimension(width, 2000));
		contentPanel.setPreferredSize(new Dimension(width, 1000));
		contentPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		contentPanel.setLayout(new BoxLayout(contentPanel, BoxLayout.Y_AXIS));

		listHiddenWorkItemDot = new JList(hiddenWorkItemModel);
		listHiddenWorkItemDot.setFont(listHiddenWorkItemDot.getFont().deriveFont(12f));
		listHiddenWorkItemDot.setBackground(Color.BLACK);
		listHiddenWorkItemDot.setForeground(new Color(160, 160, 160));
		listHiddenWorkItemDot.setCellRenderer(hidden);

		JScrollPane hiddenScrollPane = new JScrollPane(listHiddenWorkItemDot);
		hiddenScrollPane.getHorizontalScrollBar().setUI(
				new SlickerScrollBarUI(hiddenScrollPane.getHorizontalScrollBar()));
		hiddenScrollPane.getVerticalScrollBar().setUI(new SlickerScrollBarUI(hiddenScrollPane.getVerticalScrollBar()));
		hiddenScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		hiddenScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		hiddenScrollPane.setBorder(BorderFactory.createEmptyBorder());
		hiddenScrollPane.setBackground(Color.BLACK);

		listInvalidWorkItemDot = new JList(invalidWorkItemModel);
		listInvalidWorkItemDot.setFont(listInvalidWorkItemDot.getFont().deriveFont(12f));
		listInvalidWorkItemDot.setBackground(Color.BLACK);
		listInvalidWorkItemDot.setForeground(new Color(160, 160, 160));
		listInvalidWorkItemDot.setCellRenderer(invalid);

		JScrollPane invalidScrollPane = new JScrollPane(listInvalidWorkItemDot);
		invalidScrollPane.getHorizontalScrollBar().setUI(
				new SlickerScrollBarUI(invalidScrollPane.getHorizontalScrollBar()));
		invalidScrollPane.getVerticalScrollBar()
				.setUI(new SlickerScrollBarUI(invalidScrollPane.getVerticalScrollBar()));
		invalidScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		invalidScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		invalidScrollPane.setBorder(BorderFactory.createEmptyBorder());
		invalidScrollPane.setBackground(Color.BLACK);

		JPanel hiddenPnl = new JPanel(new BorderLayout());
		hiddenPnl.setOpaque(false);
		hiddenPnl.add(hiddenScrollPane, BorderLayout.CENTER);
		JLabel label = SlickerFactory.instance().createLabel("Activities with no position:");
		label.setOpaque(false);
		label.setForeground(Color.WHITE);
		hiddenPnl.add(label, BorderLayout.NORTH);
		numberHiddenDotLbl = SlickerFactory.instance().createLabel("");
		numberHiddenDotLbl.setOpaque(false);
		numberHiddenDotLbl.setForeground(Color.WHITE);
		hiddenPnl.add(numberHiddenDotLbl, BorderLayout.SOUTH);

		JPanel invalidPnl = new JPanel(new BorderLayout());
		invalidPnl.add(invalidScrollPane, BorderLayout.CENTER);
		invalidPnl.setOpaque(false);
		label = SlickerFactory.instance().createLabel("Invalid positions:");
		label.setOpaque(false);
		label.setForeground(Color.WHITE);
		invalidPnl.add(label, BorderLayout.NORTH);
		numberInvalidDotLbl = SlickerFactory.instance().createLabel("");
		numberInvalidDotLbl.setOpaque(false);
		numberInvalidDotLbl.setForeground(Color.WHITE);
		invalidPnl.add(numberInvalidDotLbl, BorderLayout.SOUTH);

		JSplitPane textPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT, hiddenPnl, invalidPnl);
		textPanel.setResizeWeight(0.5);
		textPanel.setOpaque(false);
		textPanel.setMinimumSize(new Dimension(width, 400));
		textPanel.setMaximumSize(new Dimension(width, 2000));
		textPanel.setPreferredSize(new Dimension(width, 1000));

		contentPanel.add(textPanel);

		expandButton = new ExpandButton();
		expandButton.setAlignmentX(ExpandButton.LEFT_ALIGNMENT);
		expandButton.setExpanded(true);
		expandButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (expandButton.isExpanded() == true) {
					// fold
					visible = false;
					expandButton.setExpanded(false);
					removeAll();
					add(expandPanel);
					add(Box.createVerticalGlue());
					revalidate();
					repaint();
				} else {
					// expand
					visible = true;
					expandButton.setExpanded(true);
					removeAll();
					add(expandPanel);
					add(contentPanel);
					revalidate();
					repaint();
				}
			}
		});
		expandPanel = new JPanel();
		expandPanel.setMaximumSize(new Dimension(400, 10));
		expandPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
		expandPanel.setOpaque(false);
		expandPanel.setLayout(new BoxLayout(expandPanel, BoxLayout.X_AXIS));
		expandPanel.add(expandButton);
		expandPanel.add(Box.createHorizontalGlue());
		this.setBorder(BorderFactory.createEmptyBorder());
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.add(expandPanel);
		this.add(contentPanel);
	}

	public void clearWorkItemDots() {
		synchronized (invalidWorkItemModel) {
			invalidWorkItemDots.clear();
			hiddenWorkItemDots.clear();
		}
	}

	public void addWorkItemDot(InvalidWorkItemDot workItem) {
		synchronized (invalidWorkItemModel) {
			invalidWorkItemDots.add(workItem);
		}
	}

	public void addWorkItemDot(HiddenWorkItemDot workItem) {
		synchronized (invalidWorkItemModel) {
			hiddenWorkItemDots.add(workItem);
		}
	}

	public void setFreezeIndextation(boolean isIndextation) {
		synchronized (invalidWorkItemModel) {
			this.isFreezeIndextation = isIndextation;
			invalid.isFreezeIndextation = isIndextation;
			hidden.isFreezeIndextation = isIndextation;
		}
	}

	public boolean isFreezeTimeIndextation() {
		synchronized (invalidWorkItemModel) {
			return isFreezeIndextation;
		}
	}

	/*
	 * get work item dot of freeze time frame
	 */
	public void findDiffBetweenSideFrame(int invalid, int hidden, long modellingTime) {
		synchronized (invalidWorkItemModel) {
			diffInvalidCount = invalidWorkItemDots.size() - invalid;
			diffHiddenCount = hiddenWorkItemDots.size() - hidden;
		}
	}

	public int getFreezeInvalidCount() {
		synchronized (invalidWorkItemModel) {
			return invalidWorkItemDots.size();
		}
	}

	public int getFreezeHiddenCount() {
		synchronized (invalidWorkItemModel) {
			return hiddenWorkItemDots.size();
		}
	}

	@SuppressWarnings("unchecked")
	public synchronized void paintWorkItemDots() {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				synchronized (invalidWorkItemModel) {
					invalidWorkItemModel.clear();
					hiddenWorkItemModel.clear();

					if (!isFreezeIndextation) {
						for (Object dot : invalidWorkItemDots)
							invalidWorkItemModel.addElement(dot);
						for (Object dot : hiddenWorkItemDots)
							hiddenWorkItemModel.addElement(dot);
						numberHiddenDotLbl.setText(hiddenWorkItemDots.size() + " activity element(s)");
						numberInvalidDotLbl.setText(invalidWorkItemDots.size() + " activity element(s)");
					}
					if (isIndextation && isFreezeIndextation) {
						invalidWorkItemModel.addElement(diffInvalidCount);
						hiddenWorkItemModel.addElement(diffHiddenCount);
					}

				}
			}
		});

	}

	public void setSidebarIndex(boolean isIndextation) {
		synchronized (invalidWorkItemModel) {
			this.isIndextation = isIndextation;
			invalid.isIndexation = isIndextation;
			hidden.isIndexation = isIndextation;
		}
	}

	protected void paintComponent(Graphics g) {
		Graphics2D g2d = (Graphics2D) g.create();
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2d.setFont(g2d.getFont().deriveFont(9f));
		float width = this.getWidth();
		float height = this.getHeight();
		// fill background
		g2d.setColor(Color.BLACK);
		g2d.fill(new Rectangle2D.Float(0, 0, width + 1, height + 1));
		if (this.visible == true) {
			GradientPaint gradient = new GradientPaint(width - 4, 0, colorLeft, width, 0, colorRight, false);
			g2d.setPaint(gradient);
			g2d.fill(new RoundRectangle2D.Float(-30, 0, width + 30, height, 30f, 30f));
		}

	}
}