package org.processmining.plugins.mapvis.visualize;

import java.awt.Dimension;
import java.awt.GridLayout;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSeparator;
import javax.swing.SwingConstants;

import org.processmining.framework.util.ui.scalableview.ScalableComponent;
import org.processmining.framework.util.ui.scalableview.ScalableViewPanel;
import org.processmining.framework.util.ui.scalableview.interaction.ViewInteractionPanel;
import org.processmining.framework.util.ui.widgets.ProMComboBox;

import com.fluxicon.slickerbox.components.NiceIntegerSlider;
import com.fluxicon.slickerbox.factory.SlickerFactory;

public class ParameterPanel extends JPanel implements ViewInteractionPanel {
	private static final long serialVersionUID = 1L;
	private MapImagePanel imagePanel;
	protected JCheckBox mergeOverlappingCBox = SlickerFactory.instance().createCheckBox(
			"Merge dots even if they partially overlap", false);
	private JCheckBox fadingCBox = SlickerFactory.instance().createCheckBox("Fade out dots when disappearing", false);
	public JCheckBox showDotsNumberCBox = SlickerFactory.instance().createCheckBox("Show the number of merged dots",
			true);
	private JCheckBox BlackNWhiteCBox = SlickerFactory.instance().createCheckBox("Create black and white backgroud",
			false);
	protected JRadioButton stateColoringRBtn = SlickerFactory.instance().createRadioButton(
			"Color according to the state");
	protected JRadioButton ageColoring2RBtn = SlickerFactory.instance().createRadioButton(
			"Color according to the age of activity instances");
	protected JRadioButton propertyColoringRBtn = SlickerFactory.instance().createRadioButton(
			"Color according to the activity properties");

	protected ButtonGroup btnGroup = new ButtonGroup();
	private final ParameterPanelListener listener;
	private NiceIntegerSlider fadingSlider = new NiceIntegerSlider("Number of photographs when starting fading dots",
			10, 500, 15);

	@SuppressWarnings("rawtypes")
	private JComboBox discriminatingAttributeCombo = null;

	@SuppressWarnings({ "rawtypes" })
	public ParameterPanel(MapImagePanel imagePanel, String[] attributes) {
		listener = new ParameterPanelListener(this);
		discriminatingAttributeCombo = new ProMComboBox(attributes);
		this.imagePanel = imagePanel;
		this.setLayout(new GridLayout(11, 1));

		this.add(mergeOverlappingCBox);
		mergeOverlappingCBox.addActionListener(listener);
		mergeOverlappingCBox.setActionCommand(ParameterPanelListener.MERGEDOTCBOX);

		this.add(fadingCBox);
		fadingCBox.addActionListener(listener);
		fadingCBox.setActionCommand(ParameterPanelListener.FADINGCBOX);

		fadingSlider.setFont(discriminatingAttributeCombo.getFont());
		fadingSlider.addChangeListener(listener);
		fadingSlider.setMaximumSize(new Dimension(30, 100));

		this.add(showDotsNumberCBox);
		showDotsNumberCBox.addActionListener(listener);
		showDotsNumberCBox.setActionCommand(ParameterPanelListener.SHOWDOTSNUMBER);

		this.add(BlackNWhiteCBox);
		BlackNWhiteCBox.addActionListener(listener);
		BlackNWhiteCBox.setActionCommand(ParameterPanelListener.BLACKWHITE);

		this.add(fadingSlider);
		this.add(new JSeparator(SwingConstants.HORIZONTAL));

		this.add(stateColoringRBtn);
		stateColoringRBtn.addActionListener(listener);
		stateColoringRBtn.setActionCommand(ParameterPanelListener.STATERADIOBTN);
		stateColoringRBtn.setSelected(true);

		this.add(ageColoring2RBtn);
		ageColoring2RBtn.addActionListener(listener);
		ageColoring2RBtn.setActionCommand(ParameterPanelListener.AGEWRTSINGLERADIOBTN);

		this.add(propertyColoringRBtn);
		propertyColoringRBtn.addActionListener(listener);
		propertyColoringRBtn.setActionCommand(ParameterPanelListener.PROPERTYRADIOBTN);

		JPanel discriminatingPanel = new JPanel();
		discriminatingPanel.add(new JLabel("Discriminating Attribute:"));
		discriminatingPanel.add(discriminatingAttributeCombo);
		discriminatingAttributeCombo.addActionListener(listener);
		discriminatingAttributeCombo.setActionCommand(ParameterPanelListener.DISCRIMINATINGCOMBO);
		discriminatingAttributeCombo.setMaximumSize(new Dimension(300, 30));
		discriminatingAttributeCombo.setPreferredSize(new Dimension(300, 30));
		this.add(discriminatingPanel);

		btnGroup.add(stateColoringRBtn);
		btnGroup.add(ageColoring2RBtn);
		btnGroup.add(propertyColoringRBtn);

	}

	public JComponent getComponent() {
		return this;
	}

	public double getHeightInView() {
		return this.getPreferredSize().getHeight();
	}

	public String getRadioButtonSelect() {
		String radioText = "";
		if (stateColoringRBtn.isSelected()) {
			radioText = "State";
		}
		if (ageColoring2RBtn.isSelected()) {
			radioText = "Age";
		}
		if (propertyColoringRBtn.isSelected()) {
			radioText = "Attribute";
		}
		return radioText;
	}

	public String getPanelName() {
		return "Parameters";
	}

	public MapImagePanel getImagePanel() {
		return imagePanel;
	}

	public int getFadingSliderValue() {
		return fadingSlider.getValue();
	}

	public String getSelectedDiscriminatingAttribute() {
		return (String) discriminatingAttributeCombo.getSelectedItem();
	}

	public double getWidthInView() {
		return this.getPreferredSize().getWidth();
	}

	public void setParent(ScalableViewPanel viewPanel) {
	}

	public void setScalableComponent(ScalableComponent scalable) {

	}

	public void willChangeVisibility(boolean to) {

	}

	public void updated() {

	}

}
