package org.processmining.plugins.mapvis.visualize;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextPane;

import org.processmining.framework.util.ui.ExpandButton;
import org.processmining.plugins.mapvis.replay.TimeStampArray;

import com.fluxicon.slickerbox.components.RoundedPanel;

public class Sidebar extends JPanel implements AnimationListener {

	private static final long serialVersionUID = -8041446574539590035L;
	private int width = 190;
	protected org.processmining.plugins.mapvis.VisData visData;
	protected AnimationTimer timer;
	private ExpandButton expandButton;
	private JPanel expandPanel;
	private JPanel contentPanel;
	private JTextPane content;
	private int counter=-1;

	public Sidebar(org.processmining.plugins.mapvis.VisData visData, AnimationTimer timer) {
		contentPanel = new JPanel();
		contentPanel.setOpaque(false);
		contentPanel.setMinimumSize(new Dimension(width, 200));
		contentPanel.setMaximumSize(new Dimension(width, 2000));
		contentPanel.setPreferredSize(new Dimension(width, 1000));
		this.visData = visData;
		this.timer = timer;
		timer.addListener(this);
		this.setOpaque(true);
		contentPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		contentPanel.setLayout(new BoxLayout(contentPanel, BoxLayout.Y_AXIS));
		
		RoundedPanel caseView = new RoundedPanel(15, 5, 0);
		caseView.setAlignmentX(JPanel.CENTER_ALIGNMENT);
		caseView.setMinimumSize(new Dimension(170, 100));
		caseView.setMaximumSize(new Dimension(170, 2000));
		caseView.setPreferredSize(new Dimension(170, 800));
		caseView.setBackground(new Color(0, 0, 0));
		caseView.setLayout(new BorderLayout());
		JLabel caseViewLabel = new JLabel("case progress");
		caseViewLabel.setFont(caseViewLabel.getFont().deriveFont(12f));
		caseViewLabel.setBackground((new JTabbedPane()).getBackground());
		caseViewLabel.setForeground(new Color(60, 60, 60));
		caseView.add(caseViewLabel, BorderLayout.NORTH);
		contentPanel.add(Box.createHorizontalStrut(10));
		contentPanel.add(Box.createHorizontalStrut(5));
		
		content = new JTextPane();
		content.setContentType("text/html");
		content.setEditable(false);
		content.setFont(caseViewLabel.getFont().deriveFont(12f));
		content.setBackground(Color.BLACK);
		content.setForeground(new Color(160, 160, 160));
		contentPanel.add(content);
		
		expandButton = new ExpandButton();
		expandButton.setAlignmentX(ExpandButton.LEFT_ALIGNMENT);
		expandButton.setExpanded(true);
		expandButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(expandButton.isExpanded() == true) {
					// fold

					expandButton.setExpanded(false);
					removeAll();
					add(expandPanel);
					add(Box.createVerticalGlue());
					revalidate();
					repaint();
				} else {
					// expand

					expandButton.setExpanded(true);
					removeAll();
					add(expandPanel);
					add(contentPanel);
					revalidate();
					repaint();
				}
			}		
		});
		expandPanel = new JPanel();
		expandPanel.setMaximumSize(new Dimension(200, 10));
		expandPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
		expandPanel.setOpaque(false);
		expandPanel.setLayout(new BoxLayout(expandPanel, BoxLayout.X_AXIS));
		expandPanel.add(expandButton);
		expandPanel.add(Box.createHorizontalGlue());
		this.setBorder(BorderFactory.createEmptyBorder());
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.add(expandPanel);
		this.add(contentPanel);
	}


	public void started() {

	}


	public void stopped() {

	}


	public void updateModelTime(long modelTime) {
		int newCounter = TimeStampArray.getInstance().getCounter();
		if (counter!=newCounter)
		{
			counter=newCounter;
			repaint();
		}	
			
	}

}