package org.processmining.plugins.mapvis.visualize;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.RoundRectangle2D;
import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.swing.JComponent;

public class TimeDisplay extends JComponent implements AnimationListener {

	/**
	 * 
	 */
	private static final long serialVersionUID = 6645442210867977377L;

	private static final String[] DAY = new String[] { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
	private static final String[] MONTH = new String[] { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep","Oct", "Nov", "Dec" };

	private static Color colorActive = new Color(240, 0, 0);
	protected GregorianCalendar calendar;

	private int cipherWidth = 15;
	private int cipherHeight = 30;
	private int segmentWidth = 3;
	private int border = 7;

	public TimeDisplay() {
		this.setDoubleBuffered(true);
		this.calendar = null;
		int width = (6 * cipherWidth) + (9 * segmentWidth) + (2 * border) + 60;
		int height = cipherHeight + 3 * border;
	
		Dimension dim = new Dimension(width, height);
		this.setMinimumSize(dim);
		this.setMaximumSize(dim);
		this.setPreferredSize(dim);
		this.setSize(dim);
		this.setOpaque(false);
	}

	

	/*
	 * (non-Javadoc)
	 * 
	 * @seeorg.processmining.mining.fuzzymining.vis.anim.AnimationListener#
	 * updateModelTime(long)
	 */
	public void updateModelTime(long modelTime) {
		if (modelTime > 0) {
			if (this.calendar == null) {
				this.calendar = new GregorianCalendar();
			}
			this.calendar.setTimeInMillis(modelTime);
			if (this.isDisplayable()) {
				repaint();
			}
		}
	}

	@Override
	protected void paintComponent(Graphics g) {
		int hourA = 0, hourB = 0, minuteA = 0, minuteB = 0, secondA = 0, secondB = 0;
		int width = this.getWidth();
		int height = this.getHeight();
		Graphics2D g2d = (Graphics2D) g;
		RoundRectangle2D.Float clip = new RoundRectangle2D.Float(0f, 0f, width, height, 2 * border, 2 * border);
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		// draw background
		g2d.setColor(new Color(0, 0, 0));
		g2d.fill(clip);
		g2d.setClip(clip);

	
		String day = "Ready";
		String date = "";

		String time = "";
		if (this.calendar != null) {
		int hour = this.calendar.get(Calendar.HOUR_OF_DAY);
		hourA = hour / 10;
		hourB = hour % 10;
		int minute = this.calendar.get(Calendar.MINUTE);
		minuteA = minute / 10;
		minuteB = minute % 10;
		int second = this.calendar.get(Calendar.SECOND);
		secondA = second / 10;
		secondB = second % 10;
		time= String.valueOf(hourA) + "" + String.valueOf(hourB) + ":" +String.valueOf(minuteA) +String.valueOf(minuteB) + ":" +String.valueOf(secondA)+String.valueOf(secondB);

			day = DAY[this.calendar.get(Calendar.DAY_OF_WEEK) - 1] + " " + this.calendar.get(Calendar.DAY_OF_MONTH);
			int year = this.calendar.get(Calendar.YEAR) % 100;
			String yearStr = Integer.toString(year);
			if (year < 10) {
				yearStr = "0" + yearStr;
			}
			date = MONTH[this.calendar.get(Calendar.MONTH)] + " '" + yearStr;
		}

		// draw hours
		int x = border + 5;

		// draw day
		g2d.setColor(colorActive);
		g2d.setFont(g2d.getFont().deriveFont(25f));
		g2d.drawString(day, x, border + 16);
		
		//draw date
		x = x + cipherWidth + 24 * segmentWidth+2;
		g2d.drawString(date, x, border + 16 );

		// draw time
		x =  20 * segmentWidth;

		g2d.setFont(g2d.getFont().deriveFont(18f));
		g2d.drawString(time, x, border + 38);
	}

	public void started() {
		// TODO Auto-generated method stub

	}

	public void stopped() {
		// TODO Auto-generated method stub
	}
}
