package org.processmining.plugins.mapvis.visualize;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.jfree.data.time.Minute;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.processmining.plugins.mapvis.importdata.visualization.MapVisMap;
import org.processmining.plugins.mapvis.replay.History;
import org.processmining.plugins.mapvis.replay.TimeStampArray;
import org.processmining.plugins.mapvis.visualize.workitem.AbstractWorkDotColorSchema;
import org.processmining.plugins.mapvis.visualize.workitem.AbstractWorkItemDot;
import org.processmining.plugins.mapvis.visualize.workitem.HiddenWorkItemDot;
import org.processmining.plugins.mapvis.visualize.workitem.InvalidWorkItemDot;
import org.processmining.plugins.mapvis.visualize.workitem.WorkItemDot;

public class ValidChartPanel {

	public static TimeChartPanel createColorPanel(MapImagePanel imagePanel, AnimationTimer timer, boolean showNumbers,
			History instHistory) {
		MapVisMap map = imagePanel.getMapVisMap();
		HashMap<Color, TimeSeries> colorSeries = new HashMap<Color, TimeSeries>();
		HashMap<Color, Integer> numPerColor = new HashMap<Color, Integer>();

		int numState = instHistory.getNumStates();
		long length = (TimeStampArray.getInstance().getEnd() - TimeStampArray.getInstance().getStart()) / 100;
		long lastPlot = 0;
		for (int counter = 0; counter < numState; counter++) {
			if (TimeStampArray.getInstance().getTimeForCounter(counter).getTime() - lastPlot <= length)
				continue;
			lastPlot = TimeStampArray.getInstance().getTimeForCounter(counter).getTime();

			boolean eventFilterOn = false;
			boolean attributeFilterOn = false;
			Collection<String> eventFilter = null;
			Collection<?> attributeFilter = null;
			Map<String, ArrayList<?>> selectedAttributeValues = new HashMap<String, ArrayList<?>>();

			if (imagePanel.getParentPane() != null) {
				eventFilterOn = imagePanel.getParentPane().getEventFilterOn();
				eventFilter = imagePanel.getParentPane().getEventFilterValue();

				attributeFilterOn = imagePanel.getParentPane().getattributeFilterOn();
				attributeFilter = imagePanel.getParentPane().getAttributeFilterValue();
				selectedAttributeValues.putAll(imagePanel.getParentPane().getAttributeValues());
			}

			List<AbstractWorkItemDot> dotList = instHistory.getWorkItemDots(map.getName(), counter);
			{
				if (dotList != null) {
					for (AbstractWorkItemDot dot : dotList) {
						Color col = null;
						// When both event and attribute filters are active
						if (eventFilterOn && attributeFilterOn) {
							ChartEventAttribute filter = new ChartEventAttribute(eventFilter, attributeFilter,
									selectedAttributeValues);
							boolean eventResult = filter.eventAttributeFilter(dot);
							if (eventResult) {
								boolean attributeResult = filter.attributeFilter(dot, counter);

								if (attributeResult) {
									col = dot.getColor(TimeStampArray.getInstance().getTimeForCounter(counter)
											.getTime());
									col = new Color((col.getRed() - col.getRed() % 85),
											(col.getGreen() - col.getGreen() % 85),
											(col.getBlue() - col.getBlue() % 85));
									Integer x = numPerColor.get(col);
									if (x == null)
										x = 1;
									else
										x++;
									numPerColor.put(col, x);

								}
							}
						} else if (eventFilterOn) {
							if (eventFilter != null) {
								ChartEventAttribute filter = new ChartEventAttribute(eventFilter, attributeFilter,
										selectedAttributeValues);
								// When event filter is active
								boolean result;
								result = filter.eventFilter(dot, counter);
								if (result) {
									col = dot.getColor(TimeStampArray.getInstance().getTimeForCounter(counter)
											.getTime());
									col = new Color((col.getRed() - col.getRed() % 85),
											(col.getGreen() - col.getGreen() % 85),
											(col.getBlue() - col.getBlue() % 85));
									Integer x = numPerColor.get(col);
									if (x == null)
										x = 1;
									else
										x++;
									numPerColor.put(col, x);
								}
							}
						} else if (attributeFilterOn) {
							// when attribute filter is active
							if (attributeFilter != null) {
								ChartEventAttribute filter = new ChartEventAttribute(eventFilter, attributeFilter,
										selectedAttributeValues);
								boolean attributeResult = filter.attributeFilter(dot, counter);
								if (attributeResult) {
									col = dot.getColor(TimeStampArray.getInstance().getTimeForCounter(counter)
											.getTime());
									col = new Color((col.getRed() - col.getRed() % 85),
											(col.getGreen() - col.getGreen() % 85),
											(col.getBlue() - col.getBlue() % 85));
									Integer x = numPerColor.get(col);
									if (x == null)
										x = 1;
									else
										x++;
									numPerColor.put(col, x);
								}
							}
						} else {

							col = dot.getColor(TimeStampArray.getInstance().getTimeForCounter(counter).getTime());
							col = new Color((col.getRed() - col.getRed() % 85), (col.getGreen() - col.getGreen() % 85),
									(col.getBlue() - col.getBlue() % 85));
							Integer x = numPerColor.get(col);
							if (x == null)
								x = 1;
							else
								x++;
							numPerColor.put(col, x);
						}
					}
				}
			}
			Minute xValue = new Minute(TimeStampArray.getInstance().getTimeForCounter(counter));
			for (Entry<Color, Integer> entry : numPerColor.entrySet()) {
				TimeSeries oneColorSeries = colorSeries.get(entry.getKey());
				if (oneColorSeries == null) {
					oneColorSeries = new TimeSeries(entry.getKey().getRGB());
					colorSeries.put(entry.getKey(), oneColorSeries);
				}
				oneColorSeries.addOrUpdate(xValue, entry.getValue());
			}
			numPerColor.clear();
		}
		TimeSeriesCollection dataset = new TimeSeriesCollection();
		Color[] colorset = new Color[colorSeries.keySet().size()];
		int i = 0;
		for (Entry<Color, TimeSeries> entry : colorSeries.entrySet()) {
			colorset[i++] = entry.getKey();
			dataset.addSeries(entry.getValue());
		}
		return new TimeChartPanel("Colour Trend", dataset, timer, colorset, false, showNumbers);
	}

	public static TimeChartPanel createValidChartPanel(MapImagePanel imagePanel, AnimationTimer timer,
			History instHistory) {
		MapVisMap map = imagePanel.getMapVisMap();
		Boolean showNumbers = AbstractWorkDotColorSchema.showNumberPerMap.get(map);
		if (showNumbers == null)
			showNumbers = true;
		TimeSeries validPosition = new TimeSeries("Valid Position");
		TimeSeries invalidPosition = new TimeSeries("Invalid Position");
		TimeSeries negativePosition = new TimeSeries("No Position");
		
		int numState = instHistory.getNumStates();
		long length = (TimeStampArray.getInstance().getEnd() - TimeStampArray.getInstance().getStart()) / 100;
		long lastPlot = 0;
		
		boolean eventFilterOn = false;
		boolean attributeFilterOn = false;
		Collection<String> eventFilter = null;
		Collection<?> attributeFilter = null;
		Map<String, ArrayList<?>> selectedAttributeValues = new HashMap<String, ArrayList<?>>();
		if (imagePanel.getParentPane() != null) {
			eventFilterOn = imagePanel.getParentPane().getEventFilterOn();
			attributeFilterOn = imagePanel.getParentPane().getattributeFilterOn();
			eventFilter = imagePanel.getParentPane().getEventFilterValue();
			attributeFilter = imagePanel.getParentPane().getAttributeFilterValue();
			selectedAttributeValues.putAll(imagePanel.getParentPane().getAttributeValues());
		}

		
		for (int counter = 0; counter < numState; counter++) {		
			if (TimeStampArray.getInstance().getTimeForCounter(counter).getTime() - lastPlot <= length)
				continue;
			lastPlot = TimeStampArray.getInstance().getTimeForCounter(counter).getTime();
			
			int valid = 0;
			int invalid = 0;
			int hidden = 0;
	
			List<AbstractWorkItemDot> dotList = instHistory.getWorkItemDots(map.getName(), counter);
			if (dotList != null) {
				for (AbstractWorkItemDot dot : dotList) {

					if (eventFilterOn && attributeFilterOn) {
						ChartEventAttribute filter = new ChartEventAttribute(eventFilter, attributeFilter,
								selectedAttributeValues);
						boolean eventResult = filter.eventAttributeFilter(dot);
						if (eventResult) {
							boolean attributeResult = filter.attributeFilter(dot, counter);

							if (attributeResult) {
								if (dot instanceof HiddenWorkItemDot)
									++hidden;
								else if (dot instanceof WorkItemDot)
									++valid;
								else if (dot instanceof InvalidWorkItemDot)
									++invalid;
							}
						}
					} else if (eventFilterOn) {
						if (eventFilter != null) {
							ChartEventAttribute filter = new ChartEventAttribute(eventFilter, attributeFilter,
									selectedAttributeValues);
							// When event filter is active
							boolean result;
							result = filter.eventFilter(dot, counter);
							if (result) {
							if (dot instanceof HiddenWorkItemDot)
									++hidden;
								else if (dot instanceof WorkItemDot) {
									++valid;
								} else if (dot instanceof InvalidWorkItemDot)
									++invalid;
							}
						}
					} else if (attributeFilterOn) {
						// when attribute filter is active
						if (attributeFilter != null) {
							ChartEventAttribute filter = new ChartEventAttribute(eventFilter, attributeFilter,
									selectedAttributeValues);
							boolean attributeResult = filter.attributeFilter(dot, counter);

							if (attributeResult) {
								if (dot instanceof HiddenWorkItemDot)
									++hidden;
								else if (dot instanceof WorkItemDot)
									++valid;
								else if (dot instanceof InvalidWorkItemDot)
									++invalid;
							}
						}
					} else {
						if (dot instanceof HiddenWorkItemDot)
							++hidden;
						else if (dot instanceof WorkItemDot)
							++valid;
						else if (dot instanceof InvalidWorkItemDot)
							++invalid;
					}

				}

				Date date = TimeStampArray.getInstance().getTimeForCounter(counter);
				validPosition.addOrUpdate(new Minute(date), valid);
				invalidPosition.addOrUpdate(new Minute(date),invalid);
				negativePosition.addOrUpdate(new Minute(date),hidden);
			}
		}
		TimeSeriesCollection dataset = new TimeSeriesCollection();

		dataset.addSeries(validPosition);
		dataset.addSeries(invalidPosition);
		dataset.addSeries(negativePosition);
		return new TimeChartPanel("Pos. Trend", dataset, timer, new Color[] { Color.BLUE, Color.RED, Color.GREEN },
				true, showNumbers);

	}

}
