package org.processmining.plugins.mapvis.visualize.chart;

import java.awt.Color;
import java.awt.Font;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.chart.renderer.category.StandardBarPainter;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;

public class BarChartWorkItem {

	private Map<String, Map<Color, Integer>> colorData = new TreeMap<String, Map<Color, Integer>>();
	private static Color[] barColor;

	public BarChartWorkItem(Map<String, Map<Color, Integer>> colorData) {
		this.colorData = colorData;
	}

	protected JFreeChart createBar() {
		return createBarChart(createBarDataset());
	}

	/**
	 * Creates a bar chart.
	 * 
	 * @param dataset
	 * 
	 * @return The chart.
	 */
	protected JFreeChart createBarChart(final CategoryDataset dataset) {
		JFreeChart chart = ChartFactory.createBarChart("", // chart title
				"Attribute", // domain axis label
				"Dots", // range axis label
				dataset, // data
				PlotOrientation.HORIZONTAL, // orientation
				true, // include legend
				false, // tool tip
				false // URLs?
				);

		chart.setBackgroundPaint(Color.white);
		// get a reference to the plot for further customisation
		final CategoryPlot plot = chart.getCategoryPlot();
		plot.setNoDataMessage("NO DATA!");

		plot.setBackgroundPaint(Color.WHITE);
		plot.setBackgroundAlpha(0.2f);

		// set the range axis to display integers only
		final NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();
		rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());

		final CategoryAxis domainAxis = plot.getDomainAxis();
		domainAxis.setCategoryLabelPositions(CategoryLabelPositions.createUpRotationLabelPositions(Math.PI / 6.0));

		BarRenderer renderer = new BarRenderer();
		for (int i = 0; i < barColor.length; i++) {
			renderer.setSeriesPaint(i, barColor[i]);
		}

		renderer.setItemMargin(-2);
		renderer.setMaximumBarWidth(.1);

		renderer.setBaseSeriesVisible(true);
		renderer.setBaseItemLabelFont(new Font("SansSerif", Font.PLAIN, 12));
		plot.setRenderer(renderer);

		//set  bar chart colour
		((BarRenderer) plot.getRenderer()).setBarPainter(new StandardBarPainter());
		chart.setNotify(true);
		return chart;
	}

	/**
	 * Creates a data set.
	 * 
	 * @return bar data set.
	 */
	protected CategoryDataset createBarDataset() {
		barColor = new Color[colorData.size()];
		final DefaultCategoryDataset dataset = new DefaultCategoryDataset();

		int i = 0;
		Map<Color, Integer> colvalue;
		for (Entry<String, Map<Color, Integer>> col : colorData.entrySet()) {
			colvalue = new HashMap<Color, Integer>();
			colvalue.putAll(col.getValue());

			for (Entry<Color, Integer> colInt : colvalue.entrySet()) {
				int j = i++;
				barColor[j] = colInt.getKey();
				dataset.addValue(colInt.getValue(), col.getKey() ,col.getKey());
				break;
			}

		}
		return dataset;
	}
}
