package org.processmining.plugins.mapvis.visualize.chart;

import java.awt.Color;
import java.awt.Font;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.labels.StandardPieSectionLabelGenerator;
import org.jfree.chart.plot.PiePlot;
import org.jfree.data.general.DefaultPieDataset;
import org.jfree.data.general.PieDataset;

public class PieChartWorkItem {
	private static Map<String, Map<Color, Integer>> colorData = new TreeMap<String, Map<Color, Integer>>();
	private static String[] pieKeyvalues = null;
	private static Color[] colors;

	public PieChartWorkItem( Map<String, Map<Color, Integer>> colorData) {
		PieChartWorkItem.colorData = colorData;
	}

	protected JFreeChart createPie() {
		return createPieChart(createPieDataset());
	}

	/**
	 * Creates a chart.
	 * 
	 * @param dataset
	 *            the data set.
	 * 
	 * @return A chart.
	 */
	protected static JFreeChart createPieChart(PieDataset dataset) {
		JFreeChart chart = ChartFactory.createPieChart("", // chart title
				dataset, // data
				true, // include legend
				true, false);

		
	
		final PiePlot plot = (PiePlot) chart.getPlot();
		
		for (int i = 0; i < pieKeyvalues.length; i++) {
			plot.setSectionPaint(pieKeyvalues[i], colors[i]);
		}

		plot.setLabelFont(new Font("SansSerif", Font.PLAIN, 12));

		//To display plus the percentage format label
		StandardPieSectionLabelGenerator generator = new StandardPieSectionLabelGenerator(("{2} : {0}"),
				NumberFormat.getNumberInstance(), new DecimalFormat("0%"));
		plot.setLabelGenerator(generator);
		//Set the line colour
		plot.setLabelLinkPaint(new Color(0, 180, 205));
		plot.setNoDataMessage("No data available");
		plot.setCircular(true);
		plot.setLabelGap(0.01);

		plot.setBackgroundPaint(Color.WHITE);
		plot.setBackgroundAlpha(0.2f);
		chart.setNotify(true);
		return chart;

	}

	/**
	 * Creates pie graph data set.
	 * 
	 * @return pie data set.
	 */
	protected static PieDataset createPieDataset() {
		DefaultPieDataset dataset = new DefaultPieDataset();
		pieKeyvalues = new String[colorData.size()];
		colors = new Color[colorData.size()];

		int i = 0;
		Map<Color, Integer> colvalue = new HashMap<Color, Integer>();
		for (Entry<String, Map<Color, Integer>> col : colorData.entrySet()) {
			colvalue.clear();
			colvalue.putAll(col.getValue());

			for (Entry<Color, Integer> colInt : colvalue.entrySet()) {
				dataset.setValue(col.getKey(), new Double(colInt.getValue()));
				int j = i++;
				pieKeyvalues[j] = col.getKey();
				colors[j] = colInt.getKey();
			}
		}
		return dataset;
	}

}
