package org.processmining.plugins.mapvis.visualize.chart;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.processmining.plugins.mapvis.replay.TimeStampArray;
import org.processmining.plugins.mapvis.visualize.MapImagePanel;

import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * @author ngupta show workItem dot data in more detail as a separate window
 */

public class WorkItemChartPanel extends JFrame implements ActionListener {

	private static final long serialVersionUID = 1L;
	private JButton exportButton, exportPieImage;
	private MapImagePanel imagePanel;
	private ArrayList<String> workDotList = new ArrayList<String>();
	private ChartPanel chartPanel;
	private static Map<String, Map<Color, Integer>> colorData = new TreeMap<String, Map<Color, Integer>>();
	private JFileChooser chooser;
	private static JFreeChart chart;
	private JPanel chartPnl = new JPanel(new BorderLayout());
	private JRadioButton pieRBtn = SlickerFactory.instance().createRadioButton("Pie chart");
	private JRadioButton barRBtn = SlickerFactory.instance().createRadioButton("Bar chart");
	private ButtonGroup btnGroup = new ButtonGroup();
	private JPanel mainPnl = new JPanel(new BorderLayout());
	private JPanel Pnl = new JPanel(new GridLayout());

	private BarChartWorkItem barChart;
	private PieChartWorkItem pieChart;
	

	public WorkItemChartPanel(String panelName, MapImagePanel imagePanel, Map<String, Map<String, String>> list,
			ArrayList<String> workItemDot, Map<String, Map<Color, Integer>> colorData) {
		this.imagePanel = imagePanel;
		WorkItemChartPanel.colorData = colorData;
		workDotList.addAll(workItemDot);

		initComponents(list);
	}

	private void initComponents(Map<String, Map<String, String>> list) {
		JFrame frame = new JFrame();
		frame.setLayout(new BorderLayout());

		JPanel tablePnl = new JPanel(new BorderLayout());
		JPanel btnPnl = new JPanel(new FlowLayout(FlowLayout.TRAILING));

		final JTable table = new JTable(toTableModel(list));
		table.getTableHeader().setReorderingAllowed(false);
		table.setAutoCreateRowSorter(true);
		JScrollPane scrollPane = new JScrollPane(table);

		Date date = new Date(TimeStampArray.getInstance().getModelingTime());
		SimpleDateFormat format = new SimpleDateFormat("dd MMM yyyy   HH:mm:ss");

		tablePnl.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Work Dot Information ", TitledBorder.CENTER, TitledBorder.TOP));
		tablePnl.add(table.getTableHeader(), BorderLayout.NORTH);
		tablePnl.add(scrollPane, BorderLayout.CENTER);

		SlickerFactory slickerFactory = SlickerFactory.instance();
		exportButton = slickerFactory.createButton("Export Sub-log");
		exportButton.addActionListener(this);

		exportPieImage = slickerFactory.createButton("Save Image");
		exportPieImage.addActionListener(this);
	
		btnPnl.add(exportButton);
		btnPnl.add(exportPieImage);

		chartPnl.setSize(500, 500);
		tablePnl.setSize(100, 75);

		JPanel radioPnl = new JPanel(new FlowLayout(FlowLayout.TRAILING));
		radioPnl.add(pieRBtn);
		pieRBtn.addActionListener(this);
		pieRBtn.setSelected(true);

		radioPnl.add(barRBtn);
		barRBtn.addActionListener(this);

		btnGroup.add(pieRBtn);
		btnGroup.add(barRBtn);

		chartPanel = chartPanel();
		chartPnl.add(chartPanel);
		Pnl.add(tablePnl);
		
		Pnl.add(chartPnl);

		mainPnl.add(radioPnl, BorderLayout.PAGE_START);
		mainPnl.add(Pnl, BorderLayout.CENTER);
		mainPnl.add(btnPnl, BorderLayout.SOUTH);

		JScrollPane scrPane = new JScrollPane(mainPnl);
		frame.add(scrPane);
		frame.setTitle("Drill Down" + format.format(date));
		frame.pack();
		frame.setVisible(true);

	}

	private ChartPanel chartPanel() {
		JFreeChart chart = createChart();
		chartPanel = new ChartPanel(chart);
		chartPanel.setMouseZoomable(true, false);
		chartPanel.getChart().setBackgroundPaint(new Color(0, 0, 0, 0)); // transparent black
		chart.getPlot().setBackgroundPaint(Color.BLACK);
		chart.setNotify(true);
		return chartPanel;
	}

	private JFreeChart createChart() {
		if (pieRBtn.isSelected()) {
			pieChart = new PieChartWorkItem(colorData);
			chart = pieChart.createPie();
		}

		if (barRBtn.isSelected()) {
			barChart = new BarChartWorkItem(colorData);
			chart = barChart.createBar();
		}
		return chart;
	}

	/**
	 * fill clicked work item data into a table
	 * 
	 * @param list
	 *            has work item dot data
	 * @return table
	 */
	public static TableModel toTableModel(Map<String, Map<String, String>> list) {
		DefaultTableModel model = new DefaultTableModel(new Object[] { "Trace ID", "Event Name", "Category" }, 0);
		for (Entry<String, Map<String, String>> entry : list.entrySet()) {
			Map<String, String> dotDetail = new HashMap<String, String>();
			dotDetail.putAll(entry.getValue());
			for (Map.Entry<String, String> en : dotDetail.entrySet()) {
				model.addRow(new Object[] { entry.getKey(), en.getKey(), en.getValue() });
			}
		}
		return model;
	}

	public void actionPerformed(ActionEvent e) {
		if ("Export Sub-log".equals(e.getActionCommand())) {
			imagePanel.createSubLog(workDotList);
		}

		if ("Save Image".equals(e.getActionCommand())) {
			chooser = new JFileChooser();
			chooser.setDialogTitle("Select output directory for images");
			int status = chooser.showOpenDialog(null);
			if (status == JFileChooser.APPROVE_OPTION) {
				File selectedFile = chooser.getSelectedFile();
				if (!selectedFile.getName().isEmpty()) {
					String path = selectedFile.getParent() + "/" + selectedFile.getName() + ".png";
					int width = 640; /* Width of the image */
					int height = 480; /* Height of the image */

					File saveChart = new File(path);
					try {
						ChartUtilities.saveChartAsPNG(saveChart, chart, width, height);
						JOptionPane.showMessageDialog(null, "Save chart image", "Message",
								JOptionPane.INFORMATION_MESSAGE);
					} catch (IOException e1) {
						e1.printStackTrace();
					}
				}

			} else if (status == JFileChooser.CANCEL_OPTION) {
				System.out.println("canceled");
			}
		}

		if (pieRBtn.isSelected() || barRBtn.isSelected()) {
			chartPanel.removeAll();
			chartPnl.removeAll();
			chartPnl.revalidate();
			chartPanel.revalidate();
			chartPanel = chartPanel();
			chartPnl.add(chartPanel);
			Pnl.add(chartPnl);

		}
	}
}
