package org.processmining.plugins.mapvis.visualize.workitem;

import java.awt.Color;
import java.io.Serializable;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import org.processmining.plugins.mapvis.replay.ActivityKey;
import org.processmining.plugins.mapvis.replay.ActivityState;
import org.processmining.plugins.mapvis.replay.NameLookUp;
import org.processmining.plugins.mapvis.replay.ProcessIDData;
import org.processmining.plugins.mapvis.replay.TimeStampArray;
import org.processmining.plugins.mapvis.visualize.workitem.AbstractWorkDotColorSchema.ColoringSchema;

public class AbstractWorkItemDot implements Comparable<AbstractWorkItemDot>, Serializable {

	private static final long serialVersionUID = 1L;
	protected byte state;
	private int relativeCreationTime;
	protected long creationTime;
	protected ActivityKey key;

	public AbstractWorkItemDot(byte state, long creationTime, ActivityKey key) {
		this.state = state;
		this.creationTime = creationTime;
		this.relativeCreationTime = (int) ((creationTime - TimeStampArray.getInstance().getStart()) / 1000);
		this.key = key;
	}

	public String getInfo() {
		if (key == null)
			return "";
		else
			return key.toString();
	}

	public Color getColor() {
		return getColor(TimeStampArray.getInstance().getModelingTime());
	}

	/*
	 * Based on selected property on parameter panel, it will decide shown dot
	 * type
	 */
	public String getWorkDotInfo(AbstractWorkItemDot dot, long creationTime) {
		String getState = "";
		AbstractWorkDotColorSchema.ColoringSchema schema = AbstractWorkDotColorSchema.schemaMap
				.get(AbstractWorkDotColorSchema.getInstance().getMapVisMap());
		if (schema == null)
			getState = getWorkDotInfoOnState(dot.getState());
		else {
			switch (schema) {
				case AGEDONACTIVITYMEANTIME :
					getState = getAgeWorkDotInfo(schema, TimeStampArray.getInstance().getModelingTime(), creationTime);
					break;
				case PROPERTYBASED :
					getState = getWorkItemDotInfo(dot.getKey());
					break;
				case STATEBASED :
					getState = getWorkDotInfoOnState(dot.getState());
					break;
				default :
					getState = getWorkDotInfoOnState(dot.getState());
					break;
			}
		}
		return getState;
	}

	/*
	 * This calculate the age of the working dot on the map panel(Current time
	 * on the map panel - Start time of the working dot)
	 */
	private String getAgeWorkDotInfo(ColoringSchema schema, long modellingTime, long creationTime) {
		if (this.key == null)
			return "No Key Info";
		long value = modellingTime - creationTime;
		
		String text = "";
		int cal = 0;

		// for calculating age of the working dot
		long SECOND = 1000;
		long MINUTE = 60 * SECOND;
		long HOUR = 60 * MINUTE;
		long DAY = 24 * HOUR;
		long WEEK = 7 * DAY;
		long YEAR = 52 * WEEK;

		if (cal == 0 && value > YEAR) {
			cal = (int) (value / YEAR);
			text = cal + " years ";
			value %= YEAR;
		}

		if (cal == 0 && value > WEEK) {
			cal = (int) (value / WEEK);
			text = cal + " weeks ";
			value %= WEEK;
		}
		if (cal == 0 && value > DAY) {
			cal = (int) (value / DAY);
			text = cal + " days ";
			value %= DAY;
		}
		if (cal == 0 && value > HOUR) {
			cal = (int) (value / HOUR);
			text = cal + " hours ";
			value %= HOUR;
		}
		if (cal == 0 && value > MINUTE) {
			cal = (int) (value / MINUTE);
			text = cal + " minutes ";
			value %= MINUTE;
		}
		if (cal == 0 && value > SECOND) {
			cal = (int) (value / MINUTE);
			text = cal + " seconds ";
			value %= SECOND;
		}
		
		
	
		return text;
	}

	/*
	 * Based on state of the working dot
	 */
	private String getWorkDotInfoOnState(byte dotState) {
		switch (dotState) {
			case ActivityState.SCHEDULED :
				return "Scheduled";
			case ActivityState.ASSIGNED :
				return "Assigned";
			case ActivityState.STARTED :
				return "Started";
			case ActivityState.SUSPENDED :
				return "Suspended";
			case ActivityState.ABORTED :
				return "Aborted";
			default :
				return "No Key Info";
		}
	}

	/*
	 * Based on discriminating attribute value of the combo box
	 */
	private String getWorkItemDotInfo(ActivityKey key) {
		Map<Integer, String> processIdValue = new TreeMap<Integer, String>();
		String result = "No Key Info";
		if (key == null) {
			return "No Key Info";
		}
		String discriminating = AbstractWorkDotColorSchema.discriminatingAttribute.get(AbstractWorkDotColorSchema
				.getInstance().getMapVisMap());
		processIdValue = ProcessIDData.getInstance().getProcessIdAttribute(discriminating);
		for (Entry<Integer, String> entry : processIdValue.entrySet()) {
			String processIdValueLookUp = NameLookUp.getInstance().getCaseIdbyValue(entry.getKey());
			if (key.getCaseId().equals(processIdValueLookUp)) {
				result = entry.getValue();
			}
		}

		return result;
	}

	public Color getColor(long modellingTime) {
		Color nonTransparentColor;
		ColoringSchema schema = AbstractWorkDotColorSchema.schemaMap.get(AbstractWorkDotColorSchema.getInstance()
				.getMapVisMap());
		if (schema == null)
			nonTransparentColor = getColorAccordingToState();
		else {
			switch (schema) {
			//	case AGEDONALLMEAN :
					
				case AGEDONACTIVITYMEANTIME :
					nonTransparentColor = getColorBasedOnTheAge(schema, modellingTime);
					break;
				case PROPERTYBASED :
					nonTransparentColor = getColorBasedOnProperty();
					break;
				default :
					nonTransparentColor = getColorAccordingToState();
					break;
			}
		}
		return new Color(nonTransparentColor.getRed(), nonTransparentColor.getGreen(), nonTransparentColor.getBlue(),
				150);
	}

	private Color getColorBasedOnTheAge(ColoringSchema schema, long modellingTime) {
		if (key == null)
			return Color.BLUE;
		long meanExecutionTime;
		//if (schema == ColoringSchema.AGEDONALLMEAN) {
		//	meanExecutionTime = ProcessIDData.getInstance().getMeanExecutionTime();
		//} else{
			meanExecutionTime = ProcessIDData.getInstance().getMeanExecutionTimePerActivity(key.getActivityName());
		//}
		
		long creationTime=TimeStampArray.getInstance().getStart() +relativeCreationTime*1000L;

		double value = 0.1 + 0.9 * Math.exp(-Math.log(2) * (modellingTime - creationTime) / meanExecutionTime);
		double green = 0;
		double blue = 0;
		double red = 0;
		double intensity = value;
		red = 3D * ((Math.min(intensity, 0.33333333)));
		intensity = Math.max(intensity - 0.33, 0);
		green = 3D * ((Math.min(intensity, 0.33333333)));
		intensity = Math.max(intensity - 0.33, 0);
		blue = 3D * ((Math.min(intensity, 0.33333333)));

		return new Color((float) red, (float) green, (float) blue);
	}

	private Color getColorBasedOnProperty() {
		if (key == null)
			return Color.BLUE;
		String discriminating = AbstractWorkDotColorSchema.discriminatingAttribute.get(AbstractWorkDotColorSchema
				.getInstance().getMapVisMap());
		Map<Integer, Color> colorMap = ProcessIDData.getInstance().getProcessIdColor(discriminating);
		Color retValue = colorMap.get(key.getValueForCaseId());
		if (retValue == null)

			return Color.BLUE;
		else
			return retValue;
	}

	private Color getColorAccordingToState() {
		switch (state) {
			case ActivityState.SCHEDULED :
				return Color.WHITE;
			case ActivityState.ASSIGNED :
				return Color.CYAN;
			case ActivityState.STARTED :
				return Color.GREEN;
			case ActivityState.SUSPENDED :
				return Color.YELLOW;
			case ActivityState.ABORTED :
				return Color.RED;
			default :
				return Color.BLACK;
		}
	}

	public int compareTo(AbstractWorkItemDot other) {
		if (this instanceof WorkItemDot & other instanceof MultiWorkItemDot) {
			return 1;
		} else if (this instanceof MultiWorkItemDot & other instanceof WorkItemDot) {
			return -1;
		} else {
			int compare = Float.compare(this.getColor().getRGB(), other.getColor().getRGB());
			if (compare != 0) {
				return compare;
			} else {
				if (this.key != null && other.key != null)
					return this.key.compareTo(other.key);
				else
					return (this == other ? 0 : 1);
			}
		}
	}

	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof AbstractWorkItemDot))
			return false;
		if (obj == this)
			return true;
		AbstractWorkItemDot rhs = (AbstractWorkItemDot) obj;
		return this.key.equals(rhs.key) && this.creationTime == rhs.creationTime && this.state == rhs.state;

	}

	@Override
	public int hashCode() {
		return (int) (key.hashCode() + creationTime + 32 * state);
	}

	public ActivityKey getKey() {
		return key;
	}

	public byte getState() {
		return state;
	}

	public long getCreationTime() {
		return creationTime;
	}

}
