package org.processmining.plugins.mapvis.visualize.workitem;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;

import org.processmining.plugins.mapvis.visualize.Coordinate;

/**
 * For visualising increase or decrease of a workItemDot from a freeze time
 * frame
 * 
 * @author ngupta
 * 
 */
public class IndexDot {

	private int worItemDotSize = 0;
	private Coordinate coordinate;
	private int diameter = 25;

	public IndexDot(Coordinate coordinate, int worItemDotSize) {
		this.coordinate = coordinate;
		this.worItemDotSize = worItemDotSize;
	}

	public int getDiameter() {
		if (Math.abs(worItemDotSize) > 1)
			return (int) (diameter * Math.pow(Math.abs(worItemDotSize), 0.4));
		return diameter;
	}

	public void paint(Graphics2D g, float zoom) {
		int x = coordinate.getX();
		int y = coordinate.getY();

		int adjustedDiameter = Math.round(getDiameter() / zoom);
		float angleStep = 360F / Math.abs(worItemDotSize);
		float angle = 0F;

		Color auxColor = null;

		for (int i = 0; i < Math.abs(worItemDotSize); i++) {
			if (worItemDotSize > 0) {
				auxColor = Color.BLACK;
			}
			if (worItemDotSize < 0) {
				auxColor = Color.YELLOW;
			}
			g.setColor(auxColor);
			g.fillArc((x - adjustedDiameter / 2), (y - adjustedDiameter / 2), adjustedDiameter, adjustedDiameter,
					(int) angle, ((int) angleStep + 1));
			angle += angleStep;

			auxColor = Color.BLACK;
			g.setColor(auxColor);
			g.setStroke(new BasicStroke(3 / zoom));
			g.drawArc((x - adjustedDiameter / 2), (y - adjustedDiameter / 2), adjustedDiameter, adjustedDiameter, 0,
					360);
		}

		paintDotSize(g, zoom, x, y);
	}

	private void paintDotSize(Graphics2D g, float zoom, int x, int y) {
		Boolean showNumber = AbstractWorkDotColorSchema.showNumberPerMap.get(AbstractWorkDotColorSchema.getInstance()
				.getMapVisMap());
		if (showNumber == null)
			showNumber = true;
		if (showNumber && Math.abs(worItemDotSize) > 1) {
			g.setFont(new Font("Default", Font.BOLD, Math.round(15 / zoom)));
			g.setColor(Color.GRAY);

			String workItemSize = String.valueOf(Math.abs(worItemDotSize));

			int X = x - Math.round(3 / zoom);
			int Y = y + Math.round(6 / zoom);

			FontMetrics fm = g.getFontMetrics();
			Rectangle2D rect = fm.getStringBounds(workItemSize, g);
			g.fillRect(X, Y - fm.getAscent(), (int) rect.getWidth(), (int) rect.getHeight());

			g.setColor(Color.WHITE);
			g.drawString(workItemSize, x - Math.round(3 / zoom), y + Math.round(6 / zoom));
		}
	}
}
