package org.processmining.plugins.mapvis.visualize.workitem;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.regex.Pattern;

import org.processmining.plugins.mapvis.replay.ActivityKey;
import org.processmining.plugins.mapvis.replay.ProcessIDData;

public class MultiWorkItemDot extends WorkItemDot {

	private static final long serialVersionUID = 1L;
	protected TreeSet<WorkItemDot> workItems = new TreeSet<WorkItemDot>();
	protected short x;
	protected short y;
	private boolean increasedSize;
	private TreeMap<Color, Integer> coloring = null;
	private TreeMap<Color, String> colorvalue = null;
	private Map<WorkItemDot, Integer> workSizeRelative = new HashMap<WorkItemDot, Integer>();

	public MultiWorkItemDot(short x, short y, byte state, boolean increasedSize, long creationTime, ActivityKey key) {
		super(x, y, state, creationTime, key);
		this.increasedSize = increasedSize;
	}

	public MultiWorkItemDot joinTo(WorkItemDot other, boolean increasedSize) {
		if (other instanceof MultiWorkItemDot) {
			this.workItems.addAll(((MultiWorkItemDot) other).workItems);
		} else {
			this.workItems.add(other);
		}
		byte maxTransparency = 0;
		for (WorkItemDot workitem : workItems) {
			maxTransparency = (byte) Math.max(workitem.getTransparency(), maxTransparency);
		}
		this.x = (short) ((this.x + other.x) / 2);
		this.y = (short) ((this.y + other.y) / 2);
		return this;
	}

	public int getDiameter() {
		if (isRelativeToAttributeOn) {
			return (int) (diameter * Math.pow(dotSize, 0.4));
		}
		if (increasedSize)
			return (int) (diameter * Math.pow(workItems.size(), 0.4));
		else
			return super.getDiameter();
	}

	/*
	 * calculate the percentage for each workItemDot
	 */
	@Override
	public void calPercentage(String relativeAttribute, String relativeToAttribute) {
		dotSize = 0;
		Map<WorkItemDot, Float> workSizeForRelative = new HashMap<WorkItemDot, Float>();
		for (WorkItemDot wDot : workItems) {
			int caseID = wDot.getKey().getValueForCaseId();
			String val = ProcessIDData.getInstance().getProcessIdForAttribute(relativeAttribute, caseID);
			if (val != null) {
				int count = 0;
				String decimalPattern = "([0-9]*)\\.([0-9]*)";
				boolean match = Pattern.matches(decimalPattern, val);
				if (match) {
					float i = Float.valueOf(val);
					count = (int) i;
				} else {
					count = Integer.parseInt(val);
				}
				float relative = (float) 1 / count;
				workSizeForRelative.put(wDot, relative);
			}
		}

		if (!workSizeForRelative.isEmpty()) {
			minDotSize = Collections.min(workSizeForRelative.values());
		}
		for (Entry<WorkItemDot, Float> wDot : workSizeForRelative.entrySet()) {
			int newSize = Math.round((wDot.getValue() / minDotSize));
			dotSize = (dotSize + newSize);
			workSizeRelative.put(wDot.getKey(), newSize);
		}
	}

	protected void paintInfo(Graphics2D g2D, float zoom) {
		int i = 0;
		if (workItems.size() < 10) {
			for (WorkItemDot w : workItems) {
				w.x = this.x;
				w.y = this.y;
				w.setInfoOffset(x_offset, y_offset);
				w.paintInfo(g2D, zoom, i);
				i++;
			}
		} else {
			getDotData();
			int size = workItems.size();
			for (Entry<Color, Integer> entry : coloring.entrySet()) {
				if (isRelativeToAttributeOn) {
					entry.setValue(Math.round(entry.getValue() * 100F / dotSize));
				} else {
					entry.setValue(Math.round(entry.getValue() * 100F / size));
				}
			}
			
			coloring.descendingMap();
			paintAggregateInfo(g2D, zoom);
		}
	}

	protected void paintAggregateInfo(Graphics2D g2D, float zoom) {
		int n = 0;
		for (Entry<Color, Integer> entry : coloring.entrySet()) {
			int offset = Math.round((n++) * 17 / zoom);
			int x_offset_adj = Math.round(x_offset / zoom);
			int y_offset_adj = Math.round(y_offset / zoom);
			int adjustedDiameter = Math.round(17 / zoom);

			String proValue = "";

			// get the value based on the colour of the dot
			for (Entry<Color, String> iter : colorvalue.entrySet()) {
				if (iter.getKey().getRGB() == entry.getKey().getRGB()) {
					proValue = iter.getValue();
				}
			}

			String entryValue = null;
			String showvalue = null;

			entryValue = entry.getValue() + " %";
			showvalue = "  (" + proValue + ")";

			g2D.setColor(new Color(235, 235, 235, 255));
			g2D.fillRect(x + adjustedDiameter + x_offset_adj, y + adjustedDiameter + y_offset_adj + offset,
					Math.round((entryValue.length() + showvalue.length()) * 7 / zoom), Math.round(20 / zoom));

			g2D.setColor(entry.getKey());
			g2D.fillOval(x + adjustedDiameter + Math.round(2 / zoom) + x_offset_adj,
					y + 2 * adjustedDiameter - Math.round(11 / zoom) + y_offset_adj + offset, Math.round(7 / zoom),
					Math.round(7 / zoom));

			g2D.setColor(Color.black);
			g2D.setStroke(new BasicStroke(1 / zoom));
			g2D.drawOval(x + adjustedDiameter + Math.round(2 / zoom) + x_offset_adj,
					y + 2 * adjustedDiameter - Math.round(11 / zoom) + y_offset_adj + offset, Math.round(8 / zoom),
					Math.round(8 / zoom));

			int xDraw = x + adjustedDiameter + Math.round(14 / zoom) + x_offset_adj;
			int yDraw = y + 2 * adjustedDiameter - Math.round(2 / zoom) + y_offset_adj + offset;

			g2D.setColor(Color.BLACK);
			g2D.setFont(new Font("Default", Font.BOLD, Math.round(12 / zoom)));
			g2D.drawString(entryValue, xDraw, yDraw);

			int xDotDraw = x + adjustedDiameter + Math.round((entryValue.length() + 1) * 7 / zoom) + x_offset_adj;
			g2D.setFont(new Font("Default", Font.PLAIN, Math.round(12 / zoom)));
			g2D.drawString(showvalue, xDotDraw, yDraw);

		}
	}

	public void paint(Graphics2D g, float zoom, boolean isRelativeToAttributeOn) {
		this.isRelativeToAttributeOn = isRelativeToAttributeOn;
		int nWorkItems = workItems.size();
		int adjustedDiameter = Math.round(getDiameter() / zoom);
		float angleStep = 360F / nWorkItems;
		float angle = 0F;
		for (WorkItemDot w : workItems) {
			Color auxColor = w.getColor();

			if (isRelativeToAttributeOn) {
				if (workSizeRelative.get(w) != null) {
					Float a = new Float(workSizeRelative.get(w));
					Float b = new Float(dotSize);
					angleStep = 360F * a / b;
				}
			}
			if (transparency >= 0)
				auxColor = new Color(auxColor.getRed(), auxColor.getGreen(), auxColor.getBlue(),(int) (transparency * 1.18));
			g.setColor(auxColor);
			
			g.fillArc((x - adjustedDiameter / 2), (y - adjustedDiameter / 2), adjustedDiameter, adjustedDiameter,(int) angle, ((int) angleStep + 1));
			angle += angleStep;

		}
		if (transparency >= 0)
			g.setColor(new Color(0, 0, 0, (int) (transparency * 1.18)));
		else
			g.setColor(Color.BLACK);
		g.setStroke(new BasicStroke(3 / zoom));
		g.drawArc((x - adjustedDiameter / 2), (y - adjustedDiameter / 2), adjustedDiameter, adjustedDiameter, 0, 360);

		Boolean showNumber = AbstractWorkDotColorSchema.showNumberPerMap.get(AbstractWorkDotColorSchema.getInstance()
				.getMapVisMap());
		if (showNumber == null)
			showNumber = true;
		if (showNumber && workItems.size() > 1) {
			g.setFont(new Font("Default", Font.BOLD, Math.round(15 / zoom)));
			g.setColor(Color.GRAY);

			String workItemSize = String.valueOf(workItems.size());

			int X = x - Math.round(3 / zoom);
			int Y = y + Math.round(6 / zoom);

			FontMetrics fm = g.getFontMetrics();
			Rectangle2D rect = fm.getStringBounds(workItemSize, g);
			g.fillRect(X, Y - fm.getAscent(), (int) rect.getWidth(), (int) rect.getHeight());

			g.setColor(Color.WHITE);
			g.drawString(workItemSize, x - Math.round(3 / zoom), y + Math.round(6 / zoom));
		}

		if (showInfo) {
			paintInfo(g, zoom);
		}
	}

	@Override
	public ArrayList<String> publishLog() {
		ArrayList<String> dot = new ArrayList<String>();
		for (WorkItemDot w : workItems) {
			dot.add(w.getKey().getCaseId());
		}
		return dot;
	}

	@Override
	public Map<String, Map<String, String>> publishTable() {
		Map<String, Map<String, String>> dot = new HashMap<String, Map<String, String>>();
		for (WorkItemDot w : workItems) {
			String value = getWorkDotInfo(w, w.getCreationTime());
			String caseID = w.getKey().getCaseId();

			if (dot.containsKey(caseID)) {
				dot.get(caseID).put(w.getKey().getActivityName(), value);
			} else {
				Map<String, String> dotDetail = new HashMap<String, String>();
				dotDetail.put(w.getKey().getActivityName(), value);
				dot.put(caseID, dotDetail);
			}
		}
		return dot;
	}

	private void getDotData() {
		coloring = new TreeMap<Color, Integer>(new MyColor());
		colorvalue = new TreeMap<Color, String>(new MyColor());

		for (WorkItemDot dot : workItems) {
			Color col = dot.getColor();
		//	col = new Color((col.getRed() - col.getRed() % 85), (col.getGreen() - col.getGreen() % 85), (col.getBlue() - col.getBlue() % 85));
			Integer val = coloring.get(col);

			if (val == null) {
				val = 0;
			}
			if (isRelativeToAttributeOn) {
				if (workSizeRelative.get(dot) != null) {
					int colValue = 0;
					if (coloring.containsKey(col)) {
						colValue = coloring.get(col);
					}
					val = colValue + workSizeRelative.get(dot);
					coloring.put(col, val);
				}
			} else {
				coloring.put(col, ++val);
			}
			// for get information about working dot based on the selected property on parameter panel
			String value = getWorkDotInfo(dot, dot.getCreationTime());
			Color c1 = col;
			Color c2 = Color.WHITE;

			if (c1.equals(c2)) {
				colorvalue.put(c1, "Others");
			} else {
				if (!colorvalue.containsKey(col)) {
					colorvalue.put(col, value);
				}
			}
		}
	}

	@Override
	public Map<String, Map<Color, Integer>> colorData() {
		Map<String, Map<Color, Integer>> colorData = new TreeMap<String, Map<Color, Integer>>();
		getDotData();
		for (Entry<Color, String> entry : colorvalue.entrySet()) {
			for (Entry<Color, Integer> col : coloring.entrySet()) {
				if (col.getKey().equals(entry.getKey())) {
					Map<Color, Integer> colo = new HashMap<Color, Integer>();
					colo.put(col.getKey(), col.getValue());
					colorData.put(entry.getValue(), colo);
				}
			}
		}
		return colorData;
	}

	public int compareTo(WorkItemDot other) {
		if (other == this)
			return 0;
		else
			return 1;
	}

	@Override
	public int getMultiWorkDotSize() {
		return workItems.size();
	}

	class MyColor implements Comparator<Color> {
		@Override
		public int compare(Color o1, Color o2) {
			return Integer.compare(o1.getRGB(), o2.getRGB());
		}
	}
}
