package org.processmining.plugins.mapvis.visualize.workitem;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;
import java.util.regex.Pattern;

import org.processmining.plugins.mapvis.replay.ActivityKey;
import org.processmining.plugins.mapvis.replay.ProcessIDData;

public class WorkItemDot extends AbstractWorkItemDot {
	private static final long serialVersionUID = 1L;
	protected short x;
	protected short y;
	protected transient short x_offset = 0;
	protected transient short y_offset = 0;
	protected transient boolean showInfo = false;
	protected transient byte transparency = -1;
	protected transient int diameter = 25;
	protected transient int dotSize;
	protected float minDotSize = 0f;
	protected boolean isRelativeToAttributeOn;

	public WorkItemDot(short x, short y, byte state, long creationTime, ActivityKey key) {
		super(state, creationTime, key);
		assert (x >= 0 && y >= 0);
		this.x = x;
		this.y = y;
	}

	public MultiWorkItemDot joinTo(WorkItemDot other, boolean increasedSize) {
		MultiWorkItemDot merged = new MultiWorkItemDot(this.x, this.y, state, increasedSize, creationTime, key);
		merged.workItems.add(this);
		merged.workItems.add(other);
		merged.x = (short) ((this.x + other.x) / 2);
		merged.y = (short) ((this.y + other.y) / 2);
		merged.setTransparency((byte) Math.max(other.transparency, transparency));
		return merged;
	}

	public int getDiameter() {
		return diameter;
	}

	public void setdotSize(int dotSize) {
		this.dotSize = dotSize;
	}

	public double distance(WorkItemDot other, float zoom) {
		return distance(other.x, other.y, 1) * zoom;
	}

	public double distance(short x, short y, float zoom) {
		return Math.sqrt((this.x - x / zoom) * (this.x - x / zoom) + (this.y - y / zoom) * (this.y - y / zoom));
	}

	public boolean overlapsWith(WorkItemDot other, float zoom, boolean mergeWithPartialOverlap) {
		double distance = distance(other, zoom);
		if (mergeWithPartialOverlap)
			return distance < (getDiameter() + other.getDiameter()) / 2;
		else
			return distance < 5;
	}

	public void showInfo(boolean showInfo) {
		this.showInfo = showInfo;
	}

	public void setInfoOffset(short x, short y) {
		x_offset = x;
		y_offset = y;
	}

	/*
	 * calculate the percentage for each workItemDot
	 */
	public void calPercentage(String relativeAttribute, String relativeToAttribute) {
		dotSize = 0;
		int caseID = this.getKey().getValueForCaseId();
		String atttrVal = ProcessIDData.getInstance().getProcessIdForAttribute(relativeAttribute, caseID);
		if (atttrVal != null) {
			int count = 0;
			String decimalPattern = "([0-9]*)\\.([0-9]*)";
			boolean match = Pattern.matches(decimalPattern, atttrVal);
			if (match) {
				float i = Float.valueOf(atttrVal);
				count = (int) i;
			} else {
				count = Integer.parseInt(atttrVal);
			}
			minDotSize = (float) 1 / count;
		}

		dotSize =  (int) Math.ceil(minDotSize);
		diameter= (int) (diameter * Math.pow(dotSize, 0.4));
	}

	public Map<String, Map<Color, Integer>> colorData() {
		Map<String, Map<Color, Integer>> colorData = new TreeMap<String, Map<Color, Integer>>();
		// get value based on the working dot of the map panel
		String dotInfo = getWorkDotInfo(this, this.getCreationTime());
		Map<Color, Integer> coloring = new HashMap<Color, Integer>();
		Color auxColor = getColor();
		if (transparency >= 0)
			auxColor = new Color(auxColor.getRed(), auxColor.getGreen(), auxColor.getBlue(),
					(int) (transparency * 1.18));

		coloring.put(auxColor, 1);
		colorData.put(dotInfo, coloring);
		return colorData;
	}

	/*
	 * create a sub-log list
	 */
	public ArrayList<String> publishLog() {
		ArrayList<String> dot = new ArrayList<String>();
		dot.add(this.getKey().getCaseId());
		return dot;
	}

	/*
	 * create a list for pop up table
	 */
	public Map<String, Map<String, String>> publishTable() {
		Map<String, Map<String, String>> dot = new HashMap<String, Map<String, String>>();
		Map<String, String> dotDetail = new HashMap<String, String>();
		String value = getWorkDotInfo(this, this.getCreationTime());
		dotDetail.put(this.key.getActivityName(), value);
		dot.put(this.getKey().getCaseId(), dotDetail);
		return dot;
	}

	public void paint(Graphics2D g2D, float zoom, boolean isRelativeToAttributeOn) {
		this.isRelativeToAttributeOn= isRelativeToAttributeOn;
		int adjustedDiameter = Math.round(diameter / zoom);
		Color auxColor = getColor();
		if (transparency >= 0)
			auxColor = new Color(auxColor.getRed(), auxColor.getGreen(), auxColor.getBlue(),
					(int) (transparency * 1.18));
		g2D.setColor(auxColor);
		g2D.fillOval((short) (x - adjustedDiameter / 2), (short) (y - adjustedDiameter / 2), adjustedDiameter,
				adjustedDiameter);

		if (transparency >= 0)
			g2D.setColor(new Color(0, 0, 0, (int) (transparency * 1.18)));
		else
			g2D.setColor(Color.BLACK);
		g2D.setStroke(new BasicStroke(3 / zoom));
		g2D.drawOval((short) (x - adjustedDiameter / 2), (short) (y - adjustedDiameter / 2), adjustedDiameter,
				adjustedDiameter);

		if (showInfo) {
			paintInfo(g2D, zoom);
		}
	}

	protected void paintInfo(Graphics2D g2D, float zoom) {
		paintInfo(g2D, zoom, 0);
	}

	protected void paintInfo(Graphics2D g2D, float zoom, int n) {
		int offset = Math.round(n * 17 / zoom);
		int x_offset_adj = Math.round(x_offset / zoom);
		int y_offset_adj = Math.round(y_offset / zoom);
		int adjustedDiameter = Math.round(17 / zoom);

		// get value based on the working dot of the map panel
		String dotInfo = " (" + getWorkDotInfo(this, this.getCreationTime()) + ")";

		g2D.setColor(new Color(235, 235, 235, 255));
		g2D.fillRect(x + adjustedDiameter + x_offset_adj, y + adjustedDiameter + y_offset_adj + offset,
				Math.round((this.getInfo().length() + dotInfo.length()) * 7 / zoom), Math.round(20 / zoom));

		g2D.setColor(getColor());
		g2D.fillOval(x + adjustedDiameter + Math.round(2 / zoom) + x_offset_adj,
				y + 2 * adjustedDiameter - Math.round(11 / zoom) + y_offset_adj + offset, Math.round(7 / zoom),
				Math.round(7 / zoom));

		g2D.setColor(Color.black);
		g2D.setStroke(new BasicStroke(1 / zoom));
		g2D.drawOval(x + adjustedDiameter + Math.round(2 / zoom) + x_offset_adj,
				y + 2 * adjustedDiameter - Math.round(11 / zoom) + y_offset_adj + offset, Math.round(8 / zoom),
				Math.round(8 / zoom));

		int xDraw = x + adjustedDiameter + Math.round(14 / zoom) + x_offset_adj;
		int yDraw = y + 2 * adjustedDiameter - Math.round(2 / zoom) + y_offset_adj + offset;

		g2D.setColor(Color.BLACK);
		g2D.setFont(new Font("Default", Font.BOLD, Math.round(12 / zoom)));
		g2D.drawString(this.getInfo(), xDraw, yDraw);

		int xDotDraw = x + adjustedDiameter + Math.round(this.getInfo().length() * 7 / zoom) + x_offset_adj;
		g2D.setFont(new Font("Default", Font.PLAIN, Math.round(12 / zoom)));
		g2D.drawString(dotInfo, xDotDraw, yDraw);
	}

	public int compareTo(MultiWorkItemDot other) {
		if (other == this)
			return 0;
		else
			return Float.compare(x * x + y * y, other.x * other.x + other.y * other.y);
	}

	public void setTransparency(byte f) {
		if (f >= -1 && f <= 255)
			this.transparency = f;
		else
			this.transparency = 127;

	}

	public byte getTransparency() {
		return transparency;
	}

	public short getX() {
		return x;
	}

	public short getY() {
		return y;
	}

	public int getMultiWorkDotSize() {
		return 1;
	}

}
